@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009-2011,2014,2015                      Université de Bordeaux
+ * Copyright (C) 2012                                     Inria
+ * Copyright (C) 2010-2013,2015,2017                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -31,7 +32,7 @@
 #include <stdint.h>
 #include <starpu.h>
 
-#define FPRINTF(ofile, fmt, args ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ##args); }} while(0)
+#define FPRINTF(ofile, fmt, ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ## __VA_ARGS__); }} while(0)
 
 /* When the task is done, task->callback_func(task->callback_arg) is called. Any
  * callback function must have the prototype void (*)(void *).
@@ -54,15 +55,15 @@ struct params
 
 void cpu_func(void *buffers[], void *cl_arg)
 {
+	(void)buffers;
 	struct params *params = (struct params *) cl_arg;
 
 	FPRINTF(stdout, "Hello world (params = {%i, %f} )\n", params->i, params->f);
 }
 
-struct starpu_codelet cl = {};
-
-int main(int argc, char **argv)
+int main(void)
 {
+	struct starpu_codelet cl;
 	struct starpu_task *task;
 	struct params params = {1, 2.0f};
 	int ret;
@@ -80,13 +81,15 @@ int main(int argc, char **argv)
 	 * called */
 	task = starpu_task_create();
 
+	starpu_codelet_init(&cl);
 	/* this codelet may only be executed on a CPU, and its cpu
  	 * implementation is function "cpu_func" */
-	cl.where = STARPU_CPU;
 	cl.cpu_funcs[0] = cpu_func;
+	cl.cpu_funcs_name[0] = "cpu_func";
 	/* the codelet does not manipulate any data that is managed
 	 * by our DSM */
 	cl.nbuffers = 0;
+	cl.name="hello";
 
 	/* the task uses codelet "cl" */
 	task->cl = &cl;
