@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2013                                Inria
- * Copyright (C) 2012-2013,2015-2017                      CNRS
- * Copyright (C) 2014-2015                                Université de Bordeaux
+ * Copyright (C) 2011-2013,2017                           Inria
+ * Copyright (C) 2011-2013,2015-2017                      CNRS
+ * Copyright (C) 2012,2014,2015                           Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -38,6 +38,8 @@ multiformat_scal_cpu_func(void *buffers[], void *args)
 	struct point *aos;
 	unsigned int n, i;
 
+	(void)args;
+
 	aos = (struct point *) STARPU_MULTIFORMAT_GET_CPU_PTR(buffers[0]);
 	n = STARPU_MULTIFORMAT_GET_NX(buffers[0]);
 
@@ -276,7 +278,13 @@ main(void)
 #ifdef STARPU_USE_CPU
 	int ret;
 
-	ret = starpu_init(NULL);
+	struct starpu_conf conf;
+	starpu_conf_init(&conf);
+
+	/* this example doesn't support MPI Master-Slave */
+	conf.nmpi_ms = 0;
+
+	ret = starpu_init(&conf);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	ncpu = starpu_cpu_worker_get_count();
