@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010-2011, 2013-2015  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013, 2014, 2016  CNRS
+ * Copyright (C) 2010-2017, 2019                                CNRS
+ * Copyright (C) 2013,2017                                Inria
+ * Copyright (C) 2009-2011,2013-2015                      Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -25,7 +26,7 @@
 #define FPRINTF(ofile, fmt, ...) do { if (!getenv("STARPU_SSILENT")) {fprintf(ofile, fmt, ## __VA_ARGS__); }} while(0)
 
 #ifdef STARPU_USE_OPENCL
-extern void opencl_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args);
+extern void opencl_codelet(void *descr[], void *_args);
 struct starpu_opencl_program opencl_program;
 #endif
 
@@ -117,16 +118,13 @@ int compute(char *file_name, int load_as_file, int with_malloc)
 	return ret;
 }
 
-int main(int argc, char **argv)
+int main(void)
 {
 	int ret = 0;
 	struct starpu_conf conf;
 
 	starpu_conf_init(&conf);
-	conf.ncpus = 0;
 	conf.ncuda = 0;
-	conf.nmic = 0;
-	conf.nscc = 0;
 
         ret = starpu_init(&conf);
 	if (STARPU_UNLIKELY(ret == -ENODEV))
@@ -135,6 +133,12 @@ int main(int argc, char **argv)
 		return 77;
 	}
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
+	if (starpu_opencl_worker_get_count() == 0)
+	{
+                FPRINTF(stderr, "This application requires an OpenCL worker.\n");
+		starpu_shutdown();
+		return 77;
+	}
 
 	ret = compute("examples/incrementer/incrementer_kernels_opencl_kernel.cl", 1, -1);
 	if (ret == 0)
