@@ -1,8 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
+ * Copyright (C) 2011-2013,2015                           Inria
  * Copyright (C) 2009-2017                                Université de Bordeaux
  * Copyright (C) 2010-2013,2015-2017                      CNRS
+ * Copyright (C) 2013                                     Thibaut Lambert
  * Copyright (C) 2010                                     Mehdi Juhoor
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -34,8 +35,9 @@
  *	and construct the DAG
  */
 
-static void callback_turn_spmd_on(void *arg STARPU_ATTRIBUTE_UNUSED)
+static void callback_turn_spmd_on(void *arg)
 {
+	(void)arg;
 	cl22.type = STARPU_SPMD;
 }
 
@@ -50,9 +52,8 @@ static int _cholesky(starpu_data_handle_t dataA, unsigned nblocks)
 
 	unsigned unbound_prio = STARPU_MAX_PRIO == INT_MAX && STARPU_MIN_PRIO == INT_MIN;
 
-	if (bound || bound_lp || bound_mps)
-		starpu_bound_start(bound_deps, 0);
-
+	if (bound_p || bound_lp_p || bound_mps_p)
+		starpu_bound_start(bound_deps_p, 0);
 	starpu_fxt_start_profiling();
 
 	start = starpu_timing_now();
@@ -65,7 +66,7 @@ static int _cholesky(starpu_data_handle_t dataA, unsigned nblocks)
                 starpu_data_handle_t sdatakk = starpu_data_get_sub_data(dataA, 2, k, k);
 
                 ret = starpu_task_insert(&cl11,
-					 STARPU_PRIORITY, noprio ? STARPU_DEFAULT_PRIO : unbound_prio ? (int)(2*nblocks - 2*k) : STARPU_MAX_PRIO,
+					 STARPU_PRIORITY, noprio_p ? STARPU_DEFAULT_PRIO : unbound_prio ? (int)(2*nblocks - 2*k) : STARPU_MAX_PRIO,
 					 STARPU_RW, sdatakk,
 					 STARPU_CALLBACK, (k == 3*nblocks/4)?callback_turn_spmd_on:NULL,
 					 STARPU_FLOPS, (double) FLOPS_SPOTRF(nn),
@@ -79,7 +80,7 @@ static int _cholesky(starpu_data_handle_t dataA, unsigned nblocks)
                         starpu_data_handle_t sdatakj = starpu_data_get_sub_data(dataA, 2, k, j);
 
                         ret = starpu_task_insert(&cl21,
-						 STARPU_PRIORITY, noprio ? STARPU_DEFAULT_PRIO : unbound_prio ? (int)(2*nblocks - 2*k - j) : (j == k+1)?STARPU_MAX_PRIO:STARPU_DEFAULT_PRIO,
+						 STARPU_PRIORITY, noprio_p ? STARPU_DEFAULT_PRIO : unbound_prio ? (int)(2*nblocks - 2*k - j) : (j == k+1)?STARPU_MAX_PRIO:STARPU_DEFAULT_PRIO,
 						 STARPU_R, sdatakk,
 						 STARPU_RW, sdatakj,
 						 STARPU_FLOPS, (double) FLOPS_STRSM(nn, nn),
@@ -101,7 +102,7 @@ static int _cholesky(starpu_data_handle_t dataA, unsigned nblocks)
 					starpu_data_handle_t sdataij = starpu_data_get_sub_data(dataA, 2, i, j);
 
 					ret = starpu_task_insert(&cl22,
-								 STARPU_PRIORITY, noprio ? STARPU_DEFAULT_PRIO : unbound_prio ? (int)(2*nblocks - 2*k - j - i) : ((i == k+1) && (j == k+1))?STARPU_MAX_PRIO:STARPU_DEFAULT_PRIO,
+								 STARPU_PRIORITY, noprio_p ? STARPU_DEFAULT_PRIO : unbound_prio ? (int)(2*nblocks - 2*k - j - i) : ((i == k+1) && (j == k+1))?STARPU_MAX_PRIO:STARPU_DEFAULT_PRIO,
 								 STARPU_R, sdataki,
 								 STARPU_R, sdatakj,
 								 cl22.modes[2], sdataij,
@@ -122,34 +123,36 @@ static int _cholesky(starpu_data_handle_t dataA, unsigned nblocks)
 	end = starpu_timing_now();
 
 	starpu_fxt_stop_profiling();
-	if (bound || bound_lp || bound_mps)
+	if (bound_p || bound_lp_p || bound_mps_p)
 		starpu_bound_stop();
 
 	double timing = end - start;
 
 	double flop = FLOPS_SPOTRF(n);
 
-	if(with_ctxs || with_noctxs || chole1 || chole2)
+	if(with_ctxs_p || with_noctxs_p || chole1_p || chole2_p)
 		update_sched_ctx_timing_results((flop/timing/1000.0f), (timing/1000000.0f));
 	else
 	{
 		PRINTF("# size\tms\tGFlops");
-		if (bound)
+		if (bound_p)
 			PRINTF("\tTms\tTGFlops");
 		PRINTF("\n");
 
 		PRINTF("%lu\t%.0f\t%.1f", n, timing/1000, (flop/timing/1000.0f));
-		if (bound_lp)
+		if (bound_lp_p)
 		{
 			FILE *f = fopen("cholesky.lp", "w");
 			starpu_bound_print_lp(f);
+			fclose(f);
 		}
-		if (bound_mps)
+		if (bound_mps_p)
 		{
 			FILE *f = fopen("cholesky.mps", "w");
 			starpu_bound_print_mps(f);
+			fclose(f);
 		}
-		if (bound)
+		if (bound_p)
 		{
 			double res;
 			starpu_bound_compute(&res, NULL, 0);
@@ -201,9 +204,9 @@ static int cholesky(float *matA, unsigned size, unsigned ld, unsigned nblocks)
 static void execute_cholesky(unsigned size, unsigned nblocks)
 {
 	float *mat = NULL;
-	unsigned i,j;
 
 #ifndef STARPU_SIMGRID
+	unsigned i,j;
 	starpu_malloc_flags((void **)&mat, (size_t)size*size*sizeof(float), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
 	for (i = 0; i < size; i++)
 	{
@@ -213,7 +216,6 @@ static void execute_cholesky(unsigned size, unsigned nblocks)
 			/* mat[j +i*size] = ((i == j)?1.0f*size:0.0f); */
 		}
 	}
-#endif
 
 /* #define PRINT_OUTPUT */
 #ifdef PRINT_OUTPUT
@@ -234,6 +236,7 @@ static void execute_cholesky(unsigned size, unsigned nblocks)
 		}
 		FPRINTF(stdout, "\n");
 	}
+#endif
 #endif
 
 	cholesky(mat, size, size, nblocks);
@@ -259,7 +262,7 @@ static void execute_cholesky(unsigned size, unsigned nblocks)
 	}
 #endif
 
-	if (check)
+	if (check_p)
 	{
 		FPRINTF(stderr, "compute explicit LLt ...\n");
 		for (j = 0; j < size; j++)
@@ -304,7 +307,7 @@ static void execute_cholesky(unsigned size, unsigned nblocks)
 				if (i <= j)
 				{
 	                                float orig = (1.0f/(1.0f+i+j)) + ((i == j)?1.0f*size:0.0f);
-	                                float err = abs(test_mat[j +i*size] - orig) / orig;
+	                                float err = fabsf(test_mat[j +i*size] - orig) / orig;
 	                                if (err > 0.00001)
 					{
 	                                        FPRINTF(stderr, "Error[%u, %u] --> %2.6f != %2.6f (err %2.6f)\n", i, j, test_mat[j +i*size], orig, err);
@@ -341,7 +344,7 @@ int main(int argc, char **argv)
 
 	parse_args(argc, argv);
 
-	if(with_ctxs || with_noctxs || chole1 || chole2)
+	if(with_ctxs_p || with_noctxs_p || chole1_p || chole2_p)
 		parse_args_ctx(argc, argv);
 
 #ifdef STARPU_USE_CUDA
@@ -356,19 +359,19 @@ int main(int argc, char **argv)
 
 	starpu_cublas_init();
 
-	if(with_ctxs)
+	if(with_ctxs_p)
 	{
-		construct_contexts(execute_cholesky);
+		construct_contexts();
 		start_2benchs(execute_cholesky);
 	}
-	else if(with_noctxs)
+	else if(with_noctxs_p)
 		start_2benchs(execute_cholesky);
-	else if(chole1)
+	else if(chole1_p)
 		start_1stbench(execute_cholesky);
-	else if(chole2)
+	else if(chole2_p)
 		start_2ndbench(execute_cholesky);
 	else
-		execute_cholesky(size, nblocks);
+		execute_cholesky(size_p, nblocks_p);
 
 	starpu_cublas_shutdown();
 	starpu_shutdown();
