@@ -1,9 +1,10 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
  * Copyright (C) 2008-2017                                Université de Bordeaux
- * Copyright (C) 2010                                     Mehdi Juhoor
+ * Copyright (C) 2012,2013                                Inria
  * Copyright (C) 2010-2013,2015,2017                      CNRS
+ * Copyright (C) 2013                                     Thibaut Lambert
+ * Copyright (C) 2010                                     Mehdi Juhoor
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -60,7 +61,7 @@ static struct starpu_task * create_task_11(starpu_data_handle_t dataA, unsigned
 	task->handles[0] = starpu_data_get_sub_data(dataA, 2, k, k);
 
 	/* this is an important task */
-	if (!noprio)
+	if (!noprio_p)
 		task->priority = STARPU_MAX_PRIO;
 
 	/* enforce dependencies ... */
@@ -85,7 +86,7 @@ static void create_task_21(starpu_data_handle_t dataA, unsigned k, unsigned j)
 	task->handles[0] = starpu_data_get_sub_data(dataA, 2, k, k);
 	task->handles[1] = starpu_data_get_sub_data(dataA, 2, k, j);
 
-	if (!noprio && (j == k+1))
+	if (!noprio_p && (j == k+1))
 	{
 		task->priority = STARPU_MAX_PRIO;
 	}
@@ -125,7 +126,7 @@ static void create_task_22(starpu_data_handle_t dataA, unsigned k, unsigned i, u
 	task->handles[1] = starpu_data_get_sub_data(dataA, 2, k, j);
 	task->handles[2] = starpu_data_get_sub_data(dataA, 2, i, j);
 
-	if (!noprio && (i == k + 1) && (j == k +1) )
+	if (!noprio_p && (i == k + 1) && (j == k +1) )
 	{
 		task->priority = STARPU_MAX_PRIO;
 	}
@@ -262,7 +263,7 @@ static int initialize_system(int argc, char **argv, float **A, unsigned pinned)
 
 	if (pinned)
 		flags |= STARPU_MALLOC_PINNED;
-	starpu_malloc_flags((void **)A, size*size*sizeof(float), flags);
+	starpu_malloc_flags((void **)A, size_p*size_p*sizeof(float), flags);
 
 	return 0;
 }
@@ -316,17 +317,17 @@ int main(int argc, char **argv)
 	 * */
 
 	float *mat = NULL;
-	int ret = initialize_system(argc, argv, &mat, pinned);
+	int ret = initialize_system(argc, argv, &mat, pinned_p);
 	if (ret) return ret;
 
 #ifndef STARPU_SIMGRID
 	unsigned i,j;
-	for (i = 0; i < size; i++)
+	for (i = 0; i < size_p; i++)
 	{
-		for (j = 0; j < size; j++)
+		for (j = 0; j < size_p; j++)
 		{
-			mat[j +i*size] = (1.0f/(1.0f+i+j)) + ((i == j)?1.0f*size:0.0f);
-			/* mat[j +i*size] = ((i == j)?1.0f*size:0.0f); */
+			mat[j +i*size_p] = (1.0f/(1.0f+i+j)) + ((i == j)?1.0f*size_p:0.0f);
+			/* mat[j +i*size_p] = ((i == j)?1.0f*size_p:0.0f); */
 		}
 	}
 #endif
@@ -335,13 +336,13 @@ int main(int argc, char **argv)
 #ifdef CHECK_OUTPUT
 	FPRINTF(stdout, "Input :\n");
 
-	for (j = 0; j < size; j++)
+	for (j = 0; j < size_p; j++)
 	{
-		for (i = 0; i < size; i++)
+		for (i = 0; i < size_p; i++)
 		{
 			if (i <= j)
 			{
-				FPRINTF(stdout, "%2.2f\t", mat[j +i*size]);
+				FPRINTF(stdout, "%2.2f\t", mat[j +i*size_p]);
 			}
 			else
 			{
@@ -353,43 +354,43 @@ int main(int argc, char **argv)
 #endif
 
 
-	cholesky(mat, size, size, nblocks);
+	cholesky(mat, size_p, size_p, nblocks_p);
 
 #ifdef CHECK_OUTPUT
 	FPRINTF(stdout, "Results :\n");
 
-	for (j = 0; j < size; j++)
+	for (j = 0; j < size_p; j++)
 	{
-		for (i = 0; i < size; i++)
+		for (i = 0; i < size_p; i++)
 		{
 			if (i <= j)
 			{
-				FPRINTF(stdout, "%2.2f\t", mat[j +i*size]);
+				FPRINTF(stdout, "%2.2f\t", mat[j +i*size_p]);
 			}
 			else
 			{
 				FPRINTF(stdout, ".\t");
-				mat[j+i*size] = 0.0f; /* debug */
+				mat[j+i*size_p] = 0.0f; /* debug */
 			}
 		}
 		FPRINTF(stdout, "\n");
 	}
 
 	FPRINTF(stderr, "compute explicit LLt ...\n");
-	float *test_mat = malloc(size*size*sizeof(float));
+	float *test_mat = malloc(size_p*size_p*sizeof(float));
 	STARPU_ASSERT(test_mat);
 
-	STARPU_SSYRK("L", "N", size, size, 1.0f,
-				mat, size, 0.0f, test_mat, size);
+	STARPU_SSYRK("L", "N", size_p, size_p, 1.0f,
+		     mat, size_p, 0.0f, test_mat, size_p);
 
 	FPRINTF(stderr, "comparing results ...\n");
-	for (j = 0; j < size; j++)
+	for (j = 0; j < size_p; j++)
 	{
-		for (i = 0; i < size; i++)
+		for (i = 0; i < size_p; i++)
 		{
 			if (i <= j)
 			{
-				FPRINTF(stdout, "%2.2f\t", test_mat[j +i*size]);
+				FPRINTF(stdout, "%2.2f\t", test_mat[j +i*size_p]);
 			}
 			else
 			{
@@ -401,6 +402,6 @@ int main(int argc, char **argv)
 	free(test_mat);
 #endif
 
-	shutdown_system(&mat, size, pinned);
+	shutdown_system(&mat, size_p, pinned_p);
 	return 0;
 }
