@@ -1,8 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
  * Copyright (C) 2009-2017                                Université de Bordeaux
- * Copyright (C) 2010-2013,2015-2016                      CNRS
+ * Copyright (C) 2012,2013                                Inria
+ * Copyright (C) 2010-2013,2015-2017                      CNRS
+ * Copyright (C) 2013                                     Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -39,9 +40,9 @@ starpu_data_handle_t A_state[NMAXBLOCKS][NMAXBLOCKS];
 static struct starpu_task *create_task(starpu_tag_t id)
 {
 	struct starpu_task *task = starpu_task_create();
-		task->cl_arg = NULL;
-		task->use_tag = 1;
-		task->tag_id = id;
+	task->cl_arg = NULL;
+	task->use_tag = 1;
+	task->tag_id = id;
 
 	return task;
 }
@@ -52,10 +53,11 @@ static struct starpu_task *create_task(starpu_tag_t id)
 
 static struct starpu_task * create_task_11(unsigned k, unsigned nblocks)
 {
-/*	FPRINTF(stdout, "task 11 k = %d TAG = %llx\n", k, (TAG11(k))); */
+	(void)nblocks;
+	/*	FPRINTF(stdout, "task 11 k = %d TAG = %llx\n", k, (TAG11(k))); */
 
 	struct starpu_task *task = create_task(TAG11(k));
-	
+
 	task->cl = &cl11;
 
 	/* which sub-data is manipulated ? */
@@ -82,7 +84,7 @@ static int create_task_21(unsigned k, unsigned j)
 
 	struct starpu_task *task = create_task(TAG21(k, j));
 
-	task->cl = &cl21;	
+	task->cl = &cl21;
 
 	/* which sub-data is manipulated ? */
 	task->handles[0] = A_state[k][k];
@@ -150,7 +152,7 @@ static int create_task_22(unsigned k, unsigned i, unsigned j)
 }
 
 /*
- *	code to bootstrap the factorization 
+ *	code to bootstrap the factorization
  *	and construct the DAG
  */
 
@@ -166,10 +168,10 @@ static int cholesky_no_stride(void)
 	/* create all the DAG nodes */
 	unsigned i,j,k;
 
-	for (k = 0; k < nblocks; k++)
+	for (k = 0; k < nblocks_p; k++)
 	{
 		starpu_iteration_push(k);
-		struct starpu_task *task = create_task_11(k, nblocks);
+		struct starpu_task *task = create_task_11(k, nblocks_p);
 		/* we defer the launch of the first task */
 		if (k == 0)
 		{
@@ -180,13 +182,13 @@ static int cholesky_no_stride(void)
 			ret = starpu_task_submit(task);
 			STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 		}
-		
-		for (j = k+1; j<nblocks; j++)
+
+		for (j = k+1; j<nblocks_p; j++)
 		{
 			ret = create_task_21(k, j);
 			if (ret == -ENODEV) return 77;
 
-			for (i = k+1; i<nblocks; i++)
+			for (i = k+1; i<nblocks_p; i++)
 			{
 				if (i <= j)
 				{
@@ -205,15 +207,15 @@ static int cholesky_no_stride(void)
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 
 	/* stall the application until the end of computations */
-	starpu_tag_wait(TAG11(nblocks-1));
+	starpu_tag_wait(TAG11(nblocks_p-1));
 
 	end = starpu_timing_now();
 
 	double timing = end - start;
 
-	double flop = (1.0f*size*size*size)/3.0f;
+	double flop = (1.0f*size_p*size_p*size_p)/3.0f;
 	PRINTF("# size\tms\tGFlops\n");
-	PRINTF("%u\t%.0f\t%.1f\n", size, timing/1000, (flop/timing/1000.0f));
+	PRINTF("%u\t%.0f\t%.1f\n", size_p, timing/1000, (flop/timing/1000.0f));
 
 	return 0;
 }
@@ -235,9 +237,9 @@ int main(int argc, char **argv)
 	init_sizes();
 
 	parse_args(argc, argv);
-	assert(nblocks <= NMAXBLOCKS);
+	assert(nblocks_p <= NMAXBLOCKS);
 
-	FPRINTF(stderr, "BLOCK SIZE = %u\n", size / nblocks);
+	FPRINTF(stderr, "BLOCK SIZE = %u\n", size_p / nblocks_p);
 
 #ifdef STARPU_USE_CUDA
 	initialize_chol_model(&chol_model_11,"chol_model_11",cpu_chol_task_11_cost,cuda_chol_task_11_cost);
@@ -254,8 +256,8 @@ int main(int argc, char **argv)
 
 	starpu_cublas_init();
 
-	for (y = 0; y < nblocks; y++)
-	for (x = 0; x < nblocks; x++)
+	for (y = 0; y < nblocks_p; y++)
+	for (x = 0; x < nblocks_p; x++)
 	{
 		if (x <= y)
 		{
@@ -267,10 +269,10 @@ int main(int argc, char **argv)
 #ifndef STARPU_SIMGRID
 	/* create a simple definite positive symetric matrix example
 	 *
-	 *	Hilbert matrix : h(i,j) = 1/(i+j+1) ( + n In to make is stable ) 
+	 *	Hilbert matrix : h(i,j) = 1/(i+j+1) ( + n In to make is stable )
 	 * */
-	for (y = 0; y < nblocks; y++)
-	for (x = 0; x < nblocks; x++)
+	for (y = 0; y < nblocks_p; y++)
+	for (x = 0; x < nblocks_p; x++)
 	if (x <= y)
 	{
 		unsigned i, j;
@@ -282,26 +284,26 @@ int main(int argc, char **argv)
 
 			/* make it a little more numerically stable ... ;) */
 			if ((x == y) && (i == j))
-				A[y][x][i*BLOCKSIZE + j] += (float)(2*size);
+				A[y][x][i*BLOCKSIZE + j] += (float)(2*size_p);
 		}
 	}
 #endif
 
-	for (y = 0; y < nblocks; y++)
-	for (x = 0; x < nblocks; x++)
+	for (y = 0; y < nblocks_p; y++)
+	for (x = 0; x < nblocks_p; x++)
 	{
 		if (x <= y)
 		{
-			starpu_matrix_data_register(&A_state[y][x], STARPU_MAIN_RAM, (uintptr_t)A[y][x], 
-				BLOCKSIZE, BLOCKSIZE, BLOCKSIZE, sizeof(float));
+			starpu_matrix_data_register(&A_state[y][x], STARPU_MAIN_RAM, (uintptr_t)A[y][x],
+						    BLOCKSIZE, BLOCKSIZE, BLOCKSIZE, sizeof(float));
 			starpu_data_set_coordinates(A_state[y][x], 2, x, y);
 		}
 	}
 
 	ret = cholesky_no_stride();
 
-	for (y = 0; y < nblocks; y++)
-	for (x = 0; x < nblocks; x++)
+	for (y = 0; y < nblocks_p; y++)
+	for (x = 0; x < nblocks_p; x++)
 	{
 		if (x <= y)
 		{
@@ -315,5 +317,3 @@ int main(int argc, char **argv)
 	starpu_shutdown();
 	return ret;
 }
-
-
