@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
- * Copyright (C) 2012-2013,2015-2017                      CNRS
- * Copyright (C) 2012-2014, 2018                                Université de Bordeaux
+ * Copyright (C) 2012,2013                                Inria
+ * Copyright (C) 2012,2013,2015-2017                      CNRS
+ * Copyright (C) 2012-2014,2018                           Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -73,8 +73,7 @@ static void     register_custom_handle(starpu_data_handle_t handle,
 				       void *data_interface);
 static starpu_ssize_t  allocate_custom_buffer_on_node(void *data_interface_,
 					       unsigned dst_node);
-static void*    custom_handle_to_pointer(starpu_data_handle_t data_handle,
-					 unsigned node);
+static void*    custom_to_pointer(void *data_interface, unsigned node);
 static int      custom_pointer_is_inside(void *data_interface, unsigned node, void *ptr);
 static void     free_custom_buffer_on_node(void *data_interface, unsigned node);
 static size_t   custom_interface_get_size(starpu_data_handle_t handle);
@@ -95,7 +94,7 @@ static struct starpu_data_interface_ops interface_custom_ops =
 {
 	.register_data_handle  = register_custom_handle,
 	.allocate_data_on_node = allocate_custom_buffer_on_node,
-	.handle_to_pointer     = custom_handle_to_pointer,
+	.to_pointer            = custom_to_pointer,
 	.pointer_is_inside     = custom_pointer_is_inside,
 	.free_data_on_node     = free_custom_buffer_on_node,
 	.copy_methods          = &custom_copy_data_methods_s,
@@ -205,10 +204,9 @@ static void free_custom_buffer_on_node(void *data_interface, unsigned node)
 }
 
 static void*
-custom_handle_to_pointer(starpu_data_handle_t handle, unsigned node)
+custom_to_pointer(void *data, unsigned node)
 {
-	struct custom_data_interface *data_interface =
-		(struct custom_data_interface *) starpu_data_get_interface_on_node(handle, node);
+	struct custom_data_interface *data_interface = data;
 
 
 	switch(starpu_node_get_kind(node))
@@ -318,6 +316,8 @@ copy_cuda_common_async(void *src_interface, unsigned src_node,
 		       void *dst_interface, unsigned dst_node,
 		       cudaStream_t stream, enum cudaMemcpyKind kind)
 {
+	(void)src_node;
+	(void)dst_node;
 	struct custom_data_interface *src_custom, *dst_custom;
 
 	src_custom = (struct custom_data_interface *) src_interface;
@@ -386,12 +386,21 @@ static int copy_cuda_to_ram_async(void *src_interface, unsigned src_node,
 static int copy_cuda_to_cuda(void *src_interface, unsigned src_node,
 			     void *dst_interface, unsigned dst_node)
 {
+	(void)src_interface;
+	(void)src_node;
+	(void)dst_interface;
+	(void)dst_node;
 	assert(0);
 }
 static int copy_cuda_to_cuda_async(void *src_interface, unsigned src_node,
 				   void *dst_interface, unsigned dst_node,
 				   cudaStream_t stream)
 {
+	(void)src_interface;
+	(void)src_node;
+	(void)dst_interface;
+	(void)dst_node;
+	(void)stream;
 	assert(0);
 }
 #endif /* !STARPU_USE_CUDA */
@@ -431,6 +440,7 @@ static int copy_ram_to_opencl_async(void *src_interface, unsigned src_node,
 				    void *dst_interface, unsigned dst_node,
 				    cl_event *event)
 {
+	(void)event;
 	starpu_ssize_t size;
 	struct custom_data_interface *src_custom, *dst_custom;
 
@@ -474,6 +484,7 @@ static int copy_opencl_to_ram_async(void *src_interface, unsigned src_node,
 				    void *dst_interface, unsigned dst_node,
 				    cl_event *event)
 {
+	(void)event;
 	starpu_ssize_t size;
 	struct custom_data_interface *src_custom, *dst_custom;
 
