@@ -1,6 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2012,2013                                Inria
+ * Copyright (C) 2010-2013,2015,2017                      CNRS
+ * Copyright (C) 2010,2011,2013-2015                      Université de Bordeaux
+ * Copyright (C) 2010                                     Mehdi Juhoor
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -14,6 +17,11 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
+/*
+ * This examplifies how to use partitioning filters.  We here just split a
+ * vector into slices, and run a dumb kernel on them.
+ */
+
 #include <starpu.h>
 
 #define NX    21
@@ -35,7 +43,7 @@ void cpu_func(void *buffers[], void *cl_arg)
                 val[i] *= *factor;
 }
 
-int main(int argc, char **argv)
+int main(void)
 {
 	int i;
         int vector[NX];
@@ -45,7 +53,8 @@ int main(int argc, char **argv)
 
         struct starpu_codelet cl =
 	{
-                .cpu_funcs = {cpu_func, NULL},
+                .cpu_funcs = {cpu_func},
+                .cpu_funcs_name = {"cpu_func"},
                 .nbuffers = 1,
 		.modes = {STARPU_RW},
 		.name = "vector_scal"
@@ -62,7 +71,7 @@ int main(int argc, char **argv)
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	/* Declare data to StarPU */
-	starpu_vector_data_register(&handle, 0, (uintptr_t)vector, NX, sizeof(vector[0]));
+	starpu_vector_data_register(&handle, STARPU_MAIN_RAM, (uintptr_t)vector, NX, sizeof(vector[0]));
 
         /* Partition the vector in PARTS sub-vectors */
 	struct starpu_data_filter f =
@@ -90,7 +99,7 @@ int main(int argc, char **argv)
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 	}
 
-	starpu_data_unpartition(handle, 0);
+	starpu_data_unpartition(handle, STARPU_MAIN_RAM);
         starpu_data_unregister(handle);
 	starpu_shutdown();
 
