@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2011,2013-2014,2017                 Université de Bordeaux
- * Copyright (C) 2010-2013,2015                           CNRS
+ * Copyright (C) 2008-2011,2013,2014,2017                 Université de Bordeaux
+ * Copyright (C) 2010-2013,2015,2017                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -63,7 +63,7 @@ struct cg_problem
 };
 
 /* some useful functions */
-static void STARPU_ATTRIBUTE_UNUSED parse_args(int argc, char **argv)
+static void parse_args(int argc, char **argv)
 {
 	int i;
 	for (i = 1; i < argc; i++)
@@ -94,7 +94,7 @@ static void STARPU_ATTRIBUTE_UNUSED parse_args(int argc, char **argv)
 }
 
 
-static void STARPU_ATTRIBUTE_UNUSED print_results(float *result, unsigned size)
+static void print_results(float *result, unsigned size)
 {
 	printf("**** RESULTS **** \n");
 	unsigned i;
