@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010-2011  Université de Bordeaux
- * Copyright (C) 2010, 2011  CNRS
+ * Copyright (C) 2008-2011,2013,2014                      Université de Bordeaux
+ * Copyright (C) 2010,2011,2015,2017                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -143,7 +143,7 @@ static void display(void)
 }
 
 
-static void pressKey(unsigned char key, int x STARPU_ATTRIBUTE_UNUSED, int y  STARPU_ATTRIBUTE_UNUSED)
+static void pressKey(unsigned char key, int x, int y)
 {
 	switch (key)
 	{
