@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2012                                Inria
- * Copyright (C) 2009-2017                                Université de Bordeaux
+ * Copyright (C) 2009-2017,2019                           Université de Bordeaux
+ * Copyright (C) 2011,2012                                Inria
  * Copyright (C) 2010-2013,2015-2018                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -111,6 +111,10 @@ static void parse_args(int argc, char **argv)
 
 static void display_matrix(TYPE *m, unsigned n, unsigned ld, char *str)
 {
+	(void)m;
+	(void)n;
+	(void)ld;
+	(void)str;
 #if 0
 	FPRINTF(stderr, "***********\n");
 	FPRINTF(stderr, "Display matrix %s\n", str);
@@ -172,7 +176,11 @@ void copy_matrix_into_blocks(void)
 static void init_matrix(void)
 {
 	/* allocate matrix */
+#ifdef STARPU_SIMGRID
+	A = (void*) 1;
+#else
 	starpu_malloc_flags((void **)&A, (size_t)size*size*sizeof(TYPE), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
+#endif
 	STARPU_ASSERT(A);
 
 	starpu_srand48((long int)time(NULL));
@@ -431,10 +439,10 @@ int main(int argc, char **argv)
 
 		check_result();
 	}
-#endif
 
 	if (pivot)
 		free(ipiv);
+#endif
 
 	starpu_free_flags(A, (size_t)size*size*sizeof(TYPE), STARPU_MALLOC_PINNED|STARPU_MALLOC_SIMULATION_FOLDED);
 
