@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2014                                     Inria
- * Copyright (C) 2010-2017                                CNRS
+ * Copyright (C) 2014,2017                                Inria
+ * Copyright (C) 2010-2017, 2019                          CNRS
  * Copyright (C) 2010-2015                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -49,13 +49,16 @@ int parallel_code(int sched_ctx)
 	return t;
 }
 
-static void sched_ctx_func(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg)
+static void sched_ctx_func(void *descr[], void *arg)
 {
-	int w = starpu_worker_get_id();
-	(void) w;
+	(void)descr;
 	unsigned sched_ctx = (uintptr_t)arg;
-	int n = parallel_code(sched_ctx);
-	(void) n;
+	int t = parallel_code(sched_ctx);
+	if (sched_ctx > 0 && sched_ctx < 3)
+	{
+		STARPU_ATOMIC_ADD(&tasks_executed[sched_ctx-1], t);
+	}
+
 	//printf("w %d executed %d it \n", w, n);
 }
 
@@ -68,7 +71,7 @@ static struct starpu_codelet sched_ctx_codelet =
 	.name = "sched_ctx"
 };
 
-int main(int argc, char **argv)
+int main(void)
 {
 	tasks_executed[0] = 0;
 	tasks_executed[1] = 0;
@@ -124,9 +127,9 @@ int main(int argc, char **argv)
 //	starpu_sched_ctx_set_inheritor(sched_ctx2, sched_ctx1);
 
 	int nprocs3 = nprocs1/2;
-	int nprocs4 = nprocs1/2;
+	int nprocs4 = nprocs3;
 	int nprocs5 = nprocs2/2;
-	int nprocs6 = nprocs2/2;
+	int nprocs6 = nprocs5;
 	int *procs3 = NULL;
 	int *procs4 = NULL;
 	int *procs5 = NULL;
@@ -233,8 +236,8 @@ int main(int argc, char **argv)
 	starpu_sched_ctx_delete(sched_ctx1);
 	starpu_sched_ctx_delete(sched_ctx2);
 
-	printf("ctx%u: tasks starpu executed %d out of %d\n", sched_ctx1, tasks_executed[0], NTASKS);
-	printf("ctx%u: tasks starpu executed %d out of %d\n", sched_ctx2, tasks_executed[1], NTASKS);
+	printf("ctx%u: tasks starpu executed %d out of %d\n", sched_ctx1, tasks_executed[0], NTASKS*NTASKS);
+	printf("ctx%u: tasks starpu executed %d out of %d\n", sched_ctx2, tasks_executed[1], NTASKS*NTASKS);
 
 #ifdef STARPU_USE_CPU
 	free(procs1);
