@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2012  Université de Bordeaux
+ * Copyright (C) 2011                                     Inria
+ * Copyright (C) 2012,2017                                CNRS
+ * Copyright (C) 2010-2012,2014                           Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -21,7 +23,7 @@
 
 void parse_args_ctx(int argc, char **argv);
 void update_sched_ctx_timing_results(double gflops, double timing);
-void construct_contexts(void (*bench)(unsigned size, unsigned nblocks));
+void construct_contexts();
 void start_2benchs(void (*bench)(unsigned size, unsigned nblocks));
 void start_1stbench(void (*bench)(unsigned size, unsigned nblocks));
 void start_2ndbench(void (*bench)(unsigned size, unsigned nblocks));
