@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2009,2011,2013,2016                 Université de Bordeaux
- * Copyright (C) 2010,2013-2016                           CNRS
+ * Copyright (C) 2010,2013-2017                           CNRS
+ * Copyright (C) 2008,2009,2011,2013,2016                 Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -483,6 +483,6 @@ char  *mm_typecode_to_str(MM_typecode matcode)
 	else
 		return NULL;
 
-	sprintf(buffer,"%s %s %s %s", types[0], types[1], types[2], types[3]);
+	snprintf(buffer, sizeof(buffer), "%s %s %s %s", types[0], types[1], types[2], types[3]);
 	return mm_strdup(buffer);
 }
