@@ -1,7 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2011, 2013, 2015, 2017  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013, 2016, 2017  CNRS
+ * Copyright (C) 2010,2011,2013-2015,2017,2018            Université de Bordeaux
+ * Copyright (C) 2012,2013                                Inria
+ * Copyright (C) 2010-2013,2015-2018                      CNRS
+ * Copyright (C) 2013                                     Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -35,6 +37,7 @@ static unsigned check = 0;
 static int p = 1;
 static int q = 1;
 static unsigned display = 0;
+static unsigned no_prio = 0;
 
 #ifdef STARPU_HAVE_LIBNUMA
 static unsigned numa = 0;
@@ -69,6 +72,7 @@ static TYPE **(tmp_21_block[2]);
 
 static void parse_args(int rank, int argc, char **argv)
 {
+	(void)rank;
 	int i;
 	for (i = 1; i < argc; i++)
 	{
@@ -178,6 +182,9 @@ starpu_data_handle_t STARPU_PLU(get_tmp_21_block_handle)(unsigned i, unsigned k)
 
 static unsigned tmp_11_block_is_needed(int rank, unsigned pnblocks, unsigned k)
 {
+	(void)rank;
+	(void)pnblocks;
+	(void)k;
 	return 1;
 }
 
@@ -418,6 +425,7 @@ int main(int argc, char **argv)
 {
 	int rank;
 	int world_size;
+	int ret;
 
 	/*
 	 *	Initialization
@@ -440,15 +448,12 @@ int main(int argc, char **argv)
 
 	parse_args(rank, argc, argv);
 
-	int ret = starpu_init(NULL);
-	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
+	ret = starpu_mpi_init_conf(NULL, NULL, 0, MPI_COMM_WORLD, NULL);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init_conf");
 
 	/* We disable sequential consistency in this example */
 	starpu_data_set_default_sequential_consistency_flag(0);
 
-	ret = starpu_mpi_init(NULL, NULL, 0);
-	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init");
-
 	STARPU_ASSERT(p*q == world_size);
 
 	starpu_cublas_init();
@@ -503,7 +508,7 @@ int main(int argc, char **argv)
 	barrier_ret = MPI_Barrier(MPI_COMM_WORLD);
 	STARPU_ASSERT(barrier_ret == MPI_SUCCESS);
 
-	double timing = STARPU_PLU(plu_main)(nblocks, rank, world_size);
+	double timing = STARPU_PLU(plu_main)(nblocks, rank, world_size, no_prio);
 
 	/*
 	 * 	Report performance
@@ -587,7 +592,6 @@ int main(int argc, char **argv)
 
 	starpu_cublas_shutdown();
 	starpu_mpi_shutdown();
-	starpu_shutdown();
 
 #if 0
 	MPI_Finalize();
