@@ -1,6 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2014, 2015, 2017  CNRS
+ * Copyright (C) 2014-2017, 2019                          CNRS
+ * Copyright (C) 2015                                     Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,51 +20,44 @@
 #include <stdio.h>
 #include <starpu_mpi_private.h>
 
-/* measure the amount of data transfers between each pair of MPI nodes */
-static size_t *comm_cache_amount;
-static int world_size;
 static int stats_enabled=0;
 
-void _starpu_mpi_cache_stats_init(MPI_Comm comm)
+void _starpu_mpi_cache_stats_init()
 {
 	stats_enabled = starpu_get_env_number("STARPU_MPI_CACHE_STATS");
 	if (stats_enabled == -1)
 	{
 		stats_enabled = 0;
 	}
-	if (stats_enabled == 0) return;
+	if (stats_enabled == 0)
+		return;
 
 	_STARPU_DISP("Warning: StarPU is executed with STARPU_MPI_CACHE_STATS=1, which slows down a bit\n");
 
-	starpu_mpi_comm_size(comm, &world_size);
-	_STARPU_MPI_DEBUG(1, "allocating for %d nodes\n", world_size);
-
-	_STARPU_MPI_CALLOC(comm_cache_amount, world_size, sizeof(size_t));
 }
 
-void _starpu_mpi_cache_stats_free()
+void _starpu_mpi_cache_stats_shutdown()
 {
-	if (stats_enabled == 0) return;
-	free(comm_cache_amount);
+	if (stats_enabled == 0)
+		return;
 }
 
 void _starpu_mpi_cache_stats_update(unsigned dst, starpu_data_handle_t data_handle, int count)
 {
 	size_t size;
 
-	if (stats_enabled == 0) return;
+	if (stats_enabled == 0)
+		return;
 
 	size = starpu_data_get_size(data_handle);
 
 	if (count == 1)
 	{
-		_STARPU_MPI_MSG("[communication cache] + %10ld to   %d\n", (long)size, dst);
+		_STARPU_MPI_MSG("[communication cache] + %10ld to   %u\n", (long)size, dst);
 	}
 	else // count == -1
 	{
-		_STARPU_MPI_MSG("[communication cache] - %10ld from %d\n", (long)size, dst);
+		_STARPU_MPI_MSG("[communication cache] - %10ld from %u\n", (long)size, dst);
 	}
-
-	comm_cache_amount[dst] += count * size;
 }
 
