@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2017                                CNRS
  * Copyright (C) 2013                                     Inria
- * Copyright (C) 2013-2015                                Université de Bordeaux
+ * Copyright (C) 2011-2017                                CNRS
+ * Copyright (C) 2013-2015,2017,2018                      Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,10 +20,11 @@
 #include <math.h>
 #include "helper.h"
 
-void func_cpu(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void func_cpu(void *descr[], void *_args)
 {
 	int node;
 	int rank;
+	(void)descr;
 
 	starpu_codelet_unpack_args(_args, &node);
 	starpu_mpi_comm_rank(MPI_COMM_WORLD, &rank);
@@ -32,23 +33,12 @@ void func_cpu(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 	assert(node == rank);
 }
 
-/* Dummy cost function for simgrid */
-static double cost_function(struct starpu_task *task STARPU_ATTRIBUTE_UNUSED, unsigned nimpl STARPU_ATTRIBUTE_UNUSED)
-{
-	return 0.000001;
-}
-static struct starpu_perfmodel dumb_model =
-{
-	.type		= STARPU_COMMON,
-	.cost_function	= cost_function
-};
-
 struct starpu_codelet mycodelet_r_w =
 {
 	.cpu_funcs = {func_cpu},
 	.nbuffers = 2,
 	.modes = {STARPU_R, STARPU_W},
-	.model = &dumb_model
+	.model = &starpu_perfmodel_nop,
 };
 
 struct starpu_codelet mycodelet_rw_r =
@@ -56,7 +46,7 @@ struct starpu_codelet mycodelet_rw_r =
 	.cpu_funcs = {func_cpu},
 	.nbuffers = 2,
 	.modes = {STARPU_RW, STARPU_R},
-	.model = &dumb_model
+	.model = &starpu_perfmodel_nop,
 };
 
 struct starpu_codelet mycodelet_rw_rw =
@@ -64,7 +54,7 @@ struct starpu_codelet mycodelet_rw_rw =
 	.cpu_funcs = {func_cpu},
 	.nbuffers = 2,
 	.modes = {STARPU_RW, STARPU_RW},
-	.model = &dumb_model
+	.model = &starpu_perfmodel_nop,
 };
 
 struct starpu_codelet mycodelet_w_r =
@@ -72,7 +62,7 @@ struct starpu_codelet mycodelet_w_r =
 	.cpu_funcs = {func_cpu},
 	.nbuffers = 2,
 	.modes = {STARPU_W, STARPU_R},
-	.model = &dumb_model
+	.model = &starpu_perfmodel_nop,
 };
 
 struct starpu_codelet mycodelet_r_r =
@@ -80,7 +70,7 @@ struct starpu_codelet mycodelet_r_r =
 	.cpu_funcs = {func_cpu},
 	.nbuffers = 2,
 	.modes = {STARPU_R, STARPU_R},
-	.model = &dumb_model
+	.model = &starpu_perfmodel_nop,
 };
 
 int main(int argc, char **argv)
@@ -91,13 +81,19 @@ int main(int argc, char **argv)
 	starpu_data_handle_t data_handlesx0 = NULL;
 	starpu_data_handle_t data_handlesx1 = NULL;
 
-	ret = starpu_init(NULL);
-	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
-	ret = starpu_mpi_init(&argc, &argv, 1);
-	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init");
+	ret = starpu_mpi_init_conf(&argc, &argv, 1, MPI_COMM_WORLD, NULL);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init_conf");
 	starpu_mpi_comm_rank(MPI_COMM_WORLD, &rank);
 	starpu_mpi_comm_size(MPI_COMM_WORLD, &size);
 
+	if (starpu_cpu_worker_get_count() == 0)
+	{
+		if (rank == 0)
+			FPRINTF(stderr, "We need at least 1 CPU worker.\n");
+		starpu_mpi_shutdown();
+		return STARPU_TEST_SKIPPED;
+	}
+
 	if (rank != 0 && rank != 1)
 		goto end;
 
@@ -184,7 +180,6 @@ int main(int argc, char **argv)
 
 end:
 	starpu_mpi_shutdown();
-	starpu_shutdown();
 
 	return 0;
 }
