@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
+ * Copyright (C) 2012,2013                                Inria
  * Copyright (C) 2012-2017                                CNRS
- * Copyright (C) 2013-2015,2017                           Université de Bordeaux
+ * Copyright (C) 2013-2015,2017,2018                      Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -25,26 +25,13 @@ extern void redux_cpu_func(void *descr[], void *cl_arg);
 extern void dot_cpu_func(void *descr[], void *cl_arg);
 extern void display_cpu_func(void *descr[], void *cl_arg);
 
-#ifdef STARPU_SIMGRID
-/* Dummy cost function for simgrid */
-static double cost_function(struct starpu_task *task STARPU_ATTRIBUTE_UNUSED, unsigned nimpl STARPU_ATTRIBUTE_UNUSED)
-{
-	return 0.000001;
-}
-static struct starpu_perfmodel dumb_model =
-{
-	.type		= STARPU_COMMON,
-	.cost_function	= cost_function
-};
-#endif
-
 static struct starpu_codelet init_codelet =
 {
 	.cpu_funcs = {init_cpu_func},
 	.nbuffers = 1,
 	.modes = {STARPU_W},
 #ifdef STARPU_SIMGRID
-	.model = &dumb_model,
+	.model = &starpu_perfmodel_nop,
 #endif
 	.name = "init_codelet"
 };
@@ -55,7 +42,7 @@ static struct starpu_codelet redux_codelet =
 	.modes = {STARPU_RW, STARPU_R},
 	.nbuffers = 2,
 #ifdef STARPU_SIMGRID
-	.model = &dumb_model,
+	.model = &starpu_perfmodel_nop,
 #endif
 	.name = "redux_codelet"
 };
@@ -66,7 +53,7 @@ static struct starpu_codelet dot_codelet =
 	.nbuffers = 2,
 	.modes = {STARPU_R, STARPU_REDUX},
 #ifdef STARPU_SIMGRID
-	.model = &dumb_model,
+	.model = &starpu_perfmodel_nop,
 #endif
 	.name = "dot_codelet"
 };
@@ -77,7 +64,7 @@ static struct starpu_codelet display_codelet =
 	.nbuffers = 1,
 	.modes = {STARPU_R},
 #ifdef STARPU_SIMGRID
-	.model = &dumb_model,
+	.model = &starpu_perfmodel_nop,
 #endif
 	.name = "display_codelet"
 };
@@ -95,6 +82,7 @@ int main(int argc, char **argv)
 	long int dot, sum=0;
 	starpu_data_handle_t *handles;
 	starpu_data_handle_t dot_handle;
+	int ret;
 
 	int nb_elements, step, loops;
 
@@ -104,13 +92,19 @@ int main(int argc, char **argv)
 	if (starpu_get_env_number_default("STARPU_GLOBAL_ARBITER", 0) > 0)
 		return STARPU_TEST_SKIPPED;
 
-	int ret = starpu_init(NULL);
-	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
-	ret = starpu_mpi_init(&argc, &argv, 1);
-	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init");
+	ret = starpu_mpi_init_conf(&argc, &argv, 1, MPI_COMM_WORLD, NULL);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init_conf");
 	starpu_mpi_comm_rank(MPI_COMM_WORLD, &my_rank);
 	starpu_mpi_comm_size(MPI_COMM_WORLD, &size);
 
+	if (starpu_cpu_worker_get_count() == 0)
+	{
+		if (my_rank == 0)
+			FPRINTF(stderr, "We need at least 1 CPU worker.\n");
+		starpu_mpi_shutdown();
+		return STARPU_TEST_SKIPPED;
+	}
+
 	nb_elements = size*8000;
 	step = 4;
 	loops = 5;
@@ -191,7 +185,6 @@ int main(int argc, char **argv)
 	free(handles);
 
 	starpu_mpi_shutdown();
-	starpu_shutdown();
 
 	if (my_rank == 0)
 	{
