@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
  * Copyright (C) 2011-2013                                Inria
- * Copyright (C) 2012-2013,2015,2017                      CNRS
+ * Copyright (C) 2012,2013,2017                           CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -26,7 +26,7 @@
 
 void parse_args_ctx(int argc, char **argv);
 void update_sched_ctx_timing_results(double gflops, double timing);
-void construct_contexts(void (*bench)(float *mat, unsigned size, unsigned nblocks));
+void construct_contexts();
 void end_contexts(void);
 void start_2benchs(void (*bench)(float *mat, unsigned size, unsigned nblocks));
 void start_1stbench(void (*bench)(float *mat, unsigned size, unsigned nblocks));
