@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2014                                Inria
- * Copyright (C) 2012-2017                                CNRS
+ * Copyright (C) 2011-2016                                Inria
+ * Copyright (C) 2012-2014,2016,2017                      CNRS
  * Copyright (C) 2017                                     Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -41,7 +41,6 @@ static void _try_resizing(unsigned *sched_ctxs, int nsched_ctxs, int *workers, i
 
 	int total_nw[nw];
 	sc_hypervisor_group_workers_by_type(tw, total_nw);
-	unsigned can_redistrib = 0;
 
 
 	struct timeval start_time;
@@ -69,6 +68,7 @@ static void _try_resizing(unsigned *sched_ctxs, int nsched_ctxs, int *workers, i
 #ifdef STARPU_SC_HYPERVISOR_DEBUG
 	printf("*****finished resize \n");
 #endif
+	free(tw);
 	return;
 }
 
@@ -176,7 +176,6 @@ static void _resize_if_speed_diff(unsigned sched_ctx, int worker)
 		unsigned *sched_ctxs_old = NULL;
 		int nsched_ctxs_old = 0;
 		unsigned is_speed_diff = 0;
-		unsigned last_level_diff = 0;
 
 		for(level = current_level ; level >= 0; level--)
 		{
@@ -298,7 +297,7 @@ static void feft_lp_size_ctxs(unsigned *sched_ctxs, int nsched_ctxs, int *worker
 
 static void _resize_leaves(int worker)
 {
-	unsigned s, s2;
+	unsigned s;
 	unsigned *sched_ctxs = NULL;
 	unsigned nsched_ctxs = starpu_worker_get_sched_ctx_list(worker, &sched_ctxs);
        	unsigned workers_sched_ctxs[nsched_ctxs];
@@ -372,6 +371,7 @@ struct sc_hypervisor_policy feft_lp_policy = {
 	.handle_post_exec_hook = NULL,
 	.handle_submitted_job = NULL,
 	.end_ctx = NULL,
+	.init_worker = NULL,
 	.custom = 0,
 	.name = "feft_lp"
 };
