@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011, 2012  INRIA
+ * Copyright (C) 2011-2017                                Inria
+ * Copyright (C) 2012-2014,2016,2017,2019                 CNRS
+ * Copyright (C) 2014,2017                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,6 +20,8 @@
 #include <sc_hypervisor_policy.h>
 #include <starpu_config.h>
 
+struct sc_hypervisor hypervisor;
+starpu_pthread_mutex_t act_hypervisor_mutex;
 double hyp_overhead = 0.0;
 unsigned imposed_resize = 0;
 unsigned type_of_tasks_known = 0;
@@ -43,6 +47,7 @@ extern struct sc_hypervisor_policy throughput_lp_policy;
 #endif // STARPU_HAVE_GLPK_
 extern struct sc_hypervisor_policy ispeed_policy;
 extern struct sc_hypervisor_policy hard_coded_policy;
+extern struct sc_hypervisor_policy perf_count_policy;
 
 
 static struct sc_hypervisor_policy *predefined_policies[] =
@@ -57,7 +62,8 @@ static struct sc_hypervisor_policy *predefined_policies[] =
 #endif // STARPU_HAVE_GLPK_H
 	&gflops_rate_policy,
 	&ispeed_policy,
-	&hard_coded_policy
+	&hard_coded_policy,
+	&perf_count_policy
 };
 
 static void _load_hypervisor_policy(struct sc_hypervisor_policy *policy)
@@ -74,6 +80,8 @@ static void _load_hypervisor_policy(struct sc_hypervisor_policy *policy)
 	hypervisor.policy.handle_post_exec_hook = policy->handle_post_exec_hook;
 	hypervisor.policy.handle_submitted_job = policy->handle_submitted_job;
 	hypervisor.policy.end_ctx = policy->end_ctx;
+	hypervisor.policy.start_ctx = policy->start_ctx;
+	hypervisor.policy.init_worker = policy->init_worker;
 }
 
 
@@ -331,6 +339,9 @@ void sc_hypervisor_print_overhead()
 /* the hypervisor is in charge only of the contexts registered to it*/
 void sc_hypervisor_register_ctx(unsigned sched_ctx, double total_flops)
 {
+	if(hypervisor.policy.start_ctx)
+		hypervisor.policy.start_ctx(sched_ctx);
+
 	STARPU_PTHREAD_MUTEX_LOCK(&act_hypervisor_mutex);
 	hypervisor.configurations[sched_ctx] = NULL;
 	hypervisor.resize_requests[sched_ctx] = NULL;
@@ -396,7 +407,6 @@ void sc_hypervisor_unregister_ctx(unsigned sched_ctx)
 	if(npus)
 	{
 		starpu_sched_ctx_set_priority(pus, npus, father, 1);
-		starpu_sched_ctx_set_priority_on_level(pus, npus, father, 1);
 		free(pus);
 	}
 
@@ -884,7 +894,7 @@ void _sc_hypervisor_allow_compute_idle(unsigned sched_ctx, int worker, unsigned
 
 int _update_max_hierarchically(unsigned *sched_ctxs, int nsched_ctxs)
 {
-	int s, i;
+	int s;
 	unsigned leaves[hypervisor.nsched_ctxs];
 	int nleaves = 0;
 	sc_hypervisor_get_leaves(hypervisor.sched_ctxs, hypervisor.nsched_ctxs, leaves, &nleaves);
@@ -1371,10 +1381,10 @@ static void notify_post_exec_task(struct starpu_task *task, size_t data_size, ui
 
 	STARPU_PTHREAD_MUTEX_LOCK(&hypervisor.sched_ctx_w[sched_ctx].mutex);
 	hypervisor.sched_ctx_w[sched_ctx].remaining_flops -= flops;
+	STARPU_PTHREAD_MUTEX_UNLOCK(&hypervisor.sched_ctx_w[sched_ctx].mutex);
 
 	if(_sc_hypervisor_use_lazy_resize())
 		_ack_resize_completed(sched_ctx, worker);
-	starpu_pthread_mutex_unlock(&hypervisor.sched_ctx_w[sched_ctx].mutex);
 
 	if(starpu_sched_ctx_get_hierarchy_level(sched_ctx) > 0)
 	{
@@ -1643,7 +1653,10 @@ void sc_hypervisor_get_ctxs_on_level(unsigned **sched_ctxs, int *nsched_ctxs, un
 		        (*sched_ctxs)[(*nsched_ctxs)++] = hypervisor.sched_ctxs[s];
 	}
 	if(*nsched_ctxs == 0)
+	{
 		free(*sched_ctxs);
+		*sched_ctxs = NULL;
+	}
 	return;
 }
 
@@ -1687,3 +1700,9 @@ void sc_hypervisor_get_leaves(unsigned *sched_ctxs, int nsched_ctxs, unsigned *l
 	return;
 }
 
+
+void sc_hypervisor_init_worker(int workerid, unsigned sched_ctx)
+{
+	if(hypervisor.policy.init_worker)
+                hypervisor.policy.init_worker(workerid, sched_ctx);
+}
