@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010,2011, 2015 University of Bordeaux
+ * Copyright (C) 2012,2017                                CNRS
+ * Copyright (C) 2010,2011,2015                           Université de Bordeaux
+ * Copyright (C) 2011                                     Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -17,12 +19,11 @@
 #ifndef SOCL_COMMAND_QUEUE_H
 #define SOCL_COMMAND_QUEUE_H
 
-void command_queue_enqueue_ex(
-	cl_command_queue 	cq,		/* Command queue */
-	cl_command		cmd,		/* Command to enqueue */
-	cl_uint			num_events,	/* Number of explicit dependencies */
-	const cl_event *	events		/* Explicit dependencies */
-	);
+void command_queue_enqueue_ex(cl_command_queue 	cq,		/* Command queue */
+			      cl_command		cmd,		/* Command to enqueue */
+			      cl_uint			num_events,	/* Number of explicit dependencies */
+			      const cl_event *	events		/* Explicit dependencies */
+			      );
 
 #define command_queue_enqueue(cq, cmd, num_events, events)\
 	command_queue_enqueue_ex(cq, (cl_command)cmd, num_events, events)
