@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2011, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2013                                     Inria
+ * Copyright (C) 2010,2011,2013-2015,2017                 CNRS
+ * Copyright (C) 2011,2012,2014,2015,2017                 Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -21,9 +23,9 @@
 
 struct _starpu_barrier
 {
-	int count;
-	int reached_start;
-	int reached_exit;
+	unsigned count;
+	unsigned reached_start;
+	unsigned reached_exit;
 	double reached_flops;
 	starpu_pthread_mutex_t mutex;
 	starpu_pthread_mutex_t mutex_exit;
