@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2015,2017                           Université de Bordeaux
- * Copyright (C) 2013,2015                                Inria
- * Copyright (C) 2010-2013,2015,2017                      CNRS
+ * Copyright (C) 2013,2015-2017                           Inria
+ * Copyright (C) 2009-2015,2017,2018                      Université de Bordeaux
+ * Copyright (C) 2010-2013,2015,2017,2018,2019            CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -63,7 +63,7 @@
 /* the handle header lock must be taken by the caller */
 static struct _starpu_data_requester *may_unlock_data_req_list_head(starpu_data_handle_t handle)
 {
-	struct _starpu_data_requester_list *req_list;
+	struct _starpu_data_requester_prio_list *req_list;
 
 	if (handle->reduction_refcnt > 0)
 	{
@@ -71,26 +71,26 @@ static struct _starpu_data_requester *may_unlock_data_req_list_head(starpu_data_
 	}
 	else
 	{
-		if (_starpu_data_requester_list_empty(&handle->reduction_req_list))
+		if (_starpu_data_requester_prio_list_empty(&handle->reduction_req_list))
 			req_list = &handle->req_list;
 		else
 			req_list = &handle->reduction_req_list;
 	}
 
 	/* if there is no one to unlock ... */
-	if (_starpu_data_requester_list_empty(req_list))
+	if (_starpu_data_requester_prio_list_empty(req_list))
 		return NULL;
 
 	/* if there is no reference to the data anymore, we can use it */
 	if (handle->refcnt == 0)
-		return _starpu_data_requester_list_pop_front(req_list);
+		return _starpu_data_requester_prio_list_pop_front_highest(req_list);
 
 	/* Already writing to it, do not let another write access through */
 	if (handle->current_mode == STARPU_W)
 		return NULL;
 
 	/* data->current_mode == STARPU_R, so we can process more readers */
-	struct _starpu_data_requester *r = _starpu_data_requester_list_front(req_list);
+	struct _starpu_data_requester *r = _starpu_data_requester_prio_list_front_highest(req_list);
 
 	enum starpu_data_access_mode r_mode = r->mode;
 	if (r_mode == STARPU_RW)
@@ -100,7 +100,7 @@ static struct _starpu_data_requester *may_unlock_data_req_list_head(starpu_data_
 	 * access, we only proceed if the current mode is the same as the
 	 * requested mode. */
 	if (r_mode == handle->current_mode)
-		return _starpu_data_requester_list_pop_front(req_list);
+		return _starpu_data_requester_prio_list_pop_front_highest(req_list);
 	else
 		return NULL;
 }
@@ -118,6 +118,10 @@ static unsigned _starpu_attempt_to_submit_data_request(unsigned request_from_cod
 	if (handle->arbiter)
 		return _starpu_attempt_to_submit_arbitered_data_request(request_from_codelet, handle, mode, callback, argcb, j, buffer_index);
 
+	/* Do not care about some flags */
+	mode &= ~STARPU_COMMUTE;
+	mode &= ~STARPU_SSEND;
+	mode &= ~STARPU_LOCALITY;
 	if (mode == STARPU_RW)
 		mode = STARPU_W;
 
@@ -130,7 +134,7 @@ static unsigned _starpu_attempt_to_submit_data_request(unsigned request_from_cod
 		while (cpt < STARPU_SPIN_MAXTRY && _starpu_spin_trylock(&handle->header_lock))
 		{
 			cpt++;
-			_starpu_datawizard_progress(_starpu_memory_node_get_local_key(), 0);
+			_starpu_datawizard_progress(0);
 		}
 		if (cpt == STARPU_SPIN_MAXTRY)
 			_starpu_spin_lock(&handle->header_lock);
@@ -148,7 +152,7 @@ static unsigned _starpu_attempt_to_submit_data_request(unsigned request_from_cod
 
 	/* If we are currently performing a reduction, we freeze any request
 	 * that is not explicitely a reduction task. */
-	unsigned is_a_reduction_task = (request_from_codelet && j->reduction_task);
+	unsigned is_a_reduction_task = (request_from_codelet && j && j->reduction_task);
 
 	if (pending_reduction && !is_a_reduction_task)
 		frozen = 1;
@@ -192,14 +196,15 @@ static unsigned _starpu_attempt_to_submit_data_request(unsigned request_from_cod
 		r->is_requested_by_codelet = request_from_codelet;
 		r->j = j;
 		r->buffer_index = buffer_index;
+		r->prio = j ? j->task->priority : 0;
 		r->ready_data_callback = callback;
 		r->argcb = argcb;
 
 		/* We put the requester in a specific list if this is a reduction task */
-		struct _starpu_data_requester_list *req_list =
+		struct _starpu_data_requester_prio_list *req_list =
 			is_a_reduction_task?&handle->reduction_req_list:&handle->req_list;
 
-		_starpu_data_requester_list_push_back(req_list, r);
+		_starpu_data_requester_prio_list_push_back(req_list, r);
 
 		/* failed */
 		put_in_list = 1;
@@ -248,8 +253,9 @@ static unsigned attempt_to_submit_data_request_from_job(struct _starpu_job *j, u
 {
 	/* Note that we do not access j->task->handles, but j->ordered_buffers
 	 * which is a sorted copy of it. */
-	starpu_data_handle_t handle = _STARPU_JOB_GET_ORDERED_BUFFER_HANDLE(j, buffer_index);
-	enum starpu_data_access_mode mode = _STARPU_JOB_GET_ORDERED_BUFFER_MODE(j, buffer_index) & ~STARPU_COMMUTE;
+	struct _starpu_data_descr *buffer = &(_STARPU_JOB_GET_ORDERED_BUFFERS(j)[buffer_index]);
+	starpu_data_handle_t handle = buffer->handle;
+	enum starpu_data_access_mode mode = buffer->mode & ~STARPU_COMMUTE;
 
 	return _starpu_attempt_to_submit_data_request(1, handle, mode, NULL, NULL, j, buffer_index);
 }
@@ -295,6 +301,43 @@ static unsigned _submit_job_enforce_data_deps(struct _starpu_job *j, unsigned st
 	return 0;
 }
 
+/* This is called when the tag+task dependencies are to be finished releasing.  */
+void _starpu_enforce_data_deps_notify_job_ready_soon(struct _starpu_job *j, _starpu_notify_job_start_data *data)
+{
+	unsigned buf;
+
+	if (j->task->cl) {
+		unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(j->task);
+
+		for (buf = 0; buf < nbuffers; buf++)
+		{
+			starpu_data_handle_t handle = STARPU_TASK_GET_HANDLE(j->task, buf);
+			if (handle->arbiter)
+				/* Oops, it's the arbiter's decision */
+				return;
+		}
+
+		/* We need to check data availability only if sequential consistency
+		 * dependencies have not been used */
+		if (!j->sequential_consistency) {
+			for (buf = 0; buf < nbuffers; buf++)
+			{
+				starpu_data_handle_t handle = STARPU_TASK_GET_HANDLE(j->task, buf);
+				enum starpu_data_access_mode mode = STARPU_TASK_GET_MODE(j->task, buf) & ~STARPU_COMMUTE;
+
+				if (handle->reduction_refcnt)
+					/* Reduction pending, don't bother trying */
+					return;
+				if (handle->refcnt != 0 && (mode == STARPU_W || handle->current_mode != mode))
+					/* Incompatible modes, not ready immediately */
+					return;
+			}
+		}
+	}
+	/* Ok, it really looks like this job will be ready soon */
+	_starpu_job_notify_ready_soon(j, data);
+}
+
 void _starpu_job_set_ordered_buffers(struct _starpu_job *j)
 {
 	/* Compute an ordered list of the different pieces of data so that we
@@ -303,19 +346,20 @@ void _starpu_job_set_ordered_buffers(struct _starpu_job *j)
 	unsigned i;
 	unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(j->task);
 	struct starpu_task *task = j->task;
+	struct _starpu_data_descr *buffers = _STARPU_JOB_GET_ORDERED_BUFFERS(j);
 
 	for (i=0 ; i<nbuffers; i++)
 	{
-		starpu_data_handle_t handle = STARPU_TASK_GET_HANDLE(task, i);
-		_STARPU_JOB_SET_ORDERED_BUFFER_HANDLE(j, handle, i);
-		enum starpu_data_access_mode mode = STARPU_TASK_GET_MODE(task, i);
-		_STARPU_JOB_SET_ORDERED_BUFFER_MODE(j, mode, i);
-		int node = -1;
-		if (task->cl->specific_nodes)
-			node = STARPU_CODELET_GET_NODE(task->cl, i);
-		_STARPU_JOB_SET_ORDERED_BUFFER_NODE(j, node, i);
+		buffers[i].index = i;
+		buffers[i].handle = STARPU_TASK_GET_HANDLE(task, i);
+		buffers[i].mode = STARPU_TASK_GET_MODE(task, i);
+		buffers[i].node = -1;
+	}
+	_starpu_sort_task_handles(buffers, nbuffers);
+	for (i=0 ; i<nbuffers; i++)
+	{
+		buffers[buffers[i].index].orderedindex = i;
 	}
-	_starpu_sort_task_handles(_STARPU_JOB_GET_ORDERED_BUFFERS(j), nbuffers);
 }
 
 /* Sort the data used by the given job by handle pointer value order, and
@@ -328,8 +372,6 @@ unsigned _starpu_submit_job_enforce_data_deps(struct _starpu_job *j)
 	if ((cl == NULL) || (STARPU_TASK_GET_NBUFFERS(j->task) == 0))
 		return 0;
 
-	_starpu_job_set_ordered_buffers(j);
-
 	return _submit_job_enforce_data_deps(j, 0);
 }
 
@@ -359,6 +401,20 @@ static unsigned unlock_one_requester(struct _starpu_data_requester *r)
 int _starpu_notify_data_dependencies(starpu_data_handle_t handle)
 {
 	_starpu_spin_checklocked(&handle->header_lock);
+
+	if (handle->arbiter)
+	{
+		/* Keep our reference for now, _starpu_notify_arbitered_dependencies
+		 * will drop it when it needs to */
+		STARPU_ASSERT(_starpu_data_requester_prio_list_empty(&handle->req_list));
+		STARPU_ASSERT(_starpu_data_requester_prio_list_empty(&handle->reduction_req_list));
+		_starpu_spin_unlock(&handle->header_lock);
+		/* _starpu_notify_arbitered_dependencies will handle its own locking */
+		_starpu_notify_arbitered_dependencies(handle);
+		/* We have already unlocked */
+		return 1;
+	}
+
 	/* A data access has finished so we remove a reference. */
 	STARPU_ASSERT(handle->refcnt > 0);
 	handle->refcnt--;
@@ -368,19 +424,7 @@ int _starpu_notify_data_dependencies(starpu_data_handle_t handle)
 		/* Handle was destroyed, nothing left to do.  */
 		return 1;
 
-	if (handle->arbiter)
-	{
-		unsigned refcnt = handle->refcnt;
-		STARPU_ASSERT(_starpu_data_requester_list_empty(&handle->req_list));
-		STARPU_ASSERT(_starpu_data_requester_list_empty(&handle->reduction_req_list));
-		_starpu_spin_unlock(&handle->header_lock);
-		/* _starpu_notify_arbitered_dependencies will handle its own locking */
-		if (!refcnt)
-			_starpu_notify_arbitered_dependencies(handle);
-		/* We have already unlocked */
-		return 1;
-	}
-	STARPU_ASSERT(_starpu_data_requester_list_empty(&handle->arbitered_req_list));
+	STARPU_ASSERT(_starpu_data_requester_prio_list_empty(&handle->arbitered_req_list));
 
 	/* In case there is a pending reduction, and that this is the last
 	 * requester, we may go back to a "normal" coherency model. */
@@ -429,7 +473,7 @@ int _starpu_notify_data_dependencies(starpu_data_handle_t handle)
 		{
 			/* We need to put the request back because we must
 			 * perform a reduction before. */
-			_starpu_data_requester_list_push_front(&handle->req_list, r);
+			_starpu_data_requester_prio_list_push_front(&handle->req_list, r);
 		}
 		else
 		{
