@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
  * Copyright (C) 2011,2012,2016                           Inria
- * Copyright (C) 2010-2018                                Université de Bordeaux
+ * Copyright (C) 2010-2019                                Université de Bordeaux
  * Copyright (C) 2010-2013,2015-2018                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -273,7 +273,8 @@ struct starpu_task *_starpu_detect_implicit_data_deps_with_handle(struct starpu_
 			_STARPU_DEP_DEBUG("dependency\n");
 
 			if ((l != &handle->last_submitted_accessors && l->next != &handle->last_submitted_accessors)
-					|| (handle->last_submitted_ghost_accessors_id && handle->last_submitted_ghost_accessors_id->next))
+					|| (handle->last_submitted_ghost_accessors_id && handle->last_submitted_ghost_accessors_id->next)
+					|| (l != &handle->last_submitted_accessors && handle->last_submitted_ghost_accessors_id))
 			{
 				/* Several previous accessors */
 
@@ -314,10 +315,12 @@ struct starpu_task *_starpu_detect_implicit_data_deps_with_handle(struct starpu_
 			}
 			else
 			{
-				/* One previous accessor, make it the sync
-				 * task, and start depending on it. */
+				struct _starpu_jobid_list *ghost_accessors_id = handle->last_submitted_ghost_accessors_id;
+				/* At most one previous accessor or one ghost */
 				if (l != &handle->last_submitted_accessors)
 				{
+					/* One accessor, make it the sync task,
+					 * and start depending on it. */
 					_STARPU_DEP_DEBUG("One previous accessor, depending on it\n");
 					handle->last_sync_task = l->task;
 					l->next = NULL;
@@ -325,13 +328,15 @@ struct starpu_task *_starpu_detect_implicit_data_deps_with_handle(struct starpu_
 					handle->last_submitted_accessors.next = &handle->last_submitted_accessors;
 					handle->last_submitted_accessors.prev = &handle->last_submitted_accessors;
 				}
-				else if (handle->last_submitted_ghost_accessors_id)
+				else if (ghost_accessors_id)
 				{
+					/* One ghost, just remember its id */
 					_STARPU_DEP_DEBUG("No more currently running accessor, but a ghost id, taking it.\n");
-					handle->last_submitted_ghost_sync_id = handle->last_submitted_ghost_accessors_id->id;
+					handle->last_submitted_ghost_sync_id = ghost_accessors_id->id;
 					handle->last_submitted_ghost_sync_id_is_valid = 1;
-					free(handle->last_submitted_ghost_accessors_id);
+					STARPU_ASSERT(!ghost_accessors_id->next);
 					handle->last_submitted_ghost_accessors_id = NULL;
+					free(ghost_accessors_id);
 				}
 				else
 				{
