@@ -1,9 +1,11 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2012                                Inria
- * Copyright (C) 2008-2016, 2018                          Université de Bordeaux
- * Copyright (C) 2010-2017                                CNRS
+ * Copyright (C) 2011,2012,2014,2016,2017                 Inria
+ * Copyright (C) 2008-2019                                Université de Bordeaux
+ * Copyright (C) 2010-2017, 2019                          CNRS
+ * Copyright (C) 2013                                     Thibaut Lambert
  * Copyright (C) 2011                                     Télécom-SudParis
+ * Copyright (C) 2016                                     Uppsala University
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -29,6 +31,7 @@
 #include <core/jobs.h>
 #include <core/workers.h>
 #include <datawizard/datawizard.h>
+#include <core/task.h>
 
 #ifdef STARPU_HAVE_WINDOWS
 #include <windows.h>
@@ -59,6 +62,9 @@ struct starpu_perfmodel_arch* starpu_worker_get_perf_archtype(int workerid, unsi
 		unsigned child_sched_ctx = starpu_sched_ctx_worker_is_master_for_child_ctx(workerid, sched_ctx_id);
 		if(child_sched_ctx != STARPU_NMAX_SCHED_CTXS)
 			return _starpu_sched_ctx_get_perf_archtype(child_sched_ctx);
+		struct _starpu_sched_ctx *stream_ctx = _starpu_worker_get_ctx_stream(workerid);
+		if(stream_ctx != NULL)
+			return _starpu_sched_ctx_get_perf_archtype(stream_ctx->id); 
 	}
 
 	struct _starpu_machine_config *config = _starpu_get_machine_config();
@@ -117,8 +123,8 @@ double starpu_worker_get_relative_speedup(struct starpu_perfmodel_arch* perf_arc
 			coef = _STARPU_OPENCL_ALPHA;
 		else if (perf_arch->devices[dev].type == STARPU_MIC_WORKER)
 			coef = _STARPU_MIC_ALPHA;
-		else if (perf_arch->devices[dev].type == STARPU_SCC_WORKER)
-			coef = _STARPU_SCC_ALPHA;
+		else if (perf_arch->devices[dev].type == STARPU_MPI_MS_WORKER)
+			coef = _STARPU_MPI_MS_ALPHA;
 
 		speedup += coef * (perf_arch->devices[dev].ncores);
 	}
@@ -161,6 +167,7 @@ void _starpu_init_and_load_perfmodel(struct starpu_perfmodel *model)
 			_starpu_load_history_based_model(model, 1);
 			break;
 		case STARPU_REGRESSION_BASED:
+		case STARPU_MULTIPLE_REGRESSION_BASED:
 			_starpu_load_history_based_model(model, 0);
 			break;
 
@@ -173,6 +180,7 @@ void _starpu_init_and_load_perfmodel(struct starpu_perfmodel *model)
 
 static double starpu_model_expected_perf(struct starpu_task *task, struct starpu_perfmodel *model, struct starpu_perfmodel_arch* arch,  unsigned nimpl)
 {
+	double exp_perf = 0.0;
 	if (model)
 	{
 		_starpu_init_and_load_perfmodel(model);
@@ -182,22 +190,36 @@ static double starpu_model_expected_perf(struct starpu_task *task, struct starpu
 		switch (model->type)
 		{
 			case STARPU_PER_ARCH:
-				return per_arch_task_expected_perf(model, arch, task, nimpl);
+				exp_perf = per_arch_task_expected_perf(model, arch, task, nimpl);
+				STARPU_ASSERT_MSG(isnan(exp_perf)||exp_perf>=0,"exp_perf=%lf\n",exp_perf);
+				break;
 			case STARPU_COMMON:
-				return common_task_expected_perf(model, arch, task, nimpl);
+				exp_perf = common_task_expected_perf(model, arch, task, nimpl);
+				STARPU_ASSERT_MSG(isnan(exp_perf)||exp_perf>=0,"exp_perf=%lf\n",exp_perf);
+				break;
 			case STARPU_HISTORY_BASED:
-				return _starpu_history_based_job_expected_perf(model, arch, j, nimpl);
+				exp_perf = _starpu_history_based_job_expected_perf(model, arch, j, nimpl);
+				STARPU_ASSERT_MSG(isnan(exp_perf)||exp_perf>=0,"exp_perf=%lf\n",exp_perf);
+				break;
 			case STARPU_REGRESSION_BASED:
-				return _starpu_regression_based_job_expected_perf(model, arch, j, nimpl);
+				exp_perf = _starpu_regression_based_job_expected_perf(model, arch, j, nimpl);
+				STARPU_ASSERT_MSG(isnan(exp_perf)||exp_perf>=0,"exp_perf=%lf\n",exp_perf);
+				break;
 			case STARPU_NL_REGRESSION_BASED:
-				return _starpu_non_linear_regression_based_job_expected_perf(model, arch, j,nimpl);
+				exp_perf = _starpu_non_linear_regression_based_job_expected_perf(model, arch, j,nimpl);
+				STARPU_ASSERT_MSG(isnan(exp_perf)||exp_perf>=0,"exp_perf=%lf\n",exp_perf);
+				break;
+			case STARPU_MULTIPLE_REGRESSION_BASED:
+				exp_perf = _starpu_multiple_regression_based_job_expected_perf(model, arch, j, nimpl);
+				STARPU_ASSERT_MSG(isnan(exp_perf)||exp_perf>=0,"exp_perf=%lf\n",exp_perf);
+				break;
 			default:
 				STARPU_ABORT();
 		}
 	}
 
 	/* no model was found */
-	return 0.0;
+	return exp_perf;
 }
 
 double starpu_task_expected_length(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl)
@@ -222,7 +244,6 @@ double starpu_task_expected_conversion_time(struct starpu_task *task,
 {
 	unsigned i;
 	double sum = 0.0;
-	enum starpu_node_kind node_kind;
 	unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(task);
 
 #ifdef STARPU_DEVEL
@@ -234,32 +255,13 @@ double starpu_task_expected_conversion_time(struct starpu_task *task,
 	{
 		starpu_data_handle_t handle;
 		struct starpu_task *conversion_task;
+		enum starpu_node_kind node_kind;
 
 		handle = STARPU_TASK_GET_HANDLE(task, i);
 		if (!_starpu_data_is_multiformat_handle(handle))
 			continue;
 
-		switch(arch->devices[0].type)
-		{
-			case STARPU_CPU_WORKER:
-				node_kind = STARPU_CPU_RAM;
-				break;
-			case STARPU_CUDA_WORKER:
-				node_kind = STARPU_CUDA_RAM;
-				break;
-			case STARPU_OPENCL_WORKER:
-				node_kind = STARPU_OPENCL_RAM;
-				break;
-			case STARPU_MIC_WORKER:
-				node_kind = STARPU_MIC_RAM;
-				break;
-			case STARPU_SCC_WORKER:
-				node_kind = STARPU_SCC_RAM;
-				break;
-			default:
-				STARPU_ABORT();
-				break;
-		}
+		node_kind = _starpu_worker_get_node_kind(arch->devices[0].type);
 		if (!_starpu_handle_needs_conversion_task_for_arch(handle, node_kind))
 			continue;
 
@@ -284,7 +286,7 @@ double starpu_data_expected_transfer_time(starpu_data_handle_t handle, unsigned
 	if (!(mode & STARPU_R))
 		return 0.0;
 
-	if (_starpu_is_data_present_or_requested(handle, memory_node))
+	if (starpu_data_is_on_node(handle, memory_node))
 		return 0.0;
 
 	size_t size = _starpu_data_get_size(handle);
@@ -301,7 +303,20 @@ double starpu_data_expected_transfer_time(starpu_data_handle_t handle, unsigned
 		/* Will just create it in place. Ideally we should take the
 		 * time to create it into account */
 		return 0.0;
-	return starpu_transfer_predict(src_node, memory_node, size);
+
+#define MAX_REQUESTS 4
+	unsigned src_nodes[MAX_REQUESTS];
+	unsigned dst_nodes[MAX_REQUESTS];
+	unsigned handling_nodes[MAX_REQUESTS];
+	int nhops = _starpu_determine_request_path(handle, src_node, memory_node, mode,
+			MAX_REQUESTS,
+			src_nodes, dst_nodes, handling_nodes, 0);
+	int i;
+	double duration = 0.;
+
+	for (i = 0; i < nhops; i++)
+		duration += starpu_transfer_predict(src_nodes[i], dst_nodes[i], size);
+	return duration;
 }
 
 /* Data transfer performance modeling */
@@ -316,9 +331,27 @@ double starpu_task_expected_data_transfer_time(unsigned memory_node, struct star
 	{
 		starpu_data_handle_t handle = STARPU_TASK_GET_HANDLE(task, buffer);
 		enum starpu_data_access_mode mode = STARPU_TASK_GET_MODE(task, buffer);
-		unsigned node = memory_node;
-		if (task->cl->specific_nodes)
-			node = STARPU_CODELET_GET_NODE(task->cl, buffer);
+		int node = _starpu_task_data_get_node_on_node(task, buffer, memory_node);
+
+		penalty += starpu_data_expected_transfer_time(handle, node, mode);
+	}
+
+	return penalty;
+}
+
+/* Data transfer performance modeling */
+double starpu_task_expected_data_transfer_time_for(struct starpu_task *task, unsigned worker)
+{
+	unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(task);
+	unsigned buffer;
+
+	double penalty = 0.0;
+
+	for (buffer = 0; buffer < nbuffers; buffer++)
+	{
+		starpu_data_handle_t handle = STARPU_TASK_GET_HANDLE(task, buffer);
+		enum starpu_data_access_mode mode = STARPU_TASK_GET_MODE(task, buffer);
+		int node = _starpu_task_data_get_node_on_worker(task, buffer, worker);
 
 		penalty += starpu_data_expected_transfer_time(handle, node, mode);
 	}
@@ -533,3 +566,16 @@ void starpu_perfmodel_free_sampling_directories(void)
 	_perf_model_dir_debug = NULL;
 	directory_existence_was_tested = 0;
 }
+
+
+static double nop_cost_function(struct starpu_task *t STARPU_ATTRIBUTE_UNUSED, struct starpu_perfmodel_arch *a STARPU_ATTRIBUTE_UNUSED, unsigned i STARPU_ATTRIBUTE_UNUSED)
+{
+	return 0.000001;
+}
+
+struct starpu_perfmodel starpu_perfmodel_nop =
+{
+	.type = STARPU_PER_ARCH,
+	.arch_cost_function = nop_cost_function,
+};
+
