@@ -1,8 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2014,2017                           Inria
- * Copyright (C) 2008-2018                                Université de Bordeaux
- * Copyright (C) 2010-2018                                CNRS
+ * Copyright (C) 2011-2014,2016,2017                      Inria
+ * Copyright (C) 2008-2019                                Université de Bordeaux
+ * Copyright (C) 2018                                     Federal University of Rio Grande do Sul (UFRGS)
+ * Copyright (C) 2010-2019                                CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,6 +17,9 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
+#include <limits.h>
+#include <math.h>
+
 #include <common/config.h>
 #include <datawizard/coherency.h>
 #include <datawizard/copy_driver.h>
@@ -24,7 +28,6 @@
 #include <core/dependencies/data_concurrency.h>
 #include <core/disk.h>
 #include <profiling/profiling.h>
-#include <math.h>
 #include <core/task.h>
 #include <starpu_scheduler.h>
 #include <core/workers.h>
@@ -104,7 +107,7 @@ int _starpu_select_src_node(starpu_data_handle_t handle, unsigned destination)
 		STARPU_ASSERT(handle->per_node[src_node].initialized);
 		return src_node;
 	}
-	
+
 	int i_ram = -1;
 	int i_gpu = -1;
 	int i_disk = -1;
@@ -112,7 +115,6 @@ int _starpu_select_src_node(starpu_data_handle_t handle, unsigned destination)
 	/* Revert to dumb strategy: take RAM unless only a GPU has it */
 	for (i = 0; i < nnodes; i++)
 	{
-		
 		if (src_node_mask & (1<<i))
 		{
 			int (*can_copy)(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, unsigned handling_node) = handle->ops->copy_methods->can_copy;
@@ -144,15 +146,14 @@ int _starpu_select_src_node(starpu_data_handle_t handle, unsigned destination)
 			    starpu_node_get_kind(i) == STARPU_MIC_RAM)
 				i_gpu = i;
 
-			if (starpu_node_get_kind(i) == STARPU_CPU_RAM || 
-			    starpu_node_get_kind(i) == STARPU_SCC_RAM ||
-			    starpu_node_get_kind(i) == STARPU_SCC_SHM)
+			if (starpu_node_get_kind(i) == STARPU_CPU_RAM ||
+                            starpu_node_get_kind(i) == STARPU_MPI_MS_RAM)
 				i_ram = i;
-			if (starpu_node_get_kind(i) == STARPU_DISK_RAM)			
+			if (starpu_node_get_kind(i) == STARPU_DISK_RAM)
 				i_disk = i;
 		}
 	}
-	
+
 	/* we have to use cpu_ram in first */
 	if (i_ram != -1)
 		src_node = i_ram;
@@ -190,10 +191,10 @@ void _starpu_update_data_state(starpu_data_handle_t handle,
 		unsigned node;
 		for (node = 0; node < nnodes; node++)
 		{
-			_STARPU_TRACE_DATA_INVALIDATE(handle, node);
+                       _STARPU_TRACE_DATA_STATE_INVALID(handle, node);
 			handle->per_node[node].state = STARPU_INVALID;
 		}
-
+               _STARPU_TRACE_DATA_STATE_OWNER(handle, requesting_node);
 		requesting_replicate->state = STARPU_OWNER;
 		if (handle->home_node != -1 && handle->per_node[handle->home_node].state == STARPU_INVALID)
 			/* Notify that this MC is now dirty */
@@ -209,9 +210,12 @@ void _starpu_update_data_state(starpu_data_handle_t handle,
 			for (node = 0; node < nnodes; node++)
 			{
 				struct _starpu_data_replicate *replicate = &handle->per_node[node];
-				if (replicate->state != STARPU_INVALID)
+                               if (replicate->state != STARPU_INVALID){
+                                       _STARPU_TRACE_DATA_STATE_SHARED(handle, node);
 					replicate->state = STARPU_SHARED;
+                               }
 			}
+                       _STARPU_TRACE_DATA_STATE_SHARED(handle, requesting_node);
 			requesting_replicate->state = STARPU_SHARED;
 		}
 	}
@@ -219,10 +223,6 @@ void _starpu_update_data_state(starpu_data_handle_t handle,
 
 static int worker_supports_direct_access(unsigned node, unsigned handling_node)
 {
-	/* only support disk <-> ram and disk <-> disk */
-	if (starpu_node_get_kind(node) == STARPU_DISK_RAM || starpu_node_get_kind(handling_node) == STARPU_DISK_RAM)
-		return 0;
-
 	if (node == handling_node)
 		return 1;
 
@@ -230,44 +230,13 @@ static int worker_supports_direct_access(unsigned node, unsigned handling_node)
 		/* No worker to process the request from that node */
 		return 0;
 
-	int type = starpu_node_get_kind(node);
-	switch (type)
+	struct _starpu_node_ops *node_ops = _starpu_memory_node_get_node_ops(node);
+	if (node_ops && node_ops->is_direct_access_supported)
+		return node_ops->is_direct_access_supported(node, handling_node);
+	else
 	{
-		case STARPU_CUDA_RAM:
-		{
-			/* GPUs not always allow direct remote access: if CUDA4
-			 * is enabled, we allow two CUDA devices to communicate. */
-#ifdef STARPU_SIMGRID
-			if (starpu_node_get_kind(handling_node) == STARPU_CUDA_RAM)
-			{
-				char name[16];
-				msg_host_t host;
-				const char* cuda_memcpy_peer;
-				snprintf(name, sizeof(name), "CUDA%d", _starpu_memory_node_get_devid(handling_node));
-				host = _starpu_simgrid_get_host_by_name(name);
-				cuda_memcpy_peer = MSG_host_get_property_value(host, "memcpy_peer");
-				return cuda_memcpy_peer && atoll(cuda_memcpy_peer);
-			}
-			else
-				return 0;
-#elif defined(STARPU_HAVE_CUDA_MEMCPY_PEER)
-			/* simgrid */
-			enum starpu_node_kind kind = starpu_node_get_kind(handling_node);
-			return kind == STARPU_CUDA_RAM;
-#else /* STARPU_HAVE_CUDA_MEMCPY_PEER */
-			/* Direct GPU-GPU transfers are not allowed in general */
-			return 0;
-#endif /* STARPU_HAVE_CUDA_MEMCPY_PEER */
-		}
-		case STARPU_OPENCL_RAM:
-			return 0;
-		case STARPU_MIC_RAM:
-			/* We don't handle direct MIC-MIC transfers yet */
-			return 0;
-		case STARPU_SCC_RAM:
-			return 1;
-		default:
-			return 1;
+		STARPU_ABORT_MSG("Node %s does not define the operation 'is_direct_access_supported'", _starpu_node_get_prefix(starpu_node_get_kind(node)));
+		return 1;
 	}
 }
 
@@ -302,27 +271,37 @@ static int link_supports_direct_transfers(starpu_data_handle_t handle, unsigned
 		return 1;
 	}
 
-	/* Link between disk and ram */
-	if ((starpu_node_get_kind(src_node) == STARPU_DISK_RAM && starpu_node_get_kind(dst_node) == STARPU_CPU_RAM) ||
-	    (starpu_node_get_kind(src_node) == STARPU_CPU_RAM && starpu_node_get_kind(dst_node) == STARPU_DISK_RAM))
-	{
-		/* FIXME: not necessarily a worker :/ */
-		*handling_node = STARPU_MAIN_RAM;
-		return 1;
-	}
-
-	/* link between disk and disk, and they have the same kind */
-	if (_starpu_is_same_kind_disk(src_node, dst_node))
-		return 1;
-
 	return 0;
 }
 
+/* Now, we use slowness/bandwidth to compare numa nodes, is it better to use latency ? */
+static unsigned chose_best_numa_between_src_and_dest(int src, int dst)
+{
+	double timing_best;
+	int best_numa = -1;
+	unsigned numa;
+	const unsigned nb_numa_nodes = starpu_memory_nodes_get_numa_count();
+	for(numa = 0; numa < nb_numa_nodes; numa++)
+	{
+		double actual = 1.0/starpu_transfer_bandwidth(src, numa) + 1.0/starpu_transfer_bandwidth(numa, dst);
+
+		/* Compare slowness : take the lowest */
+		if (best_numa < 0 || actual < timing_best)
+		{
+			best_numa = numa;
+			timing_best = actual;
+		}
+	}
+	STARPU_ASSERT(best_numa >= 0);
+
+	return best_numa;
+}
+
 /* Determines the path of a request : each hop is defined by (src,dst) and the
  * node that handles the hop. The returned value indicates the number of hops,
  * and the max_len is the maximum number of hops (ie. the size of the
  * src_nodes, dst_nodes and handling_nodes arrays. */
-static int determine_request_path(starpu_data_handle_t handle,
+int _starpu_determine_request_path(starpu_data_handle_t handle,
 				  int src_node, int dst_node,
 				  enum starpu_data_access_mode mode, int max_len,
 				  unsigned *src_nodes, unsigned *dst_nodes,
@@ -367,10 +346,13 @@ static int determine_request_path(starpu_data_handle_t handle,
 		/* We need an intermediate hop to implement data staging
 		 * through main memory. */
 		STARPU_ASSERT(max_len >= 2);
+		STARPU_ASSERT(src_node >= 0);
+
+		unsigned numa = chose_best_numa_between_src_and_dest(src_node, dst_node);
 
 		/* GPU -> RAM */
 		src_nodes[0] = src_node;
-		dst_nodes[0] = STARPU_MAIN_RAM;
+		dst_nodes[0] = numa;
 
 		if (starpu_node_get_kind(src_node) == STARPU_DISK_RAM)
 			/* Disks don't have their own driver thread */
@@ -386,7 +368,7 @@ static int determine_request_path(starpu_data_handle_t handle,
 		}
 
 		/* RAM -> GPU */
-		src_nodes[1] = STARPU_MAIN_RAM;
+		src_nodes[1] = numa;
 		dst_nodes[1] = dst_node;
 
 		if (starpu_node_get_kind(dst_node) == STARPU_DISK_RAM)
@@ -488,7 +470,7 @@ struct _starpu_data_request *_starpu_create_request_to_fetch_data(starpu_data_ha
 								  struct _starpu_data_replicate *dst_replicate,
 								  enum starpu_data_access_mode mode, unsigned is_prefetch,
 								  unsigned async,
-								  void (*callback_func)(void *), void *callback_arg)
+								  void (*callback_func)(void *), void *callback_arg, int prio, const char *origin)
 {
 	/* We don't care about commuting for data requests, that was handled before. */
 	mode &= ~STARPU_COMMUTE;
@@ -579,7 +561,7 @@ struct _starpu_data_request *_starpu_create_request_to_fetch_data(starpu_data_ha
 		/* if the data is in write only mode (and not SCRATCH or REDUX), there is no need for a source, data will be initialized by the task itself */
 		if (mode & STARPU_W)
 			dst_replicate->initialized = 1;
-		if (requesting_node == STARPU_MAIN_RAM && !nwait)
+		if (starpu_node_get_kind(requesting_node) == STARPU_CPU_RAM && !nwait)
 		{
 			/* And this is the main RAM, really no need for a
 			 * request, just allocate */
@@ -603,7 +585,7 @@ struct _starpu_data_request *_starpu_create_request_to_fetch_data(starpu_data_ha
 	unsigned src_nodes[MAX_REQUESTS], dst_nodes[MAX_REQUESTS], handling_nodes[MAX_REQUESTS];
 	/* keep one slot for the last W request, if any */
 	int write_invalidation = (mode & STARPU_W) && nwait && !is_prefetch;
-	int nhops = determine_request_path(handle, src_node, requesting_node, mode, MAX_REQUESTS,
+	int nhops = _starpu_determine_request_path(handle, src_node, requesting_node, mode, MAX_REQUESTS,
 					   src_nodes, dst_nodes, handling_nodes, write_invalidation);
 
 	STARPU_ASSERT(nhops >= 0 && nhops <= MAX_REQUESTS-1);
@@ -643,7 +625,7 @@ struct _starpu_data_request *_starpu_create_request_to_fetch_data(starpu_data_ha
 			/* Create a new request if there was no request to reuse */
 			r = _starpu_create_data_request(handle, hop_src_replicate,
 							hop_dst_replicate, hop_handling_node,
-							mode, ndeps, is_prefetch, 0);
+							mode, ndeps, is_prefetch, prio, 0, origin);
 			nwait++;
 		}
 
@@ -681,7 +663,7 @@ struct _starpu_data_request *_starpu_create_request_to_fetch_data(starpu_data_ha
 		 */
 		struct _starpu_data_request *r = _starpu_create_data_request(handle, dst_replicate,
 							dst_replicate, requesting_node,
-							STARPU_W, nwait, is_prefetch, 1);
+							STARPU_W, nwait, is_prefetch, prio, 1, origin);
 
 		/* and perform the callback after termination */
 		_starpu_data_request_append_callback(r, callback_func, callback_arg);
@@ -732,16 +714,15 @@ struct _starpu_data_request *_starpu_create_request_to_fetch_data(starpu_data_ha
 
 int _starpu_fetch_data_on_node(starpu_data_handle_t handle, int node, struct _starpu_data_replicate *dst_replicate,
 			       enum starpu_data_access_mode mode, unsigned detached, unsigned is_prefetch, unsigned async,
-			       void (*callback_func)(void *), void *callback_arg)
+			       void (*callback_func)(void *), void *callback_arg, int prio, const char *origin)
 {
-	unsigned local_node = _starpu_memory_node_get_local_key();
         _STARPU_LOG_IN();
 
 	int cpt = 0;
 	while (cpt < STARPU_SPIN_MAXTRY && _starpu_spin_trylock(&handle->header_lock))
 	{
 		cpt++;
-		_starpu_datawizard_progress(local_node, 1);
+		_starpu_datawizard_progress(1);
 	}
 	if (cpt == STARPU_SPIN_MAXTRY)
 		_starpu_spin_lock(&handle->header_lock);
@@ -751,12 +732,13 @@ int _starpu_fetch_data_on_node(starpu_data_handle_t handle, int node, struct _st
 		unsigned src_node_mask = 0;
 
 		unsigned nnodes = starpu_memory_nodes_get_count();
-		for (node = 0; node < nnodes; node++)
+		unsigned n;
+		for (n = 0; n < nnodes; n++)
 		{
-			if (handle->per_node[node].state != STARPU_INVALID)
+			if (handle->per_node[n].state != STARPU_INVALID)
 			{
 				/* we found a copy ! */
-				src_node_mask |= (1<<node);
+				src_node_mask |= (1<<n);
 			}
 		}
 
@@ -773,7 +755,7 @@ int _starpu_fetch_data_on_node(starpu_data_handle_t handle, int node, struct _st
 		/* Take references which will be released by _starpu_release_data_on_node */
 		if (dst_replicate)
 			dst_replicate->refcnt++;
-		else if (node == STARPU_ACQUIRE_ALL_NODES)
+		else if (node == STARPU_ACQUIRE_NO_NODE_LOCK_ALL)
 		{
 			int i;
 			for (i = 0; i < STARPU_MAXNODES; i++)
@@ -784,7 +766,7 @@ int _starpu_fetch_data_on_node(starpu_data_handle_t handle, int node, struct _st
 
 	struct _starpu_data_request *r;
 	r = _starpu_create_request_to_fetch_data(handle, dst_replicate, mode,
-						 is_prefetch, async, callback_func, callback_arg);
+						 is_prefetch, async, callback_func, callback_arg, prio, origin);
 
 	/* If no request was created, the handle was already up-to-date on the
 	 * node. In this case, _starpu_create_request_to_fetch_data has already
@@ -799,19 +781,19 @@ int _starpu_fetch_data_on_node(starpu_data_handle_t handle, int node, struct _st
         return ret;
 }
 
-static int idle_prefetch_data_on_node(starpu_data_handle_t handle, int node, struct _starpu_data_replicate *replicate, enum starpu_data_access_mode mode)
+static int idle_prefetch_data_on_node(starpu_data_handle_t handle, int node, struct _starpu_data_replicate *replicate, enum starpu_data_access_mode mode, int prio)
 {
-	return _starpu_fetch_data_on_node(handle, node, replicate, mode, 1, 2, 1, NULL, NULL);
+	return _starpu_fetch_data_on_node(handle, node, replicate, mode, 1, 2, 1, NULL, NULL, prio, "idle_prefetch_data_on_node");
 }
 
-static int prefetch_data_on_node(starpu_data_handle_t handle, int node, struct _starpu_data_replicate *replicate, enum starpu_data_access_mode mode)
+static int prefetch_data_on_node(starpu_data_handle_t handle, int node, struct _starpu_data_replicate *replicate, enum starpu_data_access_mode mode, int prio)
 {
-	return _starpu_fetch_data_on_node(handle, node, replicate, mode, 1, 1, 1, NULL, NULL);
+	return _starpu_fetch_data_on_node(handle, node, replicate, mode, 1, 1, 1, NULL, NULL, prio, "prefetch_data_on_node");
 }
 
-static int fetch_data(starpu_data_handle_t handle, int node, struct _starpu_data_replicate *replicate, enum starpu_data_access_mode mode)
+static int fetch_data(starpu_data_handle_t handle, int node, struct _starpu_data_replicate *replicate, enum starpu_data_access_mode mode, int prio)
 {
-	return _starpu_fetch_data_on_node(handle, node, replicate, mode, 0, 0, 0, NULL, NULL);
+	return _starpu_fetch_data_on_node(handle, node, replicate, mode, 0, 0, 0, NULL, NULL, prio, "fetch_data");
 }
 
 uint32_t _starpu_get_data_refcnt(starpu_data_handle_t handle, unsigned node)
@@ -824,6 +806,14 @@ size_t _starpu_data_get_size(starpu_data_handle_t handle)
 	return handle->ops->get_size(handle);
 }
 
+size_t _starpu_data_get_alloc_size(starpu_data_handle_t handle)
+{
+	if (handle->ops->get_alloc_size)
+		return handle->ops->get_alloc_size(handle);
+	else
+		return handle->ops->get_size(handle);
+}
+
 uint32_t _starpu_data_get_footprint(starpu_data_handle_t handle)
 {
 	return handle->footprint;
@@ -847,12 +837,11 @@ void _starpu_release_data_on_node(starpu_data_handle_t handle, uint32_t default_
 	if (wt_mask & ~(1<<memory_node))
 		_starpu_write_through_data(handle, memory_node, wt_mask);
 
-	unsigned local_node = _starpu_memory_node_get_local_key();
 	int cpt = 0;
 	while (cpt < STARPU_SPIN_MAXTRY && _starpu_spin_trylock(&handle->header_lock))
 	{
 		cpt++;
-		_starpu_datawizard_progress(local_node, 1);
+		_starpu_datawizard_progress(1);
 	}
 	if (cpt == STARPU_SPIN_MAXTRY)
 		_starpu_spin_lock(&handle->header_lock);
@@ -870,12 +859,11 @@ void _starpu_release_data_on_node(starpu_data_handle_t handle, uint32_t default_
 
 static void _starpu_set_data_requested_flag_if_needed(starpu_data_handle_t handle, struct _starpu_data_replicate *replicate)
 {
-	unsigned local_node = _starpu_memory_node_get_local_key();
 	int cpt = 0;
 	while (cpt < STARPU_SPIN_MAXTRY && _starpu_spin_trylock(&handle->header_lock))
 	{
 		cpt++;
-		_starpu_datawizard_progress(local_node, 1);
+		_starpu_datawizard_progress(1);
 	}
 	if (cpt == STARPU_SPIN_MAXTRY)
 		_starpu_spin_lock(&handle->header_lock);
@@ -889,8 +877,14 @@ static void _starpu_set_data_requested_flag_if_needed(starpu_data_handle_t handl
 	_starpu_spin_unlock(&handle->header_lock);
 }
 
-int starpu_prefetch_task_input_on_node(struct starpu_task *task, unsigned target_node)
+int starpu_prefetch_task_input_on_node_prio(struct starpu_task *task, unsigned target_node, int prio)
 {
+#ifdef STARPU_OPENMP
+	struct _starpu_job *j = _starpu_get_job_associated_to_task(task);
+	/* do not attempt to prefetch task input if this is an OpenMP task resuming after blocking */
+	if (j->discontinuous != 0)
+		return 0;
+#endif
 	STARPU_ASSERT(!task->prefetched);
 	unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(task);
 	unsigned index;
@@ -899,11 +893,6 @@ int starpu_prefetch_task_input_on_node(struct starpu_task *task, unsigned target
 	{
 		starpu_data_handle_t handle = STARPU_TASK_GET_HANDLE(task, index);
 		enum starpu_data_access_mode mode = STARPU_TASK_GET_MODE(task, index);
-		int node = -1;
-		if (task->cl->specific_nodes)
-			node = STARPU_CODELET_GET_NODE(task->cl, index);
-		if (node == -1)
-			node = target_node;
 
 		if (mode & (STARPU_SCRATCH|STARPU_REDUX))
 			continue;
@@ -912,8 +901,10 @@ int starpu_prefetch_task_input_on_node(struct starpu_task *task, unsigned target
 			/* Don't bother prefetching some data which will be overwritten */
 			continue;
 
+		int node = _starpu_task_data_get_node_on_node(task, index, target_node);
+
 		struct _starpu_data_replicate *replicate = &handle->per_node[node];
-		prefetch_data_on_node(handle, node, replicate, mode);
+		prefetch_data_on_node(handle, node, replicate, mode, prio);
 
 		_starpu_set_data_requested_flag_if_needed(handle, replicate);
 	}
@@ -921,7 +912,15 @@ int starpu_prefetch_task_input_on_node(struct starpu_task *task, unsigned target
 	return 0;
 }
 
-int starpu_idle_prefetch_task_input_on_node(struct starpu_task *task, unsigned target_node)
+int starpu_prefetch_task_input_on_node(struct starpu_task *task, unsigned node)
+{
+	int prio = task->priority;
+	if (task->workerorder)
+		prio = INT_MAX - task->workerorder;
+	return starpu_prefetch_task_input_on_node_prio(task, node, prio);
+}
+
+int starpu_idle_prefetch_task_input_on_node_prio(struct starpu_task *task, unsigned target_node, int prio)
 {
 	unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(task);
 	unsigned index;
@@ -930,11 +929,6 @@ int starpu_idle_prefetch_task_input_on_node(struct starpu_task *task, unsigned t
 	{
 		starpu_data_handle_t handle = STARPU_TASK_GET_HANDLE(task, index);
 		enum starpu_data_access_mode mode = STARPU_TASK_GET_MODE(task, index);
-		int node = -1;
-		if (task->cl->specific_nodes)
-			node = STARPU_CODELET_GET_NODE(task->cl, index);
-		if (node == -1)
-			node = target_node;
 
 		if (mode & (STARPU_SCRATCH|STARPU_REDUX))
 			continue;
@@ -943,15 +937,101 @@ int starpu_idle_prefetch_task_input_on_node(struct starpu_task *task, unsigned t
 			/* Don't bother prefetching some data which will be overwritten */
 			continue;
 
+		int node = _starpu_task_data_get_node_on_node(task, index, target_node);
+
 		struct _starpu_data_replicate *replicate = &handle->per_node[node];
-		idle_prefetch_data_on_node(handle, node, replicate, mode);
+		idle_prefetch_data_on_node(handle, node, replicate, mode, prio);
 	}
 
 	return 0;
 }
 
+int starpu_idle_prefetch_task_input_on_node(struct starpu_task *task, unsigned node)
+{
+	int prio = task->priority;
+	if (task->workerorder)
+		prio = INT_MAX - task->workerorder;
+	return starpu_idle_prefetch_task_input_on_node_prio(task, node, prio);
+}
 
-static struct _starpu_data_replicate *get_replicate(starpu_data_handle_t handle, enum starpu_data_access_mode mode, int workerid, unsigned node)
+int starpu_prefetch_task_input_for_prio(struct starpu_task *task, unsigned worker, int prio)
+{
+#ifdef STARPU_OPENMP
+	struct _starpu_job *j = _starpu_get_job_associated_to_task(task);
+	/* do not attempt to prefetch task input if this is an OpenMP task resuming after blocking */
+	if (j->discontinuous != 0)
+		return 0;
+#endif
+	STARPU_ASSERT(!task->prefetched);
+	unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(task);
+	unsigned index;
+
+	for (index = 0; index < nbuffers; index++)
+	{
+		starpu_data_handle_t handle = STARPU_TASK_GET_HANDLE(task, index);
+		enum starpu_data_access_mode mode = STARPU_TASK_GET_MODE(task, index);
+
+		if (mode & (STARPU_SCRATCH|STARPU_REDUX))
+			continue;
+
+		if (!(mode & STARPU_R))
+			/* Don't bother prefetching some data which will be overwritten */
+			continue;
+
+		int node = _starpu_task_data_get_node_on_worker(task, index, worker);
+
+		struct _starpu_data_replicate *replicate = &handle->per_node[node];
+		prefetch_data_on_node(handle, node, replicate, mode, prio);
+
+		_starpu_set_data_requested_flag_if_needed(handle, replicate);
+	}
+
+	return 0;
+}
+
+int starpu_prefetch_task_input_for(struct starpu_task *task, unsigned worker)
+{
+	int prio = task->priority;
+	if (task->workerorder)
+		prio = INT_MAX - task->workerorder;
+	return starpu_prefetch_task_input_for_prio(task, worker, prio);
+}
+
+int starpu_idle_prefetch_task_input_for_prio(struct starpu_task *task, unsigned worker, int prio)
+{
+	unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(task);
+	unsigned index;
+
+	for (index = 0; index < nbuffers; index++)
+	{
+		starpu_data_handle_t handle = STARPU_TASK_GET_HANDLE(task, index);
+		enum starpu_data_access_mode mode = STARPU_TASK_GET_MODE(task, index);
+
+		if (mode & (STARPU_SCRATCH|STARPU_REDUX))
+			continue;
+
+		if (!(mode & STARPU_R))
+			/* Don't bother prefetching some data which will be overwritten */
+			continue;
+
+		int node = _starpu_task_data_get_node_on_worker(task, index, worker);
+
+		struct _starpu_data_replicate *replicate = &handle->per_node[node];
+		idle_prefetch_data_on_node(handle, node, replicate, mode, prio);
+	}
+
+	return 0;
+}
+
+int starpu_idle_prefetch_task_input_for(struct starpu_task *task, unsigned worker)
+{
+	int prio = task->priority;
+	if (task->workerorder)
+		prio = INT_MAX - task->workerorder;
+	return starpu_idle_prefetch_task_input_for_prio(task, worker, prio);
+}
+
+struct _starpu_data_replicate *get_replicate(starpu_data_handle_t handle, enum starpu_data_access_mode mode, int workerid, unsigned node)
 {
 	if (mode & (STARPU_SCRATCH|STARPU_REDUX))
 	{
@@ -970,36 +1050,65 @@ static struct _starpu_data_replicate *get_replicate(starpu_data_handle_t handle,
 		return &handle->per_node[node];
 }
 
-/* Synchronously fetch data for a given task (if it's not there already) */
-int _starpu_fetch_task_input(struct _starpu_job *j)
+/* Callback used when a buffer is send asynchronously to the sink */
+static void _starpu_fetch_task_input_cb(void *arg)
 {
-	_STARPU_TRACE_START_FETCH_INPUT(NULL);
+   struct _starpu_worker * worker = (struct _starpu_worker *) arg;
+
+   /* increase the number of buffer received */
+   STARPU_WMB();
+   (void)STARPU_ATOMIC_ADD(&worker->nb_buffers_transferred, 1);
+
+#ifdef STARPU_SIMGRID
+   starpu_pthread_queue_broadcast(&_starpu_simgrid_transfer_queue[worker->memory_node]);
+#endif
+}
+
+
+/* Synchronously or asynchronously fetch data for a given task (if it's not there already)
+ * Returns the number of data acquired here.  */
+
+/* _starpu_fetch_task_input must be called before
+ * executing the task. __starpu_push_task_output but be called after the
+ * execution of the task. */
+
+/* The driver can either just call _starpu_fetch_task_input with async==0,
+ * or to improve overlapping, it can call _starpu_fetch_task_input with
+ * async==1, then wait for transfers to complete, then call
+ * _starpu_fetch_task_input_tail to complete the fetch.  */
+int _starpu_fetch_task_input(struct starpu_task *task, struct _starpu_job *j, int async)
+{
+	struct _starpu_worker *worker = _starpu_get_local_worker_key();
+	int workerid = worker->workerid;
+	if (async)
+	{
+		worker->task_transferring = task;
+		worker->nb_buffers_transferred = 0;
+		if (worker->ntasks <= 1)
+			_STARPU_TRACE_WORKER_START_FETCH_INPUT(NULL, workerid);
+	}
+	else
+		_STARPU_TRACE_START_FETCH_INPUT(NULL);
 
 	int profiling = starpu_profiling_status_get();
-	struct starpu_task *task = j->task;
 	if (profiling && task->profiling_info)
 		_starpu_clock_gettime(&task->profiling_info->acquire_data_start_time);
 
 	struct _starpu_data_descr *descrs = _STARPU_JOB_GET_ORDERED_BUFFERS(j);
 	unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(task);
-
-	unsigned local_memory_node = _starpu_memory_node_get_local_key();
-
-	int workerid = starpu_worker_get_id();
-
-#ifdef STARPU_USE_FXT
-	unsigned long total_size = 0;
-#endif
+	unsigned nacquires;
 
 	unsigned index;
+
+	nacquires = 0;
 	for (index = 0; index < nbuffers; index++)
 	{
 		int ret;
 		starpu_data_handle_t handle = descrs[index].handle;
 		enum starpu_data_access_mode mode = descrs[index].mode;
-		int node = descrs[index].node;
-		if (node == -1)
-			node = local_memory_node;
+		int node = _starpu_task_data_get_node_on_worker(task, descrs[index].index, workerid);
+		/* We set this here for coherency with __starpu_push_task_output */
+		descrs[index].node = node;
 		if (mode == STARPU_NONE ||
 			(mode & ((1<<STARPU_MODE_SHIFT) - 1)) >= STARPU_ACCESS_MODE_MAX ||
 			(mode >> STARPU_MODE_SHIFT) >= (STARPU_SHIFTED_MODE_MAX >> STARPU_MODE_SHIFT))
@@ -1015,42 +1124,43 @@ int _starpu_fetch_task_input(struct _starpu_job *j)
 
 		local_replicate = get_replicate(handle, mode, workerid, node);
 
-		ret = fetch_data(handle, node, local_replicate, mode);
-		if (STARPU_UNLIKELY(ret))
-			goto enomem;
-
-#ifdef STARPU_USE_FXT
-		total_size += _starpu_data_get_size(handle);
+		if (async)
+		{
+			ret = _starpu_fetch_data_on_node(handle, node, local_replicate, mode, 0, 0, 1,
+					_starpu_fetch_task_input_cb, worker, 0, "_starpu_fetch_task_input");
+#ifdef STARPU_SIMGRID
+			if (_starpu_simgrid_fetching_input_cost())
+				MSG_process_sleep(0.000001);
 #endif
-	}
+			if (STARPU_UNLIKELY(ret))
+			{
+				/* Ooops, not enough memory, make worker wait for these for now, and the synchronous call will finish by forcing eviction*/
+				worker->nb_buffers_totransfer = nacquires;
+				_starpu_set_worker_status(worker, STATUS_WAITING);
+				return 0;
+			}
+		}
+		else
+		{
+			ret = fetch_data(handle, node, local_replicate, mode, 0);
+#ifdef STARPU_SIMGRID
+			if (_starpu_simgrid_fetching_input_cost())
+				MSG_process_sleep(0.000001);
+#endif
+			if (STARPU_UNLIKELY(ret))
+				goto enomem;
+		}
 
-	_STARPU_TRACE_DATA_LOAD(workerid,total_size);
-	/* Now that we have taken the data locks in locking order, fill the codelet interfaces in function order.  */
-	for (index = 0; index < nbuffers; index++)
+		nacquires++;
+	}
+	if (async)
 	{
-		starpu_data_handle_t handle = STARPU_TASK_GET_HANDLE(task, index);
-		enum starpu_data_access_mode mode = STARPU_TASK_GET_MODE(task, index);
-		int node = -1;
-		if (task->cl->specific_nodes)
-			node = STARPU_CODELET_GET_NODE(task->cl, index);
-		if (node == -1)
-			node = local_memory_node;
-
-		struct _starpu_data_replicate *local_replicate;
-
-		local_replicate = get_replicate(handle, mode, workerid, node);
-
-		_STARPU_TASK_SET_INTERFACE(task , local_replicate->data_interface, index);
-
-		/* If the replicate was not initialized yet, we have to do it now */
-		if (!(mode & STARPU_SCRATCH) && !local_replicate->initialized)
-			_starpu_redux_init_data_replicate(handle, local_replicate, workerid);
+		worker->nb_buffers_totransfer = nacquires;
+		_starpu_set_worker_status(worker, STATUS_WAITING);
+		return 0;
 	}
 
-	if (profiling && task->profiling_info)
-		_starpu_clock_gettime(&task->profiling_info->acquire_data_end_time);
-
-	_STARPU_TRACE_END_FETCH_INPUT(NULL);
+	_starpu_fetch_task_input_tail(task, j, worker);
 
 	return 0;
 
@@ -1065,8 +1175,6 @@ enomem:
 		starpu_data_handle_t handle = descrs[index2].handle;
 		enum starpu_data_access_mode mode = descrs[index2].mode;
 		int node = descrs[index].node;
-		if (node == -1)
-			node = local_memory_node;
 
 		struct _starpu_data_replicate *local_replicate;
 
@@ -1084,6 +1192,51 @@ enomem:
 	return -1;
 }
 
+/* Now that we have taken the data locks in locking order, fill the codelet interfaces in function order.  */
+void _starpu_fetch_task_input_tail(struct starpu_task *task, struct _starpu_job *j, struct _starpu_worker *worker)
+{
+	int workerid = worker->workerid;
+
+	int profiling = starpu_profiling_status_get();
+
+	unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(task);
+	struct _starpu_data_descr *descrs = _STARPU_JOB_GET_ORDERED_BUFFERS(j);
+
+	unsigned index;
+	unsigned long total_size = 0;
+
+	for (index = 0; index < nbuffers; index++)
+	{
+		starpu_data_handle_t handle = descrs[index].handle;
+		enum starpu_data_access_mode mode = descrs[index].mode;
+		int node = descrs[index].node;
+
+		struct _starpu_data_replicate *local_replicate;
+
+		local_replicate = get_replicate(handle, mode, workerid, node);
+		_starpu_spin_lock(&handle->header_lock);
+		if (local_replicate->mc)
+			local_replicate->mc->diduse = 1;
+		_starpu_spin_unlock(&handle->header_lock);
+
+		_STARPU_TASK_SET_INTERFACE(task , local_replicate->data_interface, descrs[index].index);
+
+		/* If the replicate was not initialized yet, we have to do it now */
+		if (!(mode & STARPU_SCRATCH) && !local_replicate->initialized)
+			_starpu_redux_init_data_replicate(handle, local_replicate, workerid);
+
+#ifdef STARPU_USE_FXT
+		total_size += _starpu_data_get_size(handle);
+#endif
+	}
+	_STARPU_TRACE_DATA_LOAD(workerid,total_size);
+
+	if (profiling && task->profiling_info)
+		_starpu_clock_gettime(&task->profiling_info->acquire_data_end_time);
+
+	_STARPU_TRACE_END_FETCH_INPUT(NULL);
+}
+
 /* Release task data dependencies */
 void __starpu_push_task_output(struct _starpu_job *j)
 {
@@ -1099,7 +1252,6 @@ void __starpu_push_task_output(struct _starpu_job *j)
         unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(task);
 
 	int workerid = starpu_worker_get_id();
-	unsigned local_memory_node = _starpu_memory_node_get_local_key();
 
 	unsigned index;
 	for (index = 0; index < nbuffers; index++)
@@ -1107,8 +1259,6 @@ void __starpu_push_task_output(struct _starpu_job *j)
 		starpu_data_handle_t handle = descrs[index].handle;
 		enum starpu_data_access_mode mode = descrs[index].mode;
 		int node = descrs[index].node;
-		if (node == -1 && task->cl->where != STARPU_NOWHERE)
-			node = local_memory_node;
 
 		struct _starpu_data_replicate *local_replicate = NULL;
 
@@ -1174,7 +1324,11 @@ void _starpu_fetch_nowhere_task_input(struct _starpu_job *j)
 	unsigned index;
 	for (index = 0; index < nbuffers; index++)
 	{
-		int node = descrs[index].node;
+		/* Note here we just follow what was requested, and not use _starpu_task_data_get_node* */
+		int node = -1;
+		if (task->cl->specific_nodes)
+			node = STARPU_CODELET_GET_NODE(task->cl, descrs[index].index);
+		descrs[index].node = node;
 		if (node != -1)
 			nfetchbuffers++;
 	}
@@ -1188,7 +1342,7 @@ void _starpu_fetch_nowhere_task_input(struct _starpu_job *j)
 		return;
 	}
 
-	_STARPU_MALLOC(wrapper, sizeof(*wrapper));
+	_STARPU_MALLOC(wrapper, (sizeof(*wrapper)));
 	wrapper->j = j;
 	/* +1 for the call below */
 	wrapper->pending = nfetchbuffers + 1;
@@ -1211,7 +1365,7 @@ void _starpu_fetch_nowhere_task_input(struct _starpu_job *j)
 
 		local_replicate = get_replicate(handle, mode, -1, node);
 
-		_starpu_fetch_data_on_node(handle, node, local_replicate, mode, 0, 0, 1, _starpu_fetch_nowhere_task_input_cb, wrapper);
+		_starpu_fetch_data_on_node(handle, node, local_replicate, mode, 0, 0, 1, _starpu_fetch_nowhere_task_input_cb, wrapper, 0, "_starpu_fetch_nowhere_task_input");
 	}
 
 	if (profiling && task->profiling_info)
@@ -1243,7 +1397,7 @@ static void _starpu_fetch_nowhere_task_input_cb(void *arg)
 
 /* NB : this value can only be an indication of the status of a data
 	at some point, but there is no strong garantee ! */
-unsigned _starpu_is_data_present_or_requested(starpu_data_handle_t handle, unsigned node)
+unsigned starpu_data_is_on_node(starpu_data_handle_t handle, unsigned node)
 {
 	unsigned ret = 0;
 
@@ -1271,8 +1425,3 @@ unsigned _starpu_is_data_present_or_requested(starpu_data_handle_t handle, unsig
 
 	return ret;
 }
-
-void _starpu_data_set_unregister_hook(starpu_data_handle_t handle, _starpu_data_handle_unregister_hook func)
-{
-	handle->unregister_hook = func;
-}
