@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013-2014, 2016  Université Bordeaux
- * Copyright (C) 2012 INRIA
+ * Copyright (C) 2012,2017                                Inria
+ * Copyright (C) 2012-2015,2017,2019                      CNRS
+ * Copyright (C) 2012-2018                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,8 +17,6 @@
  */
 
 #include <starpu.h>
-#include <common/fxt.h>
-#include <datawizard/memalloc.h>
 
 static int
 copy_any_to_any(void *src_interface, unsigned src_node,
@@ -51,9 +50,9 @@ copy_any_to_any(void *src_interface, unsigned src_node,
 		size, async_data))
 		ret = -EAGAIN;
 
-	_STARPU_TRACE_DATA_COPY(src_node, dst_node,
-		src_coo->n_values *
-		(2 * sizeof(src_coo->rows[0]) + src_coo->elemsize));
+	starpu_interface_data_copy(src_node, dst_node,
+				   src_coo->n_values *
+				   (2 * sizeof(src_coo->rows[0]) + src_coo->elemsize));
 
 	return ret;
 }
@@ -101,9 +100,9 @@ register_coo_handle(starpu_data_handle_t handle, unsigned home_node,
 static starpu_ssize_t
 allocate_coo_buffer_on_node(void *data_interface, unsigned dst_node)
 {
-	uint32_t *addr_columns = NULL;
-	uint32_t *addr_rows = NULL;
-	uintptr_t addr_values = 0;
+	uint32_t *addr_columns;
+	uint32_t *addr_rows;
+	uintptr_t addr_values;
 
 	struct starpu_coo_interface *coo_interface =
 		(struct starpu_coo_interface *) data_interface;
@@ -147,6 +146,20 @@ free_coo_buffer_on_node(void *data_interface, unsigned node)
 	starpu_free_on_node(node, coo_interface->values, n_values * elemsize);
 }
 
+static int
+coo_pointer_is_inside(void *data_interface, unsigned node, void *ptr)
+{
+	struct starpu_coo_interface *coo_interface = (struct starpu_coo_interface *) data_interface;
+	(void) node;
+
+	return ((char*) ptr >= (char*) coo_interface->columns &&
+		(char*) ptr < (char*) coo_interface->columns + coo_interface->n_values * sizeof(coo_interface->columns[0]))
+	    || ((char*) ptr >= (char*) coo_interface->rows &&
+		(char*) ptr < (char*) coo_interface->rows + coo_interface->n_values * sizeof(coo_interface->rows[0]))
+	    || ((char*) ptr >= (char*) coo_interface->values &&
+		(char*) ptr < (char*) coo_interface->values + coo_interface->n_values * coo_interface->elemsize);
+}
+
 static size_t
 coo_interface_get_size(starpu_data_handle_t handle)
 {
@@ -175,10 +188,10 @@ coo_compare(void *a, void *b)
 	coo_a = (struct starpu_coo_interface *) a;
 	coo_b = (struct starpu_coo_interface *) b;
 
-	return (coo_a->nx == coo_b->nx &&
+	return coo_a->nx == coo_b->nx &&
 		coo_a->ny == coo_b->ny &&
 		coo_a->n_values == coo_b->n_values &&
-		coo_a->elemsize == coo_b->elemsize);
+		coo_a->elemsize == coo_b->elemsize;
 }
 
 static void
@@ -205,7 +218,8 @@ struct starpu_data_interface_ops starpu_interface_coo_ops =
 {
 	.register_data_handle  = register_coo_handle,
 	.allocate_data_on_node = allocate_coo_buffer_on_node,
-	.handle_to_pointer     = NULL,
+	.to_pointer            = NULL,
+	.pointer_is_inside     = coo_pointer_is_inside,
 	.free_data_on_node     = free_coo_buffer_on_node,
 	.copy_methods          = &coo_copy_data_methods,
 	.get_size              = coo_interface_get_size,
@@ -214,7 +228,8 @@ struct starpu_data_interface_ops starpu_interface_coo_ops =
 	.interfaceid           = STARPU_COO_INTERFACE_ID,
 	.interface_size        = sizeof(struct starpu_coo_interface),
 	.display               = display_coo_interface,
-	.describe              = describe
+	.describe              = describe,
+	.name                  = "STARPU_COO_INTERFACE"
 };
 
 void
@@ -235,7 +250,7 @@ starpu_coo_data_register(starpu_data_handle_t *handleptr, int home_node,
 		.elemsize = elemsize,
 	};
 #ifndef STARPU_SIMGRID
-	if (home_node == STARPU_MAIN_RAM)
+	if (home_node >= 0 && starpu_node_get_kind(home_node) == STARPU_CPU_RAM)
 	{
 		STARPU_ASSERT_ACCESSIBLE(columns);
 		STARPU_ASSERT_ACCESSIBLE((uintptr_t) columns + n_values*sizeof(uint32_t) - 1);
