@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2012,2014,2017                      Inria
- * Copyright (C) 2008-2016, 2018                                Université de Bordeaux
- * Copyright (C) 2010-2015,2017                           CNRS
+ * Copyright (C) 2008-2019                                Université de Bordeaux
+ * Copyright (C) 2011,2012,2014,2017                      Inria
+ * Copyright (C) 2010-2015,2017,2019                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -17,17 +17,6 @@
  */
 
 #include <starpu.h>
-#include <common/config.h>
-#include <datawizard/coherency.h>
-#include <datawizard/copy_driver.h>
-#include <datawizard/filters.h>
-#include <datawizard/memory_nodes.h>
-#include <starpu_hash.h>
-#include <starpu_cuda.h>
-#include <starpu_opencl.h>
-#include <drivers/opencl/driver_opencl.h>
-#include <drivers/mic/driver_mic_source.h>
-#include <drivers/scc/driver_scc_source.h>
 
 static int copy_any_to_any(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, void *async_data);
 
@@ -36,14 +25,18 @@ static const struct starpu_data_copy_methods vector_copy_data_methods_s =
 	.any_to_any = copy_any_to_any,
 };
 
+static void vector_init(void *data_interface);
 static void register_vector_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface);
 static starpu_ssize_t allocate_vector_buffer_on_node(void *data_interface_, unsigned dst_node);
-static void *vector_handle_to_pointer(starpu_data_handle_t data_handle, unsigned node);
+static void *vector_to_pointer(void *data_interface, unsigned node);
 static int vector_pointer_is_inside(void *data_interface, unsigned node, void *ptr);
 static void free_vector_buffer_on_node(void *data_interface, unsigned node);
 static size_t vector_interface_get_size(starpu_data_handle_t handle);
+static size_t vector_interface_get_alloc_size(starpu_data_handle_t handle);
 static uint32_t footprint_vector_interface_crc32(starpu_data_handle_t handle);
+static uint32_t alloc_footprint_vector_interface_crc32(starpu_data_handle_t handle);
 static int vector_compare(void *data_interface_a, void *data_interface_b);
+static int vector_alloc_compare(void *data_interface_a, void *data_interface_b);
 static void display_vector_interface(starpu_data_handle_t handle, FILE *f);
 static int pack_vector_handle(starpu_data_handle_t handle, unsigned node, void **ptr, starpu_ssize_t *count);
 static int unpack_vector_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count);
@@ -51,15 +44,19 @@ static starpu_ssize_t describe(void *data_interface, char *buf, size_t size);
 
 struct starpu_data_interface_ops starpu_interface_vector_ops =
 {
+	.init = vector_init,
 	.register_data_handle = register_vector_handle,
 	.allocate_data_on_node = allocate_vector_buffer_on_node,
-	.handle_to_pointer = vector_handle_to_pointer,
+	.to_pointer = vector_to_pointer,
 	.pointer_is_inside = vector_pointer_is_inside,
 	.free_data_on_node = free_vector_buffer_on_node,
 	.copy_methods = &vector_copy_data_methods_s,
 	.get_size = vector_interface_get_size,
+	.get_alloc_size = vector_interface_get_alloc_size,
 	.footprint = footprint_vector_interface_crc32,
+	.alloc_footprint = alloc_footprint_vector_interface_crc32,
 	.compare = vector_compare,
+	.alloc_compare = vector_alloc_compare,
 	.interfaceid = STARPU_VECTOR_INTERFACE_ID,
 	.interface_size = sizeof(struct starpu_vector_interface),
 	.display = display_vector_interface,
@@ -69,12 +66,16 @@ struct starpu_data_interface_ops starpu_interface_vector_ops =
 	.name = "STARPU_VECTOR_INTERFACE"
 };
 
-static void *vector_handle_to_pointer(starpu_data_handle_t handle, unsigned node)
+static void vector_init(void *data_interface)
 {
-	STARPU_ASSERT(starpu_data_test_if_allocated_on_node(handle, node));
+	struct starpu_vector_interface *vector_interface = data_interface;
+	vector_interface->allocsize = -1;
+}
 
-	struct starpu_vector_interface *vector_interface = (struct starpu_vector_interface *)
-		starpu_data_get_interface_on_node(handle, node);
+static void *vector_to_pointer(void *data_interface, unsigned node)
+{
+	(void) node;
+	struct starpu_vector_interface *vector_interface = data_interface;
 
 	return (void*) vector_interface->ptr;
 }
@@ -114,13 +115,14 @@ static void register_vector_handle(starpu_data_handle_t handle, unsigned home_no
 		local_interface->id = vector_interface->id;
 		local_interface->nx = vector_interface->nx;
 		local_interface->elemsize = vector_interface->elemsize;
+		local_interface->allocsize = vector_interface->allocsize;
 		local_interface->slice_base = vector_interface->slice_base;
 	}
 }
 
 /* declare a new data with the vector interface */
-void starpu_vector_data_register(starpu_data_handle_t *handleptr, int home_node,
-                        uintptr_t ptr, uint32_t nx, size_t elemsize)
+void starpu_vector_data_register_allocsize(starpu_data_handle_t *handleptr, int home_node,
+					   uintptr_t ptr, uint32_t nx, size_t elemsize, size_t allocsize)
 {
 	struct starpu_vector_interface vector =
 	{
@@ -130,23 +132,26 @@ void starpu_vector_data_register(starpu_data_handle_t *handleptr, int home_node,
 		.elemsize = elemsize,
                 .dev_handle = ptr,
 		.slice_base = 0,
-                .offset = 0
+                .offset = 0,
+		.allocsize = allocsize,
 	};
 #if (!defined(STARPU_SIMGRID) && !defined(STARPU_OPENMP))
-	if (home_node == STARPU_MAIN_RAM)
+	if (home_node >= 0 && starpu_node_get_kind(home_node) == STARPU_CPU_RAM)
 	{
 		STARPU_ASSERT_ACCESSIBLE(ptr);
 		STARPU_ASSERT_ACCESSIBLE(ptr + nx*elemsize - 1);
 	}
 #endif
 
-#ifdef STARPU_USE_SCC
-	_starpu_scc_set_offset_in_shared_memory((void*)vector.ptr, (void**)&(vector.dev_handle), &(vector.offset));
-#endif
-
 	starpu_data_register(handleptr, home_node, &vector, &starpu_interface_vector_ops);
 }
 
+void starpu_vector_data_register(starpu_data_handle_t *handleptr, int home_node,
+				 uintptr_t ptr, uint32_t nx, size_t elemsize)
+{
+	starpu_vector_data_register_allocsize(handleptr, home_node, ptr, nx, elemsize, nx * elemsize);
+}
+
 void starpu_vector_ptr_register(starpu_data_handle_t handle, unsigned node,
 			uintptr_t ptr, uintptr_t dev_handle, size_t offset)
 {
@@ -163,6 +168,11 @@ static uint32_t footprint_vector_interface_crc32(starpu_data_handle_t handle)
 	return starpu_hash_crc32c_be(starpu_vector_get_nx(handle), 0);
 }
 
+static uint32_t alloc_footprint_vector_interface_crc32(starpu_data_handle_t handle)
+{
+	return starpu_hash_crc32c_be(starpu_vector_get_allocsize(handle), 0);
+}
+
 static int vector_compare(void *data_interface_a, void *data_interface_b)
 {
 	struct starpu_vector_interface *vector_a = (struct starpu_vector_interface *) data_interface_a;
@@ -173,6 +183,15 @@ static int vector_compare(void *data_interface_a, void *data_interface_b)
 		&& (vector_a->elemsize == vector_b->elemsize);
 }
 
+static int vector_alloc_compare(void *data_interface_a, void *data_interface_b)
+{
+	struct starpu_vector_interface *vector_a = (struct starpu_vector_interface *) data_interface_a;
+	struct starpu_vector_interface *vector_b = (struct starpu_vector_interface *) data_interface_b;
+
+	/* Two vectors are considered compatible if they have the same size */
+	return (vector_a->allocsize == vector_b->allocsize);
+}
+
 static void display_vector_interface(starpu_data_handle_t handle, FILE *f)
 {
 	struct starpu_vector_interface *vector_interface = (struct starpu_vector_interface *)
@@ -192,7 +211,7 @@ static int pack_vector_handle(starpu_data_handle_t handle, unsigned node, void *
 
 	if (ptr != NULL)
 	{
-		starpu_malloc_flags(ptr, *count, 0);
+		*ptr = (void *)starpu_malloc_on_node_flags(node, *count, 0);
 		memcpy(*ptr, (void*)vector_interface->ptr, vector_interface->elemsize*vector_interface->nx);
 	}
 
@@ -209,6 +228,8 @@ static int unpack_vector_handle(starpu_data_handle_t handle, unsigned node, void
 	STARPU_ASSERT(count == vector_interface->elemsize * vector_interface->nx);
 	memcpy((void*)vector_interface->ptr, ptr, count);
 
+	starpu_free_on_node_flags(node, (uintptr_t)ptr, count, 0);
+
 	return 0;
 }
 
@@ -222,7 +243,23 @@ static size_t vector_interface_get_size(starpu_data_handle_t handle)
 	STARPU_ASSERT_MSG(vector_interface->id == STARPU_VECTOR_INTERFACE_ID, "Error. The given data is not a vector.");
 #endif
 
-	size = vector_interface->nx*vector_interface->elemsize;
+	size = vector_interface->nx * vector_interface->elemsize;
+
+	return size;
+}
+
+static size_t vector_interface_get_alloc_size(starpu_data_handle_t handle)
+{
+	size_t size;
+	struct starpu_vector_interface *vector_interface = (struct starpu_vector_interface *)
+		starpu_data_get_interface_on_node(handle, STARPU_MAIN_RAM);
+
+#ifdef STARPU_DEBUG
+	STARPU_ASSERT_MSG(vector_interface->id == STARPU_VECTOR_INTERFACE_ID, "Error. The given data is not a vector.");
+#endif
+
+	size = vector_interface->allocsize;
+	STARPU_ASSERT_MSG(size != (size_t)-1, "The vector allocation size needs to be defined");
 
 	return size;
 }
@@ -243,7 +280,7 @@ uint32_t starpu_vector_get_nx(starpu_data_handle_t handle)
 uintptr_t starpu_vector_get_local_ptr(starpu_data_handle_t handle)
 {
 	unsigned node;
-	node = _starpu_memory_node_get_local_key();
+	node = starpu_worker_get_local_memory_node();
 
 	STARPU_ASSERT(starpu_data_test_if_allocated_on_node(handle, node));
 
@@ -269,6 +306,18 @@ size_t starpu_vector_get_elemsize(starpu_data_handle_t handle)
 	return vector_interface->elemsize;
 }
 
+size_t starpu_vector_get_allocsize(starpu_data_handle_t handle)
+{
+	struct starpu_vector_interface *vector_interface = (struct starpu_vector_interface *)
+		starpu_data_get_interface_on_node(handle, STARPU_MAIN_RAM);
+
+#ifdef STARPU_DEBUG
+	STARPU_ASSERT_MSG(vector_interface->id == STARPU_VECTOR_INTERFACE_ID, "Error. The given data is not a vector.");
+#endif
+
+	return vector_interface->allocsize;
+}
+
 /* memory allocation/deallocation primitives for the vector interface */
 
 /* returns the size of the allocated area */
@@ -278,20 +327,14 @@ static starpu_ssize_t allocate_vector_buffer_on_node(void *data_interface_, unsi
 
 	struct starpu_vector_interface *vector_interface = (struct starpu_vector_interface *) data_interface_;
 
-	uint32_t nx = vector_interface->nx;
-	size_t elemsize = vector_interface->elemsize;
-
-	starpu_ssize_t allocated_memory;
-
-	handle = starpu_malloc_on_node(dst_node, nx*elemsize);
+	starpu_ssize_t allocated_memory = vector_interface->allocsize;
+	handle = starpu_malloc_on_node(dst_node, allocated_memory);
 	if (!handle)
 		return -ENOMEM;
 
 	if (starpu_node_get_kind(dst_node) != STARPU_OPENCL_RAM)
 		addr = handle;
 
-	allocated_memory = nx*elemsize;
-
 	/* update the data properly in consequence */
 	vector_interface->ptr = addr;
 	vector_interface->dev_handle = handle;
@@ -303,10 +346,8 @@ static starpu_ssize_t allocate_vector_buffer_on_node(void *data_interface_, unsi
 static void free_vector_buffer_on_node(void *data_interface, unsigned node)
 {
 	struct starpu_vector_interface *vector_interface = (struct starpu_vector_interface *) data_interface;
-	uint32_t nx = vector_interface->nx;
-	size_t elemsize = vector_interface->elemsize;
 
-	starpu_free_on_node(node, vector_interface->dev_handle, nx*elemsize);
+	starpu_free_on_node(node, vector_interface->dev_handle, vector_interface->allocsize);
 }
 
 static int copy_any_to_any(void *src_interface, unsigned src_node,
@@ -320,7 +361,7 @@ static int copy_any_to_any(void *src_interface, unsigned src_node,
 				    dst_vector->dev_handle, dst_vector->offset, dst_node,
 				    src_vector->nx*src_vector->elemsize, async_data);
 
-	_STARPU_TRACE_DATA_COPY(src_node, dst_node, src_vector->nx*src_vector->elemsize);
+	starpu_interface_data_copy(src_node, dst_node, src_vector->nx*src_vector->elemsize);
 	return ret;
 }
 
