@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2010, 2012-2015  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  CNRS
+ * Copyright (C) 2008-2015,2017,2018                      Université de Bordeaux
+ * Copyright (C) 2010-2013,2015,2017                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -57,6 +57,10 @@ LIST_TYPE(_starpu_mem_chunk,
 	unsigned home:1;
 	/* Whether the memchunk is in the clean part of the mc_list */
 	unsigned clean:1;
+	/* Was this chunk used since it got allocated?  */
+	unsigned diduse:1;
+	/* Was this chunk marked as "won't use"? */
+	unsigned wontuse:1;
 
 	/* the size of the data is only set when calling _starpu_request_mem_chunk_removal(),
 	 * it is needed to estimate how much memory is in mc_cache, and by
