@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2016  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013, 2014, 2015, 2017  CNRS
+ * Copyright (C) 2009-2017,2019                           Université de Bordeaux
+ * Copyright (C) 2011-2013,2016,2017                      Inria
+ * Copyright (C) 2010-2015,2017,2018,2019                 CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,11 +23,13 @@
 #include <datawizard/memory_nodes.h>
 #include <datawizard/malloc.h>
 #include <common/fxt.h>
-#include "copy_driver.h"
-#include "memalloc.h"
+#include <datawizard/copy_driver.h>
+#include <datawizard/memalloc.h>
+#include <datawizard/node_ops.h>
+
+char _starpu_worker_drives_memory[STARPU_NMAXWORKERS][STARPU_MAXNODES];
 
 struct _starpu_memory_node_descr _starpu_descr;
-starpu_pthread_key_t _starpu_memory_node_key STARPU_ATTRIBUTE_INTERNAL;
 
 void _starpu_memory_nodes_init(void)
 {
@@ -34,14 +37,14 @@ void _starpu_memory_nodes_init(void)
 	 * added using _starpu_memory_node_register */
 	_starpu_descr.nnodes = 0;
 
-	STARPU_PTHREAD_KEY_CREATE(&_starpu_memory_node_key, NULL);
-
 	unsigned i;
 	for (i = 0; i < STARPU_MAXNODES; i++)
 	{
 		_starpu_descr.nodes[i] = STARPU_UNUSED;
 		_starpu_descr.nworkers[i] = 0;
 	}
+	memset(&_starpu_worker_drives_memory, 0, sizeof(_starpu_worker_drives_memory));
+	STARPU_HG_DISABLE_CHECKING(_starpu_worker_drives_memory);
 
 	_starpu_init_mem_chunk_lists();
 	_starpu_init_data_request_lists();
@@ -57,7 +60,6 @@ void _starpu_memory_nodes_deinit(void)
 	_starpu_deinit_mem_chunk_lists();
 
 	STARPU_PTHREAD_RWLOCK_DESTROY(&_starpu_descr.conditions_rwlock);
-	STARPU_PTHREAD_KEY_DELETE(_starpu_memory_node_key);
 }
 
 #undef starpu_node_get_kind
@@ -72,51 +74,24 @@ unsigned starpu_memory_nodes_get_count(void)
 	return _starpu_memory_nodes_get_count();
 }
 
-void _starpu_memory_node_get_name(unsigned node, char *name, int size)
+int starpu_memory_node_get_name(unsigned node, char *name, size_t size)
 {
-	const char *prefix;
-	switch (_starpu_descr.nodes[node])
-	{
-	case STARPU_CPU_RAM:
-		prefix = "RAM";
-		break;
-	case STARPU_CUDA_RAM:
-		prefix = "CUDA";
-		break;
-	case STARPU_OPENCL_RAM:
-		prefix = "OpenCL";
-		break;
-	case STARPU_DISK_RAM:
-		prefix = "Disk";
-		break;
-	case STARPU_MIC_RAM:
-		prefix = "MIC";
-		break;
-	case STARPU_SCC_RAM:
-		prefix = "SCC_RAM";
-		break;
-	case STARPU_SCC_SHM:
-		prefix = "SCC_shared";
-		break;
-	case STARPU_UNUSED:
-	default:
-		prefix = "unknown";
-		STARPU_ASSERT(0);
-	}
-	snprintf(name, size, "%s %d", prefix, _starpu_descr.devid[node]);
+	const char *prefix = _starpu_node_get_prefix(_starpu_descr.nodes[node]);
+	return snprintf(name, size, "%s %d", prefix, _starpu_descr.devid[node]);
 }
 
-unsigned _starpu_memory_node_register(enum starpu_node_kind kind, int devid)
+unsigned _starpu_memory_node_register(enum starpu_node_kind kind, int devid, struct _starpu_node_ops *node_ops)
 {
 	unsigned node;
 	/* ATOMIC_ADD returns the new value ... */
 	node = STARPU_ATOMIC_ADD(&_starpu_descr.nnodes, 1) - 1;
-	STARPU_ASSERT_MSG(node < STARPU_MAXNODES,"Too many nodes (%u) for maximum %u. Use configure option --enable-maxnodes=xxx to update the maximum number of nodes.", node, STARPU_MAXNODES);
+	STARPU_ASSERT_MSG(node < STARPU_MAXNODES,"Too many nodes (%u) for maximum %d. Use configure option --enable-maxnodes=xxx to update the maximum number of nodes.", node, STARPU_MAXNODES);
 
 	_starpu_descr.nodes[node] = kind;
 	_STARPU_TRACE_NEW_MEM_NODE(node);
 
 	_starpu_descr.devid[node] = devid;
+	_starpu_descr.node_ops[node] = node_ops;
 
 	/* for now, there is no condition associated to that newly created node */
 	_starpu_descr.condition_count[node] = 0;
@@ -129,7 +104,7 @@ unsigned _starpu_memory_node_register(enum starpu_node_kind kind, int devid)
 /* TODO move in a more appropriate file  !! */
 /* Register a condition variable associated to worker which is associated to a
  * memory node itself. */
-void _starpu_memory_node_register_condition(starpu_pthread_cond_t *cond, starpu_pthread_mutex_t *mutex, unsigned nodeid)
+void _starpu_memory_node_register_condition(struct _starpu_worker *worker, starpu_pthread_cond_t *cond, unsigned nodeid)
 {
 	unsigned cond_id;
 	unsigned nconds_total, nconds;
@@ -142,7 +117,7 @@ void _starpu_memory_node_register_condition(starpu_pthread_cond_t *cond, starpu_
 	{
 		if (_starpu_descr.conditions_attached_to_node[nodeid][cond_id].cond == cond)
 		{
-			STARPU_ASSERT(_starpu_descr.conditions_attached_to_node[nodeid][cond_id].mutex == mutex);
+			STARPU_ASSERT(_starpu_descr.conditions_attached_to_node[nodeid][cond_id].worker == worker);
 
 			/* the condition is already in the list */
 			STARPU_PTHREAD_RWLOCK_UNLOCK(&_starpu_descr.conditions_rwlock);
@@ -152,7 +127,7 @@ void _starpu_memory_node_register_condition(starpu_pthread_cond_t *cond, starpu_
 
 	/* it was not found locally */
 	_starpu_descr.conditions_attached_to_node[nodeid][cond_id].cond = cond;
-	_starpu_descr.conditions_attached_to_node[nodeid][cond_id].mutex = mutex;
+	_starpu_descr.conditions_attached_to_node[nodeid][cond_id].worker = worker;
 	_starpu_descr.condition_count[nodeid]++;
 
 	/* do we have to add it in the global list as well ? */
@@ -169,7 +144,7 @@ void _starpu_memory_node_register_condition(starpu_pthread_cond_t *cond, starpu_
 
 	/* it was not in the global list either */
 	_starpu_descr.conditions_all[nconds_total].cond = cond;
-	_starpu_descr.conditions_all[nconds_total].mutex = mutex;
+	_starpu_descr.conditions_all[nconds_total].worker = worker;
 	_starpu_descr.total_condition_count++;
 
 	STARPU_PTHREAD_RWLOCK_UNLOCK(&_starpu_descr.conditions_rwlock);
@@ -181,3 +156,28 @@ unsigned starpu_worker_get_memory_node(unsigned workerid)
 	return _starpu_worker_get_memory_node(workerid);
 }
 
+/* same utility as _starpu_memory_node_add_nworkers */
+void _starpu_worker_drives_memory_node(struct _starpu_worker *worker, unsigned memnode)
+{
+	if (! _starpu_worker_drives_memory[worker->workerid][memnode])
+	{
+		_starpu_worker_drives_memory[worker->workerid][memnode] = 1;
+#ifdef STARPU_SIMGRID
+		starpu_pthread_queue_register(&worker->wait, &_starpu_simgrid_transfer_queue[memnode]);
+#endif
+		_starpu_memory_node_register_condition(worker, &worker->sched_cond, memnode);
+	}
+}
+
+unsigned starpu_worker_get_local_memory_node(void)
+{
+	struct _starpu_worker *worker = _starpu_get_local_worker_key();
+	if (!worker)
+		return STARPU_MAIN_RAM;
+	return worker->memory_node;
+}
+
+int starpu_memory_node_get_devid(unsigned node)
+{
+	return _starpu_descr.devid[node];
+}
