@@ -1,7 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2014, 2016  Université de Bordeaux
- * Copyright (C) 2011, 2012, 2013, 2017  CNRS
+ * Copyright (C) 2011,2012,2017                           Inria
+ * Copyright (C) 2010-2014,2016,2017                      Université de Bordeaux
+ * Copyright (C) 2011-2013,2015-2017,2019                 CNRS
+ * Copyright (C) 2013                                     Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -66,6 +68,10 @@ void _starpu_redux_init_data_replicate(starpu_data_handle_t handle, struct _star
 			break;
 		case STARPU_CUDA_WORKER:
 			init_func = _starpu_task_get_cuda_nth_implementation(init_cl, 0);
+#if defined(STARPU_HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
+			/* We make sure we do manipulate the proper device */
+			starpu_cuda_set_device(starpu_worker_get_devid(workerid));
+#endif
 			break;
 		case STARPU_OPENCL_WORKER:
 			init_func = _starpu_task_get_opencl_nth_implementation(init_cl, 0);
@@ -75,7 +81,11 @@ void _starpu_redux_init_data_replicate(starpu_data_handle_t handle, struct _star
 			init_func = _starpu_mic_src_get_kernel_from_codelet(init_cl, 0);
 			break;
 #endif
-			/* TODO: SCC */
+#ifdef STARPU_USE_MPI_MASTER_SLAVE
+		case STARPU_MPI_MS_WORKER:
+			init_func = _starpu_mpi_ms_src_get_kernel_from_codelet(init_cl, 0); 
+			break;
+#endif
 		default:
 			STARPU_ABORT();
 			break;
@@ -83,24 +93,45 @@ void _starpu_redux_init_data_replicate(starpu_data_handle_t handle, struct _star
 
 	STARPU_ASSERT(init_func);
 
+	switch (starpu_worker_get_type(workerid))
+	{
 #ifdef STARPU_USE_MIC
-	if (starpu_worker_get_type(workerid) == STARPU_MIC_WORKER)
-	{
-		struct _starpu_mp_node *node = _starpu_mic_src_get_actual_thread_mp_node();
-		int devid = _starpu_get_worker_struct(workerid)->devid;
-		void * arg;
-		int arg_size;
-		_starpu_src_common_execute_kernel(node,
-						 (void(*)(void))init_func, devid,
-						 STARPU_SEQ, 0, 0, &handle, 
-						 &(replicate->data_interface), 1,
-						 NULL, 0);
-		_starpu_src_common_wait_completed_execution(node,devid,&arg,&arg_size);
-	}
-	else
+		case STARPU_MIC_WORKER:
+		{
+			struct _starpu_mp_node *node = _starpu_mic_src_get_actual_thread_mp_node();
+			int devid = _starpu_get_worker_struct(workerid)->devid;
+			void * arg;
+			int arg_size;
+			_starpu_src_common_execute_kernel(node,
+					(void(*)(void))init_func, devid,
+					STARPU_SEQ, 0, 0, &handle, 
+					&(replicate->data_interface), 1,
+					NULL, 0, 1);
+			_starpu_src_common_wait_completed_execution(node,devid,&arg,&arg_size);
+			break;
+		}
 #endif
-	{
-		init_func(&replicate->data_interface, NULL);
+#ifdef STARPU_USE_MPI_MASTER_SLAVE
+		case STARPU_MPI_MS_WORKER:
+		{
+			struct _starpu_mp_node *node = _starpu_mpi_ms_src_get_actual_thread_mp_node();
+			int devid = _starpu_get_worker_struct(workerid)->devid;
+			void * arg;
+			int arg_size;
+
+			_starpu_src_common_execute_kernel(node,
+					(void(*)(void))init_func, devid,
+					STARPU_SEQ, 0, 0, &handle, 
+					&(replicate->data_interface), 1,
+					NULL, 0 , 1);
+
+			_starpu_src_common_wait_completed_execution(node,devid,&arg,&arg_size);
+			break;
+		}
+#endif
+		default:
+			init_func(&replicate->data_interface, NULL);
+			break;
 	}
 
 	replicate->initialized = 1;
@@ -160,7 +191,7 @@ void _starpu_data_end_reduction_mode(starpu_data_handle_t handle)
 	/* Register all valid per-worker replicates */
 	unsigned nworkers = starpu_worker_get_count();
 	STARPU_ASSERT(!handle->reduction_tmp_handles);
-	_STARPU_MALLOC(handle->reduction_tmp_handles, nworkers * sizeof(handle->reduction_tmp_handles[0]));
+	_STARPU_MALLOC(handle->reduction_tmp_handles, nworkers*sizeof(handle->reduction_tmp_handles[0]));
 	for (worker = 0; worker < nworkers; worker++)
 	{
 		if (handle->per_worker[worker].initialized)
