@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2016                                Inria
+ * Copyright (C) 2011-2017                                Inria
  * Copyright (C) 2010-2018                                Université de Bordeaux
- * Copyright (C) 2010-2017                                CNRS
+ * Copyright (C) 2010-2017, 2019                          CNRS
  * Copyright (C) 2013                                     Thibaut Lambert
  * Copyright (C) 2011                                     Télécom-SudParis
  *
@@ -26,20 +26,17 @@
 #include <core/debug.h>
 #include <core/sched_ctx.h>
 #include <drivers/driver_common/driver_common.h>
-#include <starpu_top.h>
 #include <core/sched_policy.h>
-#include <top/starpu_top_core.h>
 #include <core/debug.h>
-
+#include <core/task.h>
 
 #define BACKOFF_MAX 32  /* TODO : use parameter to define them */
 #define BACKOFF_MIN 1
 
-void _starpu_driver_start_job(struct _starpu_worker *worker, struct _starpu_job *j, struct starpu_perfmodel_arch* perf_arch STARPU_ATTRIBUTE_UNUSED, int rank, int profiling)
+void _starpu_driver_start_job(struct _starpu_worker *worker, struct _starpu_job *j, struct starpu_perfmodel_arch* perf_arch, int rank, int profiling)
 {
 	struct starpu_task *task = j->task;
 	struct starpu_codelet *cl = task->cl;
-	int starpu_top=_starpu_top_status_get();
 	int workerid = worker->workerid;
 	unsigned calibrate_model = 0;
 
@@ -64,21 +61,19 @@ void _starpu_driver_start_job(struct _starpu_worker *worker, struct _starpu_job
 
 		struct starpu_profiling_task_info *profiling_info = task->profiling_info;
 
-		if ((profiling && profiling_info) || calibrate_model || starpu_top)
+		if ((profiling && profiling_info) || calibrate_model)
 		{
 			_starpu_clock_gettime(&worker->cl_start);
 			_starpu_worker_register_executing_start_date(workerid, &worker->cl_start);
 		}
+		_starpu_job_notify_start(j, perf_arch);
 	}
 
-	if (starpu_top)
-		_starpu_top_task_started(task,workerid,&worker->cl_start);
-
-
 	// Find out if the worker is the master of a parallel context
 	struct _starpu_sched_ctx *sched_ctx = _starpu_sched_ctx_get_sched_ctx_for_worker_and_job(worker, j);
 	if(!sched_ctx)
 		sched_ctx = _starpu_get_sched_ctx_struct(j->task->sched_ctx);
+	_starpu_sched_ctx_lock_read(sched_ctx->id);
 	if(!sched_ctx->sched_policy)
 	{
 		if(!sched_ctx->awake_workers && sched_ctx->main_master == worker->workerid)
@@ -108,6 +103,7 @@ void _starpu_driver_start_job(struct _starpu_worker *worker, struct _starpu_job
 		_STARPU_TRACE_TASK_COLOR(j);
 		_STARPU_TRACE_START_CODELET_BODY(j, j->nimpl, perf_arch, workerid);
 	}
+	_starpu_sched_ctx_unlock_read(sched_ctx->id);
 	_STARPU_TASK_BREAK_ON(task, exec);
 }
 
@@ -115,7 +111,6 @@ void _starpu_driver_end_job(struct _starpu_worker *worker, struct _starpu_job *j
 {
 	struct starpu_task *task = j->task;
 	struct starpu_codelet *cl = task->cl;
-	int starpu_top=_starpu_top_status_get();
 	int workerid = worker->workerid;
 	unsigned calibrate_model = 0;
 
@@ -141,7 +136,7 @@ void _starpu_driver_end_job(struct _starpu_worker *worker, struct _starpu_job *j
 	if (rank == 0)
 	{
 		struct starpu_profiling_task_info *profiling_info = task->profiling_info;
-		if ((profiling && profiling_info) || calibrate_model || starpu_top)
+		if ((profiling && profiling_info) || calibrate_model)
 		{
 			_starpu_clock_gettime(&worker->cl_end);
 			_starpu_worker_register_executing_end(workerid);
@@ -149,9 +144,6 @@ void _starpu_driver_end_job(struct _starpu_worker *worker, struct _starpu_job *j
 		STARPU_AYU_POSTRUNTASK(j->job_id);
 	}
 
-	if (starpu_top)
-		_starpu_top_task_ended(task,workerid,&worker->cl_end);
-
 	_starpu_set_worker_status(worker, STATUS_UNKNOWN);
 
 	if(!sched_ctx->sched_policy && !sched_ctx->awake_workers &&
@@ -200,6 +192,7 @@ void _starpu_driver_update_job_feedback(struct _starpu_job *j, struct _starpu_wo
 
 		starpu_timespec_sub(&worker->cl_end, &worker->cl_start, &measured_ts);
 		measured = starpu_timing_timespec_to_us(&measured_ts);
+		STARPU_ASSERT_MSG(measured >= 0, "measured=%lf\n", measured);
 
 		if (profiling && profiling_info)
 		{
@@ -211,7 +204,8 @@ void _starpu_driver_update_job_feedback(struct _starpu_job *j, struct _starpu_wo
 			_starpu_worker_update_profiling_info_executing(workerid, &measured_ts, 1,
 								       profiling_info->used_cycles,
 								       profiling_info->stall_cycles,
-								       profiling_info->energy_consumed);
+								       profiling_info->energy_consumed,
+								       j->task->flops);
 			updated =  1;
 		}
 
@@ -253,7 +247,7 @@ void _starpu_driver_update_job_feedback(struct _starpu_job *j, struct _starpu_wo
 	}
 
 	if (!updated)
-		_starpu_worker_update_profiling_info_executing(workerid, NULL, 1, 0, 0, 0);
+		_starpu_worker_update_profiling_info_executing(workerid, NULL, 1, 0, 0, 0, 0);
 
 	if (profiling_info && profiling_info->energy_consumed && cl->energy_model && cl->energy_model->benchmarking)
 	{
@@ -287,9 +281,11 @@ void _starpu_driver_update_job_feedback(struct _starpu_job *j, struct _starpu_wo
 
 static void _starpu_worker_set_status_scheduling(int workerid)
 {
-	if (_starpu_worker_get_status(workerid) != STATUS_SLEEPING
-		&& _starpu_worker_get_status(workerid) != STATUS_WAKING_UP
-		&& _starpu_worker_get_status(workerid) != STATUS_SCHEDULING)
+	if (_starpu_worker_get_status(workerid) == STATUS_SLEEPING)
+	{
+		_starpu_worker_set_status(workerid, STATUS_SLEEPING_SCHEDULING);
+	}
+	else if (_starpu_worker_get_status(workerid) != STATUS_SCHEDULING)
 	{
 		_STARPU_TRACE_WORKER_SCHEDULING_START;
 		_starpu_worker_set_status(workerid, STATUS_SCHEDULING);
@@ -298,7 +294,11 @@ static void _starpu_worker_set_status_scheduling(int workerid)
 
 static void _starpu_worker_set_status_scheduling_done(int workerid)
 {
-	if (_starpu_worker_get_status(workerid) == STATUS_SCHEDULING)
+	if (_starpu_worker_get_status(workerid) == STATUS_SLEEPING_SCHEDULING)
+	{
+		_starpu_worker_set_status(workerid, STATUS_SLEEPING);
+	}
+	else if (_starpu_worker_get_status(workerid) == STATUS_SCHEDULING)
 	{
 		_STARPU_TRACE_WORKER_SCHEDULING_END;
 		_starpu_worker_set_status(workerid, STATUS_UNKNOWN);
@@ -307,9 +307,7 @@ static void _starpu_worker_set_status_scheduling_done(int workerid)
 
 static void _starpu_worker_set_status_sleeping(int workerid)
 {
-	if ( _starpu_worker_get_status(workerid) == STATUS_WAKING_UP)
-		_starpu_worker_set_status(workerid, STATUS_SLEEPING);
-	else if (_starpu_worker_get_status(workerid) != STATUS_SLEEPING)
+	if (_starpu_worker_get_status(workerid) != STATUS_SLEEPING)
 	{
 		_STARPU_TRACE_WORKER_SLEEP_START;
 		_starpu_worker_restart_sleeping(workerid);
@@ -320,7 +318,7 @@ static void _starpu_worker_set_status_sleeping(int workerid)
 
 static void _starpu_worker_set_status_wakeup(int workerid)
 {
-	if (_starpu_worker_get_status(workerid) == STATUS_SLEEPING || _starpu_worker_get_status(workerid) == STATUS_WAKING_UP)
+	if (_starpu_worker_get_status(workerid) == STATUS_SLEEPING)
 	{
 		_STARPU_TRACE_WORKER_SLEEP_END;
 		_starpu_worker_stop_sleeping(workerid);
@@ -329,6 +327,7 @@ static void _starpu_worker_set_status_wakeup(int workerid)
 }
 
 
+#if !defined(STARPU_SIMGRID)
 static void _starpu_exponential_backoff(struct _starpu_worker *worker)
 {
 	int delay = worker->spinning_backoff;
@@ -339,87 +338,51 @@ static void _starpu_exponential_backoff(struct _starpu_worker *worker)
 	while(delay--)
 		STARPU_UYIELD();
 }
+#endif
 
 
 
 /* Workers may block when there is no work to do at all. */
 struct starpu_task *_starpu_get_worker_task(struct _starpu_worker *worker, int workerid, unsigned memnode STARPU_ATTRIBUTE_UNUSED)
 {
-	STARPU_PTHREAD_MUTEX_LOCK_SCHED(&worker->sched_mutex);
 	struct starpu_task *task;
-	unsigned needed = 1;
-	unsigned executing STARPU_ATTRIBUTE_UNUSED = 0;
+#if !defined(STARPU_SIMGRID)
+	unsigned keep_awake = 0;
+#endif
 
+	STARPU_PTHREAD_MUTEX_LOCK_SCHED(&worker->sched_mutex);
+	_starpu_worker_enter_sched_op(worker);
 	_starpu_worker_set_status_scheduling(workerid);
-	while(needed)
-	{
-		struct _starpu_sched_ctx *sched_ctx = NULL;
-		struct _starpu_sched_ctx_list *l = NULL;
-		for (l = worker->sched_ctx_list; l; l = l->next)
-		{
-			sched_ctx = _starpu_get_sched_ctx_struct(l->sched_ctx);
-			if(sched_ctx && sched_ctx->id > 0 && sched_ctx->id < STARPU_NMAX_SCHED_CTXS)
-			{
-				STARPU_PTHREAD_MUTEX_LOCK(&sched_ctx->parallel_sect_mutex[workerid]);
-				if(!sched_ctx->sched_policy && sched_ctx->awake_workers) 
-					worker->slave = sched_ctx->main_master != workerid;
-
-				if(sched_ctx->parallel_sect[workerid])
-				{
-					/* don't let the worker sleep with the sched_mutex taken */
-					/* we need it until here bc of the list of ctxs of the workers
-					   that can change in another thread */
-					STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(&worker->sched_mutex);
-					needed = 0;
-					_starpu_sched_ctx_signal_worker_blocked(sched_ctx->id, workerid);
-					sched_ctx->busy[workerid] = 1;
-					STARPU_PTHREAD_COND_WAIT(&sched_ctx->parallel_sect_cond[workerid], &sched_ctx->parallel_sect_mutex[workerid]);
-					sched_ctx->busy[workerid] = 0;
-					STARPU_PTHREAD_COND_SIGNAL(&sched_ctx->parallel_sect_cond_busy[workerid]);
-					_starpu_sched_ctx_signal_worker_woke_up(sched_ctx->id, workerid);
-					sched_ctx->parallel_sect[workerid] = 0;
-					STARPU_PTHREAD_MUTEX_LOCK_SCHED(&worker->sched_mutex);
-				}
-				STARPU_PTHREAD_MUTEX_UNLOCK(&sched_ctx->parallel_sect_mutex[workerid]);
-			}
-			if(!needed)
-				break;
-		}
-		/* don't worry if the value is not correct (no lock) it will do it next time */
-		if(worker->tmp_sched_ctx != -1)
-		{
-			sched_ctx = _starpu_get_sched_ctx_struct(worker->tmp_sched_ctx);
-			STARPU_PTHREAD_MUTEX_LOCK(&sched_ctx->parallel_sect_mutex[workerid]);
-			if(sched_ctx->parallel_sect[workerid])
-			{
-//				needed = 0;
-				STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(&worker->sched_mutex);
-				_starpu_sched_ctx_signal_worker_blocked(sched_ctx->id, workerid);
-				sched_ctx->busy[workerid] = 1;
-				STARPU_PTHREAD_COND_WAIT(&sched_ctx->parallel_sect_cond[workerid], &sched_ctx->parallel_sect_mutex[workerid]);
-				sched_ctx->busy[workerid] = 0;
-				STARPU_PTHREAD_COND_SIGNAL(&sched_ctx->parallel_sect_cond_busy[workerid]);
-				_starpu_sched_ctx_signal_worker_woke_up(sched_ctx->id, workerid);
-				sched_ctx->parallel_sect[workerid] = 0;
-				STARPU_PTHREAD_MUTEX_LOCK_SCHED(&worker->sched_mutex);
-			}
-			STARPU_PTHREAD_MUTEX_UNLOCK(&sched_ctx->parallel_sect_mutex[workerid]);
-		}
-
-		needed = !needed;
-	}
-
+#if !defined(STARPU_SIMGRID)
 	if ((worker->pipeline_length == 0 && worker->current_task)
 		|| (worker->pipeline_length != 0 && worker->ntasks))
 		/* This worker is executing something */
-		executing = 1;
+		keep_awake = 1;
+#endif
 
+	/*if the worker is already executing a task then */
 	if (worker->pipeline_length && (worker->ntasks == worker->pipeline_length || worker->pipeline_stuck))
 		task = NULL;
+	/* don't push a task if we are already transferring one */
+	else if (worker->task_transferring != NULL)
+		task = NULL;
+	/*else try to pop a task*/
 	else
+	{
+		STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(&worker->sched_mutex);
 		task = _starpu_pop_task(worker);
+		STARPU_PTHREAD_MUTEX_LOCK_SCHED(&worker->sched_mutex);
+#if !defined(STARPU_SIMGRID)
+		if (worker->state_keep_awake)
+		{
+			keep_awake = 1;
+			worker->state_keep_awake = 0;
+		}
+#endif
+	}
 
-	if (task == NULL && !executing)
+#if !defined(STARPU_SIMGRID)
+	if (task == NULL && !keep_awake)
 	{
 		/* Didn't get a task to run and none are running, go to sleep */
 
@@ -427,38 +390,68 @@ struct starpu_task *_starpu_get_worker_task(struct _starpu_worker *worker, int w
 		 * from popping a task from the scheduler to blocking. Otherwise the
 		 * driver may go block just after the scheduler got a new task to be
 		 * executed, and thus hanging. */
-
 		_starpu_worker_set_status_sleeping(workerid);
+		_starpu_worker_leave_sched_op(worker);
+		STARPU_PTHREAD_COND_BROADCAST(&worker->sched_cond);
 
+#ifndef STARPU_NON_BLOCKING_DRIVERS
 		if (_starpu_worker_can_block(memnode, worker)
-#ifndef STARPU_SIMGRID
-				&& !_starpu_sched_ctx_last_worker_awake(worker)
-#endif
-				)
+			&& !worker->state_block_in_parallel_req
+			&& !worker->state_unblock_in_parallel_req
+			&& !_starpu_sched_ctx_last_worker_awake(worker))
 		{
-			STARPU_PTHREAD_COND_WAIT(&worker->sched_cond, &worker->sched_mutex);
+
+#ifdef STARPU_WORKER_CALLBACKS
+			if (_starpu_config.conf.callback_worker_going_to_sleep != NULL)
+			{
+				_starpu_config.conf.callback_worker_going_to_sleep(workerid);
+			}
+#endif
+			do
+			{
+				STARPU_PTHREAD_COND_WAIT(&worker->sched_cond, &worker->sched_mutex);
+				if (!worker->state_keep_awake
+					&& _starpu_worker_can_block(memnode, worker)
+					&& !worker->state_block_in_parallel_req
+					&& !worker->state_unblock_in_parallel_req)
+				{
+					_starpu_worker_set_status_sleeping(workerid);
+					if (_starpu_sched_ctx_last_worker_awake(worker))
+					{
+						break;
+					}
+				}
+				else
+				{
+					break;
+				}
+			}
+			while (1);
+			worker->state_keep_awake = 0;
+			_starpu_worker_set_status_scheduling_done(workerid);
 			STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(&worker->sched_mutex);
+#ifdef STARPU_WORKER_CALLBACKS
+			if (_starpu_config.conf.callback_worker_waking_up != NULL)
+			{
+				/* the wake up callback should be called once the sched_mutex has been unlocked,
+				 * so that an external resource manager can potentially defer the wake-up momentarily if
+				 * the corresponding computing unit is still in use by another runtime system */
+				_starpu_config.conf.callback_worker_waking_up(workerid);
+			}
+#endif
 		}
 		else
+#endif
 		{
+			_starpu_worker_set_status_scheduling_done(workerid);
 			STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(&worker->sched_mutex);
 			if (_starpu_machine_is_running())
-			{
 				_starpu_exponential_backoff(worker);
-#ifdef STARPU_SIMGRID
-				static int warned;
-				if (!warned)
-				{
-					warned = 1;
-					_STARPU_DISP("Has to make simgrid spin for CPU idle time.  You can try to pass --enable-blocking-drivers to ./configure to avoid this\n");
-				}
-				MSG_process_sleep(0.000010);
-#endif
-			}
 		}
 
 		return NULL;
 	}
+#endif
 
 	if (task)
 	{
@@ -471,6 +464,8 @@ struct starpu_task *_starpu_get_worker_task(struct _starpu_worker *worker, int w
 	}
 	worker->spinning_backoff = BACKOFF_MIN;
 
+	_starpu_worker_leave_sched_op(worker);
+	STARPU_PTHREAD_COND_BROADCAST(&worker->sched_cond);
 	STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(&worker->sched_mutex);
 
 
@@ -486,19 +481,26 @@ int _starpu_get_multi_worker_task(struct _starpu_worker *workers, struct starpu_
 	struct _starpu_job * j;
 	int is_parallel_task;
 	struct _starpu_combined_worker *combined_worker;
-	int executing STARPU_ATTRIBUTE_UNUSED = 0;
+#if !defined(STARPU_NON_BLOCKING_DRIVERS) && !defined(STARPU_SIMGRID)
+	int executing = 0;
+#endif
 	/*for each worker*/
 #ifndef STARPU_NON_BLOCKING_DRIVERS
 	/* This assumes only 1 worker */
 	STARPU_ASSERT_MSG(nworkers == 1, "Multiple workers is not yet possible in blocking drivers mode\n");
+	_starpu_set_local_worker_key(&workers[0]);
 	STARPU_PTHREAD_MUTEX_LOCK_SCHED(&workers[0].sched_mutex);
+	_starpu_worker_enter_sched_op(&workers[0]);
 #endif
 	for (i = 0; i < nworkers; i++)
 	{
+		unsigned keep_awake = 0;
+#if !defined(STARPU_NON_BLOCKING_DRIVERS) && !defined(STARPU_SIMGRID)
 		if ((workers[i].pipeline_length == 0 && workers[i].current_task)
 			|| (workers[i].pipeline_length != 0 && workers[i].ntasks))
 			/* At least this worker is executing something */
 			executing = 1;
+#endif
 		/*if the worker is already executing a task then */
 		if((workers[i].pipeline_length == 0 && workers[i].current_task)
 			|| (workers[i].pipeline_length != 0 &&
@@ -507,24 +509,42 @@ int _starpu_get_multi_worker_task(struct _starpu_worker *workers, struct starpu_
 		{
 			tasks[i] = NULL;
 		}
+		/* don't push a task if we are already transferring one */
+		else if (workers[i].task_transferring != NULL)
+		{
+			tasks[i] = NULL;
+		}
 		/*else try to pop a task*/
 		else
 		{
 #ifdef STARPU_NON_BLOCKING_DRIVERS
+			_starpu_set_local_worker_key(&workers[i]);
 			STARPU_PTHREAD_MUTEX_LOCK_SCHED(&workers[i].sched_mutex);
+			_starpu_worker_enter_sched_op(&workers[i]);
 #endif
 			_starpu_worker_set_status_scheduling(workers[i].workerid);
-			_starpu_set_local_worker_key(&workers[i]);
+			STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(&workers[i].sched_mutex);
 			tasks[i] = _starpu_pop_task(&workers[i]);
-			if(tasks[i] != NULL)
+			STARPU_PTHREAD_MUTEX_LOCK_SCHED(&workers[i].sched_mutex);
+			if (workers[i].state_keep_awake)
+			{
+				keep_awake = workers[i].state_keep_awake;
+				workers[i].state_keep_awake = 0;
+			}
+			if(tasks[i] != NULL || keep_awake)
 			{
 				_starpu_worker_set_status_scheduling_done(workers[i].workerid);
 				_starpu_worker_set_status_wakeup(workers[i].workerid);
+				STARPU_PTHREAD_COND_BROADCAST(&workers[i].sched_cond);
 #ifdef STARPU_NON_BLOCKING_DRIVERS
+				_starpu_worker_leave_sched_op(&workers[i]);
 				STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(&workers[i].sched_mutex);
 #endif
 
 				count ++;
+				if (tasks[i] == NULL)
+					/* no task, but keep_awake */
+					continue;
 				j = _starpu_get_job_associated_to_task(tasks[i]);
 				is_parallel_task = (j->task_size > 1);
 				if (workers[i].pipeline_length)
@@ -558,6 +578,10 @@ int _starpu_get_multi_worker_task(struct _starpu_worker *workers, struct starpu_
 			else
 			{
 				_starpu_worker_set_status_sleeping(workers[i].workerid);
+#ifdef STARPU_NON_BLOCKING_DRIVERS
+				_starpu_worker_leave_sched_op(&workers[i]);
+#endif
+				STARPU_PTHREAD_COND_BROADCAST(&workers[i].sched_cond);
 #ifdef STARPU_NON_BLOCKING_DRIVERS
 				STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(&workers[i].sched_mutex);
 #endif
@@ -565,7 +589,8 @@ int _starpu_get_multi_worker_task(struct _starpu_worker *workers, struct starpu_
 		}
 	}
 
-#ifndef STARPU_NON_BLOCKING_DRIVERS
+#if !defined(STARPU_NON_BLOCKING_DRIVERS)
+#if !defined(STARPU_SIMGRID)
 	/* Block the assumed-to-be-only worker */
 	struct _starpu_worker *worker = &workers[0];
 	unsigned workerid = workers[0].workerid;
@@ -579,40 +604,68 @@ int _starpu_get_multi_worker_task(struct _starpu_worker *workers, struct starpu_
 		 * driver may go block just after the scheduler got a new task to be
 		 * executed, and thus hanging. */
 		_starpu_worker_set_status_sleeping(workerid);
+		_starpu_worker_leave_sched_op(worker);
 
 		if (_starpu_worker_can_block(memnode, worker)
-#ifndef STARPU_SIMGRID
-				&& !_starpu_sched_ctx_last_worker_awake(worker)
-#endif
-				)
+			&& !worker->state_block_in_parallel_req
+			&& !worker->state_unblock_in_parallel_req
+			&& !_starpu_sched_ctx_last_worker_awake(worker))
 		{
-			STARPU_PTHREAD_COND_WAIT(&worker->sched_cond, &worker->sched_mutex);
+#ifdef STARPU_WORKER_CALLBACKS
+			if (_starpu_config.conf.callback_worker_going_to_sleep != NULL)
+			{
+				_starpu_config.conf.callback_worker_going_to_sleep(workerid);
+			}
+#endif
+			do
+			{
+				STARPU_PTHREAD_COND_WAIT(&worker->sched_cond, &worker->sched_mutex);
+				if (!worker->state_keep_awake
+					&& _starpu_worker_can_block(memnode, worker)
+					&& !worker->state_block_in_parallel_req
+					&& !worker->state_unblock_in_parallel_req)
+				{
+					_starpu_worker_set_status_sleeping(workerid);
+					if (_starpu_sched_ctx_last_worker_awake(worker))
+					{
+						break;
+					}
+				}
+				else
+				{
+					break;
+				}
+			}
+			while (1);
+			worker->state_keep_awake = 0;
+			_starpu_worker_set_status_scheduling_done(workerid);
 			STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(&worker->sched_mutex);
+#ifdef STARPU_WORKER_CALLBACKS
+			if (_starpu_config.conf.callback_worker_waking_up != NULL)
+			{
+				/* the wake up callback should be called once the sched_mutex has been unlocked,
+				 * so that an external resource manager can potentially defer the wake-up momentarily if
+				 * the corresponding computing unit is still in use by another runtime system */
+				_starpu_config.conf.callback_worker_waking_up(workerid);
+			}
+#endif
 		}
 		else
 		{
+			_starpu_worker_set_status_scheduling_done(workerid);
 			STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(&worker->sched_mutex);
 			if (_starpu_machine_is_running())
-			{
 				_starpu_exponential_backoff(worker);
-#ifdef STARPU_SIMGRID
-				static int warned;
-				if (!warned)
-				{
-					warned = 1;
-					_STARPU_DISP("Has to make simgrid spin for CPU idle time.  You can try to pass --enable-blocking-drivers to ./configure to avoid this\n");
-				}
-				MSG_process_sleep(0.000010);
-#endif
-			}
 		}
 		return 0;
 	}
 
 	_starpu_worker_set_status_wakeup(workerid);
 	worker->spinning_backoff = BACKOFF_MIN;
+#endif /* !STARPU_SIMGRID */
 
-	STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(&worker->sched_mutex);
+	_starpu_worker_leave_sched_op(&workers[0]);
+	STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(&workers[0].sched_mutex);
 #endif /* !STARPU_NON_BLOCKING_DRIVERS */
 
 	return count;
