@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2011, 2012, 2014, 2015, 2016, 2017  CNRS
- * Copyright (C) 2010-2014, 2016  Université de Bordeaux
+ * Copyright (C) 2010-2018                                Université de Bordeaux
+ * Copyright (C) 2011,2012,2016                           Inria
+ * Copyright (C) 2010-2019                                CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -54,32 +55,34 @@ int _starpu_opencl_locate_file(const char *source_file_name, char **located_file
 	if (access(source_file_name, R_OK) == 0)
 	{
 		_STARPU_CALLOC(*located_file_name, 1, strlen(source_file_name)+1);
-		sprintf(*located_file_name, "%s", source_file_name);
+		snprintf(*located_file_name, strlen(source_file_name)+1, "%s", source_file_name);
 		ret = EXIT_SUCCESS;
 	}
 
 	if (ret == EXIT_FAILURE && _starpu_opencl_program_dir)
 	{
 		_STARPU_CALLOC(*located_file_name, 1, strlen(_starpu_opencl_program_dir)+1+strlen(source_file_name)+1);
-		sprintf(*located_file_name, "%s/%s", _starpu_opencl_program_dir, source_file_name);
+		snprintf(*located_file_name, strlen(_starpu_opencl_program_dir)+1+strlen(source_file_name)+1, "%s/%s", _starpu_opencl_program_dir, source_file_name);
 		_STARPU_DEBUG("Trying to locate <%s>\n", *located_file_name);
 		if (access(*located_file_name, R_OK) == 0)
 			ret = EXIT_SUCCESS;
 	}
 
+#ifdef STARPU_DEVEL
 	if (ret == EXIT_FAILURE)
 	{
 		_STARPU_CALLOC(*located_file_name, 1, strlen(STARPU_SRC_DIR)+1+strlen(source_file_name)+1);
-		sprintf(*located_file_name, "%s/%s", STARPU_SRC_DIR, source_file_name);
+		snprintf(*located_file_name, strlen(STARPU_SRC_DIR)+1+strlen(source_file_name)+1, "%s/%s", STARPU_SRC_DIR, source_file_name);
 		_STARPU_DEBUG("Trying to locate <%s>\n", *located_file_name);
 		if (access(*located_file_name, R_OK) == 0)
 			ret = EXIT_SUCCESS;
 	}
+#endif
 
 	if (ret == EXIT_FAILURE)
 	{
 		_STARPU_CALLOC(*located_file_name, 1, strlen(_STARPU_STRINGIFY(STARPU_OPENCL_DATADIR))+1+strlen(source_file_name)+1);
-		sprintf(*located_file_name, "%s/%s", _STARPU_STRINGIFY(STARPU_OPENCL_DATADIR), source_file_name);
+		snprintf(*located_file_name, strlen(_STARPU_STRINGIFY(STARPU_OPENCL_DATADIR))+1+strlen(source_file_name)+1, "%s/%s", _STARPU_STRINGIFY(STARPU_OPENCL_DATADIR), source_file_name);
 		_STARPU_DEBUG("Trying to locate <%s>\n", *located_file_name);
 		if (access(*located_file_name, R_OK) == 0)
 			ret = EXIT_SUCCESS;
@@ -96,12 +99,12 @@ int _starpu_opencl_locate_file(const char *source_file_name, char **located_file
 		if (!last)
 		{
 			_STARPU_CALLOC(*located_dir_name, 2, sizeof(char));
-			sprintf(*located_dir_name, "%s", "");
+			snprintf(*located_dir_name, 2, "%s", "");
 		}
 		else
 		{
 			_STARPU_CALLOC(*located_dir_name, 1, 1+strlen(*located_file_name));
-			sprintf(*located_dir_name, "%s", *located_file_name);
+			snprintf(*located_dir_name, 1+strlen(*located_file_name), "%s", *located_file_name);
 			(*located_dir_name)[strlen(*located_file_name)-strlen(last)+1] = '\0';
 		}
 	}
@@ -153,12 +156,14 @@ char *_starpu_opencl_load_program_source(const char *filename)
 	char        *source;
 	int         x;
 	int         c;
+	int         err;
 
 	fh = fopen(filename, "r");
 	if (!fh)
 		return NULL;
 
-	stat(filename, &statbuf);
+	err = stat(filename, &statbuf);
+	STARPU_ASSERT_MSG(err == 0, "could not open file %s\n", filename);
 	_STARPU_MALLOC(source, statbuf.st_size + 1);
 
 	for(c=fgetc(fh), x=0 ; c != EOF ; c =fgetc(fh), x++)
@@ -180,12 +185,14 @@ char *_starpu_opencl_load_program_binary(const char *filename, size_t *len)
 	struct stat statbuf;
 	FILE        *fh;
 	char        *binary;
+        int         err;
 
 	fh = fopen(filename, "r");
 	if (fh == 0)
 		return NULL;
 
-	stat(filename, &statbuf);
+	err = stat(filename, &statbuf);
+	STARPU_ASSERT_MSG(err == 0, "could not open file %s\n", filename);
 
 	binary = (char *) malloc(statbuf.st_size);
 	if (!binary)
@@ -194,7 +201,8 @@ char *_starpu_opencl_load_program_binary(const char *filename, size_t *len)
 		return binary;
 	}
 
-	fread(binary, statbuf.st_size, 1, fh);
+	err = fread(binary, statbuf.st_size, 1, fh);
+	STARPU_ASSERT_MSG(err == 1, "could not read from file %s\n", filename);
 	fclose(fh);
 
 	*len = statbuf.st_size;
@@ -238,7 +246,7 @@ int _starpu_opencl_get_binary_name(char *binary_file_name, size_t maxlen, const
 	cl_int err;
 	cl_uint vendor_id;
 
-	_starpu_opencl_create_binary_directory(binary_directory, 1024);
+	_starpu_opencl_create_binary_directory(binary_directory, sizeof(binary_directory));
 
 	p = strrchr(source_file_name, '/');
 	snprintf(binary_file_name, maxlen, "%s/%s", binary_directory, p?p:source_file_name);
@@ -327,12 +335,12 @@ int _starpu_opencl_compile_or_load_opencl_from_string(const char *opencl_program
 		}
 		else
 		{
-			char binary_file_name[1024];
+			char binary_file_name[2048];
 			char *binary;
 			size_t binary_len;
 			FILE *fh;
 
-			err = _starpu_opencl_get_binary_name(binary_file_name, 1024, source_file_name, dev, device);
+			err = _starpu_opencl_get_binary_name(binary_file_name, sizeof(binary_file_name), source_file_name, dev, device);
 			if (STARPU_UNLIKELY(err != CL_SUCCESS)) STARPU_OPENCL_REPORT_ERROR(err);
 
 			err = clGetProgramInfo(program, CL_PROGRAM_BINARY_SIZES, sizeof(size_t), &binary_len, NULL);
@@ -374,7 +382,7 @@ void starpu_opencl_load_program_source_malloc(const char *source_file_name, char
 		_STARPU_ERROR("Failed to load compute program from file <%s>!\n", *located_file_name);
 
 	_STARPU_MALLOC(*opencl_program_source, strlen(source)+1);
-	sprintf(*opencl_program_source, "%s", source);
+	snprintf(*opencl_program_source, strlen(source)+1, "%s", source);
 	free(source);
 }
 
@@ -422,13 +430,11 @@ int _starpu_opencl_compile_or_load_opencl_from_file(const char *source_file_name
 
 	if (!strcmp(located_dir_name, ""))
 	{
-		sprintf(new_build_options, "%s", build_options);
+		snprintf(new_build_options, sizeof(new_build_options), "%s", build_options);
 	}
-	else if (build_options)
-		sprintf(new_build_options, "-I %s %s", located_dir_name, build_options);
 	else
 	{
-		sprintf(new_build_options, "-I %s", located_dir_name);
+		snprintf(new_build_options, sizeof(new_build_options), "-I %s %s", located_dir_name, build_options);
 	}
 	_STARPU_DEBUG("Build options: <%s>\n", new_build_options);
 
@@ -494,7 +500,7 @@ int starpu_opencl_load_binary_opencl(const char *kernel_id, struct starpu_opencl
 		}
 
 		// Load the binary buffer
-		err = _starpu_opencl_get_binary_name(binary_file_name, 1024, kernel_id, dev, device);
+		err = _starpu_opencl_get_binary_name(binary_file_name, sizeof(binary_file_name), kernel_id, dev, device);
 		if (STARPU_UNLIKELY(err != CL_SUCCESS)) STARPU_OPENCL_REPORT_ERROR(err);
 		binary = _starpu_opencl_load_program_binary(binary_file_name, &length);
 
@@ -771,9 +777,7 @@ const char *starpu_opencl_error_string(cl_int status)
 
 void starpu_opencl_display_error(const char *func, const char *file, int line, const char* msg, cl_int status)
 {
-	printf("oops in %s (%s:%d) (%s) ... <%s> (%d) \n", func, file, line, msg,
-	       starpu_opencl_error_string (status), status);
-
+	_STARPU_MSG("oops in %s (%s:%d) (%s) ... <%s> (%d) \n", func, file, line, msg, starpu_opencl_error_string (status), status);
 }
 
 int starpu_opencl_set_kernel_args(cl_int *error, cl_kernel *kernel, ...)
