@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
  * Copyright (C) 2013                                     Inria
- * Copyright (C) 2014-2015,2017                           CNRS
- * Copyright (C) 2014-2017                                Université de Bordeaux
+ * Copyright (C) 2014,2015,2017                           CNRS
+ * Copyright (C) 2014-2018                                Université de Bordeaux
  * Copyright (C) 2013                                     Simon Archipoff
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -20,6 +20,7 @@
 #include <starpu_sched_component.h>
 #include <core/workers.h>
 #include <core/sched_policy.h>
+#include <core/task.h>
 
 static double compute_relative_speedup(struct starpu_sched_component * component)
 {
@@ -46,7 +47,7 @@ static int random_push_task(struct starpu_sched_component * component, struct st
 	 * that can execute task.
 	 */
 	int indexes_components[component->nchildren];
-	int size=0;
+	unsigned size=0;
 
 	/* speedup[i] is revelant only if i is in the size firsts elements of
 	 * indexes_components
@@ -55,7 +56,7 @@ static int random_push_task(struct starpu_sched_component * component, struct st
 
 	double alpha_sum = 0.0;
 
-	int i;
+	unsigned i;
 	for(i = 0; i < component->nchildren ; i++)
 	{
 		if(starpu_sched_component_can_execute_task(component->children[i],task))
@@ -103,8 +104,9 @@ int starpu_sched_component_is_random(struct starpu_sched_component *component)
 	return component->push_task == random_push_task;
 }
 
-struct starpu_sched_component * starpu_sched_component_random_create(struct starpu_sched_tree *tree, void * arg STARPU_ATTRIBUTE_UNUSED)
+struct starpu_sched_component * starpu_sched_component_random_create(struct starpu_sched_tree *tree, void *arg)
 {
+	(void)arg;
 	struct starpu_sched_component * component = starpu_sched_component_create(tree, "random");
 	component->push_task = random_push_task;
 	return component;
