@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2014                                Inria
+ * Copyright (C) 2011-2014,2017                           Inria
  * Copyright (C) 2010-2012,2014-2017                      CNRS
- * Copyright (C) 2010-2017                                Université de Bordeaux
+ * Copyright (C) 2010-2019                                Université de Bordeaux
  * Copyright (C) 2011                                     Télécom-SudParis
  * Copyright (C) 2013                                     Simon Archipoff
  *
@@ -99,26 +99,64 @@ struct _starpu_worker_task_list
 	starpu_pthread_mutex_t mutex;
 };
 
-enum _starpu_worker_component_status
+/* This is called when a transfer request is actually pushed to the worker */
+static void _starpu_worker_task_list_transfer_started(struct _starpu_worker_task_list *l, struct starpu_task *task)
 {
-	COMPONENT_STATUS_SLEEPING,
-	COMPONENT_STATUS_RESET,
-	COMPONENT_STATUS_CHANGED
-};
+	double transfer_model = task->predicted_transfer;
+	if (isnan(transfer_model))
+		return;
+
+	/* We now start the transfer, move it from predicted to pipelined */
+	l->exp_len -= transfer_model;
+	l->pipeline_len += transfer_model;
+	l->exp_start = starpu_timing_now() + l->pipeline_len;
+	l->exp_end = l->exp_start + l->exp_len;
+}
+
+#ifdef STARPU_DEVEL
+#warning FIXME: merge with deque_modeling_policy_data_aware
+#endif
+/* This is called when a task is actually pushed to the worker (i.e. the transfer finished */
+static void _starpu_worker_task_list_started(struct _starpu_worker_task_list *l, struct starpu_task *task)
+{
+	double model = task->predicted;
+	double transfer_model = task->predicted_transfer;
+	if(!isnan(transfer_model))
+		/* The transfer is over, remove it from pipelined */
+		l->pipeline_len -= transfer_model;
+
+	if(!isnan(model))
+	{
+		/* We now start the computation, move it from predicted to pipelined */
+		l->exp_len -= model;
+		l->pipeline_len += model;
+		l->exp_start = starpu_timing_now() + l->pipeline_len;
+                l->exp_end= l->exp_start + l->exp_len;
+	}
+}
+
+/* This is called when a task is actually finished */
+static void _starpu_worker_task_list_finished(struct _starpu_worker_task_list *l, struct starpu_task *task)
+{
+	if(!isnan(task->predicted))
+		/* The execution is over, remove it from pipelined */
+		l->pipeline_len -= task->predicted;
+	l->exp_start = STARPU_MAX(starpu_timing_now() + l->pipeline_len, l->exp_start);
+	l->exp_end = l->exp_start + l->exp_len;
+}
+
 
 struct _starpu_worker_component_data
 {
 	union
 	{
-		struct
-		{
-			struct _starpu_worker * worker;
-			starpu_pthread_mutex_t lock;
-		};
-		struct _starpu_combined_worker * combined_worker;
+		struct _starpu_worker * worker;
+		struct {
+			unsigned worker_size;
+			unsigned workerids[STARPU_NMAXWORKERS];
+		} parallel_worker;
 	};
 	struct _starpu_worker_task_list * list;
-	enum _starpu_worker_component_status status;
 };
 
 /* this array store worker components */
@@ -138,6 +176,9 @@ static struct _starpu_worker_task_list * _starpu_worker_task_list_create(void)
 	memset(l, 0, sizeof(*l));
 	l->exp_len = l->pipeline_len = 0.0;
 	l->exp_start = l->exp_end = starpu_timing_now();
+	/* These are only for statistics */
+	STARPU_HG_DISABLE_CHECKING(l->exp_end);
+	STARPU_HG_DISABLE_CHECKING(l->exp_start);
 	STARPU_PTHREAD_MUTEX_INIT(&l->mutex,NULL);
 	return l;
 }
@@ -182,6 +223,41 @@ static void _starpu_worker_task_list_destroy(struct _starpu_worker_task_list * l
 	}
 }
 
+static inline void _starpu_worker_task_list_add(struct _starpu_worker_task_list * l, struct starpu_task *task)
+{
+	double predicted = task->predicted;
+	double predicted_transfer = task->predicted_transfer;
+	double end = l->exp_end;
+	const double now = starpu_timing_now();
+
+	/* Sometimes workers didn't take the tasks as early as we expected */
+	l->exp_start = STARPU_MAX(l->exp_start, now);
+
+	if (now + predicted_transfer < end)
+	{
+		/* We may hope that the transfer will be finished by
+		 * the start of the task. */
+		predicted_transfer = 0.0;
+	}
+	else
+	{
+		/* The transfer will not be finished by then, take the
+		 * remainder into account */
+		predicted_transfer = (now + predicted_transfer) - end;
+	}
+
+	if(!isnan(predicted_transfer))
+		l->exp_len += predicted_transfer;
+
+	if(!isnan(predicted))
+		l->exp_len += predicted;
+
+	l->exp_end = l->exp_start + l->exp_len;
+
+	task->predicted = predicted;
+	task->predicted_transfer = predicted_transfer;
+}
+
 static inline void _starpu_worker_task_list_push(struct _starpu_worker_task_list * l, struct _starpu_task_grid * t)
 {
 /* the task, ntasks, pntasks, left and right members of t are set by the caller */
@@ -194,40 +270,7 @@ static inline void _starpu_worker_task_list_push(struct _starpu_worker_task_list
 	l->last = t;
 	l->ntasks++;
 
-	double predicted = t->task->predicted;
-	double predicted_transfer = t->task->predicted_transfer;
-
-	/* Sometimes workers didn't take the tasks as early as we expected */
-	l->exp_start = STARPU_MAX(l->exp_start, starpu_timing_now());
-	l->exp_end = l->exp_start + l->exp_len;
-
-	if (starpu_timing_now() + predicted_transfer < l->exp_end)
-	{
-		/* We may hope that the transfer will be finished by
-		 * the start of the task. */
-		predicted_transfer = 0.0;
-	}
-	else
-	{
-		/* The transfer will not be finished by then, take the
-		 * remainder into account */
-		predicted_transfer = (starpu_timing_now() + predicted_transfer) - l->exp_end;
-	}
-
-	if(!isnan(predicted_transfer))
-	{
-		l->exp_end += predicted_transfer;
-		l->exp_len += predicted_transfer;
-	}
-
-	if(!isnan(predicted))
-	{
-		l->exp_end += predicted;
-		l->exp_len += predicted;
-	}
-
-	t->task->predicted = predicted;
-	t->task->predicted_transfer = predicted_transfer;
+	_starpu_worker_task_list_add(l, t->task);
 }
 
 /* recursively set left and right pointers to NULL */
@@ -259,8 +302,8 @@ static inline struct starpu_task * _starpu_worker_task_list_pop(struct _starpu_w
 {
  	if(!l->first)
 	{
+		l->exp_len = l->pipeline_len = 0.0;
 		l->exp_start = l->exp_end = starpu_timing_now();
-		l->exp_len = 0;
 		return NULL;
 	}
 	struct _starpu_task_grid * t = l->first;
@@ -314,27 +357,6 @@ struct _starpu_worker * _starpu_sched_component_worker_get_worker(struct starpu_
 	struct _starpu_worker_component_data * data = worker_component->data;
 	return data->worker;
 }
-struct _starpu_combined_worker * _starpu_sched_component_combined_worker_get_combined_worker(struct starpu_sched_component * worker_component)
-{
-	STARPU_ASSERT(starpu_sched_component_is_combined_worker(worker_component));
-	struct _starpu_worker_component_data * data = worker_component->data;
-	return data->combined_worker;
-}
-
-void _starpu_sched_component_lock_worker(unsigned sched_ctx_id, int workerid)
-{
-	STARPU_ASSERT(workerid >= 0 && workerid < (int) starpu_worker_get_count());
-	struct _starpu_worker_component_data * data = starpu_sched_component_worker_get(sched_ctx_id, workerid)->data;
-	STARPU_PTHREAD_MUTEX_LOCK(&data->lock);
-}
-void _starpu_sched_component_unlock_worker(unsigned sched_ctx_id, int workerid)
-{
-	STARPU_ASSERT(workerid >= 0  && workerid < (int)starpu_worker_get_count());
-	struct _starpu_worker_component_data * data = starpu_sched_component_worker_get(sched_ctx_id, workerid)->data;
-	STARPU_PTHREAD_MUTEX_UNLOCK(&data->lock);
-}
-
-
 
 /******************************************************************************
  *				Worker Components' Private Helper Functions			      	  *
@@ -342,68 +364,6 @@ void _starpu_sched_component_unlock_worker(unsigned sched_ctx_id, int workerid)
 
 
 
-/* Allows a worker to lock/unlock scheduling mutexes. Currently used in
- * self-defined can_push calls to allow can_pull calls to take those mutexes while the
- * current worker is pushing tasks on other workers (or itself).
- */
-static void _starpu_sched_component_worker_lock_scheduling(unsigned sched_ctx_id)
-{
-	unsigned workerid = starpu_worker_get_id_check();
-	starpu_pthread_mutex_t *sched_mutex;
-	starpu_pthread_cond_t *sched_cond;
-	starpu_worker_get_sched_condition(workerid, &sched_mutex, &sched_cond);
-	_starpu_sched_component_lock_worker(sched_ctx_id, workerid);
-	STARPU_PTHREAD_MUTEX_LOCK_SCHED(sched_mutex);
-}
-
-static void _starpu_sched_component_worker_unlock_scheduling(unsigned sched_ctx_id)
-{
-	unsigned workerid = starpu_worker_get_id_check();
-	starpu_pthread_mutex_t *sched_mutex;
-	starpu_pthread_cond_t *sched_cond;
-	starpu_worker_get_sched_condition(workerid, &sched_mutex, &sched_cond);
-	STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(sched_mutex);
-	_starpu_sched_component_unlock_worker(sched_ctx_id, workerid);
-}
-
-static void _starpu_sched_component_worker_set_sleep_status(struct starpu_sched_component * worker_component)
-{
-	STARPU_ASSERT(starpu_sched_component_is_worker(worker_component));
-	struct _starpu_worker_component_data * data = worker_component->data;
-	data->status = COMPONENT_STATUS_SLEEPING;
-}
-static void _starpu_sched_component_worker_set_changed_status(struct starpu_sched_component * worker_component)
-{
-	STARPU_ASSERT(starpu_sched_component_is_worker(worker_component));
-	struct _starpu_worker_component_data * data = worker_component->data;
-	data->status = COMPONENT_STATUS_CHANGED;
-}
-static void _starpu_sched_component_worker_reset_status(struct starpu_sched_component * worker_component)
-{
-	STARPU_ASSERT(starpu_sched_component_is_worker(worker_component));
-	struct _starpu_worker_component_data * data = worker_component->data;
-	data->status = COMPONENT_STATUS_RESET;
-}
-
-static int _starpu_sched_component_worker_is_reset_status(struct starpu_sched_component * worker_component)
-{
-	STARPU_ASSERT(starpu_sched_component_is_worker(worker_component));
-	struct _starpu_worker_component_data * data = worker_component->data;
-	return (data->status == COMPONENT_STATUS_RESET);
-}
-static int _starpu_sched_component_worker_is_changed_status(struct starpu_sched_component * worker_component)
-{
-	STARPU_ASSERT(starpu_sched_component_is_worker(worker_component));
-	struct _starpu_worker_component_data * data = worker_component->data;
-	return (data->status == COMPONENT_STATUS_CHANGED);
-}
-static int _starpu_sched_component_worker_is_sleeping_status(struct starpu_sched_component * worker_component)
-{
-	STARPU_ASSERT(starpu_sched_component_is_worker(worker_component));
-	struct _starpu_worker_component_data * data = worker_component->data;
-	return (data->status == COMPONENT_STATUS_SLEEPING);
-}
-
 #ifndef STARPU_NO_ASSERT
 static int _worker_consistant(struct starpu_sched_component * component)
 {
@@ -433,30 +393,11 @@ static int _worker_consistant(struct starpu_sched_component * component)
 
 
 
-static void simple_worker_can_pull(struct starpu_sched_component * worker_component)
+static int simple_worker_can_pull(struct starpu_sched_component * worker_component)
 {
-	(void) worker_component;
-	struct _starpu_worker * w = _starpu_sched_component_worker_get_worker(worker_component);
-	_starpu_sched_component_lock_worker(worker_component->tree->sched_ctx_id, w->workerid);
-	if(_starpu_sched_component_worker_is_reset_status(worker_component))
-		_starpu_sched_component_worker_set_changed_status(worker_component);
-
-	if(w->workerid == starpu_worker_get_id())
-	{
-		_starpu_sched_component_unlock_worker(worker_component->tree->sched_ctx_id, w->workerid);
-		return;
-	}
-	if(_starpu_sched_component_worker_is_sleeping_status(worker_component))
-	{
-		/* FIXME Ça n'a pas l'air de fonctionner */
-		starpu_pthread_mutex_t *sched_mutex;
-		starpu_pthread_cond_t *sched_cond;
-		starpu_worker_get_sched_condition(w->workerid, &sched_mutex, &sched_cond);
-		_starpu_sched_component_unlock_worker(worker_component->tree->sched_ctx_id, w->workerid);
-		starpu_wakeup_worker(w->workerid, sched_cond, sched_mutex);
-	}
-	else
-		_starpu_sched_component_unlock_worker(worker_component->tree->sched_ctx_id, w->workerid);
+	struct _starpu_worker * worker = _starpu_sched_component_worker_get_worker(worker_component);
+	int workerid = worker->workerid;
+	return starpu_wake_worker_relax_light(workerid);
 }
 
 static int simple_worker_push_task(struct starpu_sched_component * component, struct starpu_task *task)
@@ -471,96 +412,112 @@ static int simple_worker_push_task(struct starpu_sched_component * component, st
 	task->workerid = starpu_bitmap_first(component->workers);
 #if 1 /* dead lock problem? */
 	if (starpu_get_prefetch_flag() && !task->prefetched)
-	{
-		unsigned memory_node = starpu_worker_get_memory_node(task->workerid);
-		starpu_prefetch_task_input_on_node(task, memory_node);
-	}
+		starpu_prefetch_task_input_for(task, task->workerid);
 #endif
-	STARPU_PTHREAD_MUTEX_LOCK(&data->list->mutex);
-	_starpu_worker_task_list_push(data->list, t);
-	STARPU_PTHREAD_MUTEX_UNLOCK(&data->list->mutex);
+	struct _starpu_worker_task_list * list = data->list;
+	STARPU_COMPONENT_MUTEX_LOCK(&list->mutex);
+	_starpu_worker_task_list_push(list, t);
+	STARPU_COMPONENT_MUTEX_UNLOCK(&list->mutex);
 	simple_worker_can_pull(component);
 	return 0;
 }
 
-static struct starpu_task * simple_worker_pull_task(struct starpu_sched_component *component)
+static struct starpu_task * simple_worker_pull_task(struct starpu_sched_component *component, struct starpu_sched_component * to)
 {
 	unsigned workerid = starpu_worker_get_id_check();
+	struct _starpu_worker *worker = _starpu_get_worker_struct(workerid);
 	struct _starpu_worker_component_data * data = component->data;
 	struct _starpu_worker_task_list * list = data->list;
-	STARPU_PTHREAD_MUTEX_LOCK(&list->mutex);
-	struct starpu_task * task =  _starpu_worker_task_list_pop(list);
-	STARPU_PTHREAD_MUTEX_UNLOCK(&list->mutex);
-	if(task)
-	{
-		starpu_push_task_end(task);
-		return task;
-	}
-	_starpu_sched_component_lock_worker(component->tree->sched_ctx_id, workerid);
-	int i;
+	struct starpu_task * task;
+	unsigned i;
+	int n_tries = 0;
 	do
 	{
-		_starpu_sched_component_worker_reset_status(component);
+		const double now = starpu_timing_now();
+		/* do not reset state_keep_awake here has it may hide tasks in worker->local_tasks */
+		n_tries++;
+		STARPU_COMPONENT_MUTEX_LOCK(&list->mutex);
+		/* Take the opportunity to update start time */
+		data->list->exp_start = STARPU_MAX(now, data->list->exp_start);
+		data->list->exp_end = data->list->exp_start + data->list->exp_len;
+		task =  _starpu_worker_task_list_pop(list);
+		if(task)
+		{
+			_starpu_worker_task_list_transfer_started(list, task);
+			STARPU_COMPONENT_MUTEX_UNLOCK(&list->mutex);
+			starpu_push_task_end(task);
+			goto ret;
+		}
+		STARPU_COMPONENT_MUTEX_UNLOCK(&list->mutex);
 		for(i=0; i < component->nparents; i++)
 		{
 			if(component->parents[i] == NULL)
 				continue;
 			else
 			{
-				_starpu_sched_component_worker_unlock_scheduling(component->tree->sched_ctx_id);
 				task = starpu_sched_component_pull_task(component->parents[i],component);
-				_starpu_sched_component_worker_lock_scheduling(component->tree->sched_ctx_id);
 				if(task)
 					break;
 			}
 		}
 	}
-	while((!task) && _starpu_sched_component_worker_is_changed_status(component));
-	_starpu_sched_component_worker_set_sleep_status(component);
-	/* Remplacer par le sched condition */
-	_starpu_sched_component_unlock_worker(component->tree->sched_ctx_id, workerid);
+	while((!task) && worker->state_keep_awake && n_tries < 2);
 	if(!task)
-		return NULL;
+		goto ret;
 	if(task->cl->type == STARPU_SPMD)
 	{
 		if(!starpu_worker_is_combined_worker(workerid))
 		{
+			STARPU_COMPONENT_MUTEX_LOCK(&list->mutex);
+			_starpu_worker_task_list_add(list, task);
+			_starpu_worker_task_list_transfer_started(list, task);
+			STARPU_COMPONENT_MUTEX_UNLOCK(&list->mutex);
 			starpu_push_task_end(task);
-			return task;
+			goto ret;
 		}
 		struct starpu_sched_component * combined_worker_component = starpu_sched_component_worker_get(component->tree->sched_ctx_id, workerid);
 		starpu_sched_component_push_task(component, combined_worker_component, task);
 		/* we have pushed a task in queue, so can make a recursive call */
-		return simple_worker_pull_task(component);
+		task = simple_worker_pull_task(component, to);
+		goto ret;
 
 	}
 	if(task)
+	{
+		STARPU_COMPONENT_MUTEX_LOCK(&list->mutex);
+		_starpu_worker_task_list_add(list, task);
+		_starpu_worker_task_list_transfer_started(list, task);
+		STARPU_COMPONENT_MUTEX_UNLOCK(&list->mutex);
 		starpu_push_task_end(task);
+	}
+ret:
 	return task;
 }
 
 static double simple_worker_estimated_end(struct starpu_sched_component * component)
 {
 	struct _starpu_worker_component_data * data = component->data;
-	STARPU_PTHREAD_MUTEX_LOCK(&data->list->mutex);
-	data->list->exp_start = STARPU_MAX(starpu_timing_now(), data->list->exp_start);
-	double tmp = data->list->exp_end = data->list->exp_start + data->list->exp_len;
-	STARPU_PTHREAD_MUTEX_UNLOCK(&data->list->mutex);
-	return tmp;
+	double now = starpu_timing_now();
+	if (now + data->list->pipeline_len > data->list->exp_start )
+	{
+		data->list->exp_start = now + data->list->pipeline_len;
+		data->list->exp_end = data->list->exp_start + data->list->exp_len;
+	}
+	return data->list->exp_end;
 }
 
 static double simple_worker_estimated_load(struct starpu_sched_component * component)
 {
 	struct _starpu_worker * worker = _starpu_sched_component_worker_get_worker(component);
 	int nb_task = 0;
-	STARPU_PTHREAD_MUTEX_LOCK(&worker->mutex);
+	STARPU_COMPONENT_MUTEX_LOCK(&worker->mutex);
 	struct starpu_task_list list = worker->local_tasks;
 	struct starpu_task * task;
 	for(task = starpu_task_list_front(&list);
 	    task != starpu_task_list_end(&list);
 	    task = starpu_task_list_next(task))
 		nb_task++;
-	STARPU_PTHREAD_MUTEX_UNLOCK(&worker->mutex);
+	STARPU_COMPONENT_MUTEX_UNLOCK(&worker->mutex);
 	struct _starpu_worker_component_data * d = component->data;
 	struct _starpu_worker_task_list * l = d->list;
 	int ntasks_in_fifo = l ? l->ntasks : 0;
@@ -573,8 +530,6 @@ static void _worker_component_deinit_data(struct starpu_sched_component * compon
 {
 	struct _starpu_worker_component_data * d = component->data;
 	_starpu_worker_task_list_destroy(d->list);
-	if(starpu_sched_component_is_simple_worker(component))
-		STARPU_PTHREAD_MUTEX_DESTROY(&d->lock);
 	int i, j;
 	for(j = 0; j < STARPU_NMAX_SCHED_CTXS; j++)
 	for(i = 0; i < STARPU_NMAXWORKERS; i++)
@@ -604,8 +559,6 @@ static struct starpu_sched_component * starpu_sched_component_worker_create(stru
 	memset(data, 0, sizeof(*data));
 
 	data->worker = worker;
-	STARPU_PTHREAD_MUTEX_INIT(&data->lock,NULL);
-	data->status = COMPONENT_STATUS_SLEEPING;
 	data->list = _starpu_worker_task_list_create();
 	component->data = data;
 
@@ -623,7 +576,7 @@ static struct starpu_sched_component * starpu_sched_component_worker_create(stru
 #ifdef STARPU_HAVE_HWLOC
 	struct _starpu_machine_config *config = _starpu_get_machine_config();
 	struct _starpu_machine_topology *topology = &config->topology;
-	hwloc_obj_t obj = hwloc_get_obj_by_depth(topology->hwtopology, config->cpu_depth, worker->bindid);
+	hwloc_obj_t obj = hwloc_get_obj_by_depth(topology->hwtopology, config->pu_depth, worker->bindid);
 	STARPU_ASSERT(obj);
 	component->obj = obj;
 #endif
@@ -640,78 +593,69 @@ static struct starpu_sched_component * starpu_sched_component_worker_create(stru
 
 
 
-static void combined_worker_can_pull(struct starpu_sched_component * component)
+static int combined_worker_can_pull(struct starpu_sched_component * component)
 {
 	(void) component;
 	STARPU_ASSERT(starpu_sched_component_is_combined_worker(component));
 	struct _starpu_worker_component_data * data = component->data;
-	unsigned workerid = starpu_worker_get_id_check();
-	int i;
-	for(i = 0; i < data->combined_worker->worker_size; i++)
+	int workerid = starpu_worker_get_id();
+	unsigned i;
+	for(i = 0; i < data->parallel_worker.worker_size; i++)
 	{
-		if((unsigned) i == workerid)
+		int target = data->parallel_worker.workerids[i];
+		if(target == workerid)
 			continue;
-		int worker = data->combined_worker->combined_workerid[i];
-		_starpu_sched_component_lock_worker(component->tree->sched_ctx_id, worker);
-		if(_starpu_sched_component_worker_is_sleeping_status(component))
-		{
-			starpu_pthread_mutex_t *sched_mutex;
-			starpu_pthread_cond_t *sched_cond;
-			starpu_worker_get_sched_condition(worker, &sched_mutex, &sched_cond);
-			starpu_wakeup_worker(worker, sched_cond, sched_mutex);
-		}
-		if(_starpu_sched_component_worker_is_reset_status(component))
-			_starpu_sched_component_worker_set_changed_status(component);
-
-		_starpu_sched_component_unlock_worker(component->tree->sched_ctx_id, worker);
+		if (starpu_wake_worker_relax_light(target))
+			return 1;
 	}
+	return 0;
 }
 
 static int combined_worker_push_task(struct starpu_sched_component * component, struct starpu_task *task)
 {
 	STARPU_ASSERT(starpu_sched_component_is_combined_worker(component));
 	struct _starpu_worker_component_data * data = component->data;
-	STARPU_ASSERT(data->combined_worker && !data->worker);
-	struct _starpu_combined_worker  * combined_worker = data->combined_worker;
-	STARPU_ASSERT(combined_worker->worker_size >= 1);
-	struct _starpu_task_grid * task_alias[combined_worker->worker_size];
+	STARPU_ASSERT(data->parallel_worker.worker_size >= 1);
+	struct _starpu_task_grid * task_alias[data->parallel_worker.worker_size];
 	starpu_parallel_task_barrier_init(task, starpu_bitmap_first(component->workers));
 	task_alias[0] = _starpu_task_grid_create();
 	task_alias[0]->task = starpu_task_dup(task);
-	task_alias[0]->task->workerid = combined_worker->combined_workerid[0];
+	task_alias[0]->task->workerid = data->parallel_worker.workerids[0];
 	task_alias[0]->task->destroy = 1;
 	task_alias[0]->left = NULL;
-	task_alias[0]->ntasks = combined_worker->worker_size;
-	int i;
-	for(i = 1; i < combined_worker->worker_size; i++)
+	task_alias[0]->ntasks = data->parallel_worker.worker_size;
+	_STARPU_TRACE_JOB_PUSH(task_alias[0]->task, task_alias[0]->task->priority > 0);
+	unsigned i;
+	for(i = 1; i < data->parallel_worker.worker_size; i++)
 	{
 		task_alias[i] = _starpu_task_grid_create();
 		task_alias[i]->task = starpu_task_dup(task);
 		task_alias[i]->task->destroy = 1;
-		task_alias[i]->task->workerid = combined_worker->combined_workerid[i];
+		task_alias[i]->task->workerid = data->parallel_worker.workerids[i];
 		task_alias[i]->left = task_alias[i-1];
 		task_alias[i - 1]->right = task_alias[i];
 		task_alias[i]->pntasks = &(task_alias[0]->ntasks);
+		_STARPU_TRACE_JOB_PUSH(task_alias[i]->task, task_alias[i]->task->priority > 0);
 	}
 
 	starpu_pthread_mutex_t * mutex_to_unlock = NULL;
 	i = 0;
 	do
 	{
-		struct starpu_sched_component * worker_component = starpu_sched_component_worker_get(component->tree->sched_ctx_id, combined_worker->combined_workerid[i]);
+		struct starpu_sched_component * worker_component = starpu_sched_component_worker_get(component->tree->sched_ctx_id, data->parallel_worker.workerids[i]);
 		struct _starpu_worker_component_data * worker_data = worker_component->data;
 		struct _starpu_worker_task_list * list = worker_data->list;
-		STARPU_PTHREAD_MUTEX_LOCK(&list->mutex);
+		STARPU_COMPONENT_MUTEX_LOCK(&list->mutex);
 		if(mutex_to_unlock)
-			STARPU_PTHREAD_MUTEX_UNLOCK(mutex_to_unlock);
+			STARPU_COMPONENT_MUTEX_UNLOCK(mutex_to_unlock);
 		mutex_to_unlock = &list->mutex;
 
 		_starpu_worker_task_list_push(list, task_alias[i]);
 		i++;
 	}
-	while(i < combined_worker->worker_size);
+	while(i < data->parallel_worker.worker_size);
 
-	STARPU_PTHREAD_MUTEX_UNLOCK(mutex_to_unlock);
+	STARPU_COMPONENT_MUTEX_UNLOCK(mutex_to_unlock);
 
 	int workerid = starpu_worker_get_id();
 	if(-1 == workerid)
@@ -720,39 +664,35 @@ static int combined_worker_push_task(struct starpu_sched_component * component,
 	}
 	else
 	{
-		starpu_pthread_mutex_t *worker_sched_mutex;
-		starpu_pthread_cond_t *worker_sched_cond;
-		starpu_worker_get_sched_condition(workerid, &worker_sched_mutex, &worker_sched_cond);
-		STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(worker_sched_mutex);
-
 		/* wake up all other workers of combined worker */
-		for(i = 0; i < combined_worker->worker_size; i++)
+		for(i = 0; i < data->parallel_worker.worker_size; i++)
 		{
-			struct starpu_sched_component * worker_component = starpu_sched_component_worker_get(component->tree->sched_ctx_id, combined_worker->combined_workerid[i]);
+			struct starpu_sched_component * worker_component = starpu_sched_component_worker_get(component->tree->sched_ctx_id, data->parallel_worker.workerids[i]);
 			simple_worker_can_pull(worker_component);
 		}
 
 		combined_worker_can_pull(component);
-
-		STARPU_PTHREAD_MUTEX_LOCK_SCHED(worker_sched_mutex);
 	}
 
 	return 0;
 }
 
+static struct starpu_task *combined_worker_pull_task(struct starpu_sched_component * from STARPU_ATTRIBUTE_UNUSED, struct starpu_sched_component * to STARPU_ATTRIBUTE_UNUSED)
+{
+	return NULL;
+}
+
 static double combined_worker_estimated_end(struct starpu_sched_component * component)
 {
 	STARPU_ASSERT(starpu_sched_component_is_combined_worker(component));
-	struct _starpu_worker_component_data * data = component->data;
-	struct _starpu_combined_worker * combined_worker = data->combined_worker;
+	struct _starpu_worker_component_data * d = component->data;
 	double max = 0.0;
-	int i;
-	for(i = 0; i < combined_worker->worker_size; i++)
+	unsigned i;
+	for(i = 0; i < d->parallel_worker.worker_size; i++)
 	{
-		data = _worker_components[component->tree->sched_ctx_id][combined_worker->combined_workerid[i]]->data;
-		STARPU_PTHREAD_MUTEX_LOCK(&data->list->mutex);
+		struct _starpu_worker_component_data * data;
+		data = _worker_components[component->tree->sched_ctx_id][d->parallel_worker.workerids[i]]->data;
 		double tmp = data->list->exp_end;
-		STARPU_PTHREAD_MUTEX_UNLOCK(&data->list->mutex);
 		max = tmp > max ? tmp : max;
 	}
 	return max;
@@ -761,17 +701,43 @@ static double combined_worker_estimated_end(struct starpu_sched_component * comp
 static double combined_worker_estimated_load(struct starpu_sched_component * component)
 {
 	struct _starpu_worker_component_data * d = component->data;
-	struct _starpu_combined_worker * c = d->combined_worker;
 	double load = 0;
-	int i;
-	for(i = 0; i < c->worker_size; i++)
+	unsigned i;
+	for(i = 0; i < d->parallel_worker.worker_size; i++)
 	{
-		struct starpu_sched_component * n = starpu_sched_component_worker_get(component->tree->sched_ctx_id, c->combined_workerid[i]);
+		struct starpu_sched_component * n = starpu_sched_component_worker_get(component->tree->sched_ctx_id, d->parallel_worker.workerids[i]);
 		load += n->estimated_load(n);
 	}
 	return load;
 }
 
+struct starpu_sched_component *starpu_sched_component_parallel_worker_create(struct starpu_sched_tree *tree, unsigned nworkers, unsigned *workers)
+{
+	struct starpu_sched_component * component = starpu_sched_component_create(tree, "combined_worker");
+
+	struct _starpu_worker_component_data *data;
+	_STARPU_MALLOC(data, sizeof(*data));
+	memset(data, 0, sizeof(*data));
+	STARPU_ASSERT(nworkers <= STARPU_NMAXWORKERS);
+	STARPU_ASSERT(nworkers <= starpu_worker_get_count());
+	data->parallel_worker.worker_size = nworkers;
+	memcpy(data->parallel_worker.workerids, workers, nworkers * sizeof(unsigned));
+
+	component->data = data;
+	component->push_task = combined_worker_push_task;
+	component->pull_task = combined_worker_pull_task;
+	component->estimated_end = combined_worker_estimated_end;
+	component->estimated_load = combined_worker_estimated_load;
+	component->can_pull = combined_worker_can_pull;
+	component->deinit_data = _worker_component_deinit_data;
+	
+	unsigned i;
+	for (i = 0; i < nworkers; i++)
+		starpu_sched_component_connect(component, starpu_sched_component_worker_get(tree->sched_ctx_id, workers[i]));
+
+	return component;
+}
+
 static struct starpu_sched_component  * starpu_sched_component_combined_worker_create(struct starpu_sched_tree *tree, int workerid)
 {
 	STARPU_ASSERT(workerid >= 0 && workerid <  STARPU_NMAXWORKERS);
@@ -782,53 +748,46 @@ static struct starpu_sched_component  * starpu_sched_component_combined_worker_c
 	struct _starpu_combined_worker * combined_worker = _starpu_get_combined_worker_struct(workerid);
 	if(combined_worker == NULL)
 		return NULL;
-	struct starpu_sched_component * component = starpu_sched_component_create(tree, "combined_worker");
-	struct _starpu_worker_component_data *data;
-	_STARPU_MALLOC(data, sizeof(*data));
-	memset(data, 0, sizeof(*data));
-	data->combined_worker = combined_worker;
-	data->status = COMPONENT_STATUS_SLEEPING;
 
-	component->data = data;
-	component->push_task = combined_worker_push_task;
-	component->pull_task = NULL;
-	component->estimated_end = combined_worker_estimated_end;
-	component->estimated_load = combined_worker_estimated_load;
-	component->can_pull = combined_worker_can_pull;
-	component->deinit_data = _worker_component_deinit_data;
+	struct starpu_sched_component *component = starpu_sched_component_parallel_worker_create(tree, combined_worker->worker_size, (unsigned *) combined_worker->combined_workerid);
+
 	starpu_bitmap_set(component->workers, workerid);
 	starpu_bitmap_or(component->workers_in_ctx, component->workers);
+
 	_worker_components[tree->sched_ctx_id][workerid] = component;
 
+	/*
 #ifdef STARPU_HAVE_HWLOC
+	struct _starpu_worker_component_data * data = component->data;
 	struct _starpu_machine_config *config = _starpu_get_machine_config();
 	struct _starpu_machine_topology *topology = &config->topology;
-	hwloc_obj_t obj = hwloc_get_obj_by_depth(topology->hwtopology, config->cpu_depth, combined_worker->combined_workerid[0]);
+	struct _starpu_worker *worker = _starpu_get_worker_struct(data->parallel_worker.workerids[0]);
+	hwloc_obj_t obj = hwloc_get_obj_by_depth(topology->hwtopology, config->pu_depth, worker->bindid);
 	STARPU_ASSERT(obj);
 	component->obj = obj;
 #endif
+	*/
 	return component;
 }
 
 
-
 /******************************************************************************
  *			Worker Components' Public Helper Functions (Part 2)			      *
  *****************************************************************************/
 
 
 
-void _starpu_sched_component_lock_all_workers(unsigned sched_ctx_id)
+void _starpu_sched_component_lock_all_workers(void)
 {
 	unsigned i;
 	for(i = 0; i < starpu_worker_get_count(); i++)
-		_starpu_sched_component_lock_worker(sched_ctx_id, i);
+		starpu_worker_lock(i);
 }
-void _starpu_sched_component_unlock_all_workers(unsigned sched_ctx_id)
+void _starpu_sched_component_unlock_all_workers(void)
 {
 	unsigned i;
 	for(i = 0; i < starpu_worker_get_count(); i++)
-		_starpu_sched_component_unlock_worker(sched_ctx_id, i);
+		starpu_worker_unlock(i);
 }
 
 void _starpu_sched_component_workers_destroy(void)
@@ -849,50 +808,25 @@ int starpu_sched_component_worker_get_workerid(struct starpu_sched_component * w
 	return starpu_bitmap_first(worker_component->workers);
 }
 
-void starpu_sched_component_worker_pre_exec_hook(struct starpu_task * task)
+void starpu_sched_component_worker_pre_exec_hook(struct starpu_task * task, unsigned sched_ctx_id STARPU_ATTRIBUTE_UNUSED)
 {
-	double model = task->predicted;
-	double transfer_model = task->predicted_transfer;
-
-	if(!isnan(task->predicted) || !isnan(task->predicted_transfer))
-	{
-		unsigned sched_ctx_id = task->sched_ctx;
-		struct _starpu_worker_task_list * list = _worker_get_list(sched_ctx_id);
-		STARPU_PTHREAD_MUTEX_LOCK(&list->mutex);
-
-		list->exp_start = STARPU_MAX(starpu_timing_now(), list->exp_start);
-
-		/* The transfer is over, get rid of it in the completion
-		 * prediction */
-		if (!isnan(transfer_model))
-			list->exp_len -= transfer_model;
-
-		if (!isnan(model))
-		{
-			/* We now start the computation, get rid of it in the
-			 * completion prediction */
-			list->exp_len -= model;
-			list->exp_start += model;
-		}
-		
-		if(list->ntasks == 0)
-			list->exp_len = 0.0;
-
-		list->exp_end = list->exp_start + list->exp_len;
-		STARPU_PTHREAD_MUTEX_UNLOCK(&list->mutex);
-	}
+	struct _starpu_worker_task_list * list = _worker_get_list(sched_ctx_id);
+	const double now = starpu_timing_now();
+	STARPU_COMPONENT_MUTEX_LOCK(&list->mutex);
+	_starpu_worker_task_list_started(list, task);
+	/* Take the opportunity to update start time */
+	list->exp_start = STARPU_MAX(now + list->pipeline_len, list->exp_start);
+	STARPU_COMPONENT_MUTEX_UNLOCK(&list->mutex);
 }
 
-void starpu_sched_component_worker_post_exec_hook(struct starpu_task * task)
+void starpu_sched_component_worker_post_exec_hook(struct starpu_task * task, unsigned sched_ctx_id STARPU_ATTRIBUTE_UNUSED)
 {
 	if(task->execute_on_a_specific_worker)
 		return;
-	unsigned sched_ctx_id = task->sched_ctx;
 	struct _starpu_worker_task_list * list = _worker_get_list(sched_ctx_id);
-	STARPU_PTHREAD_MUTEX_LOCK(&list->mutex);
-	list->exp_start = starpu_timing_now();
-	list->exp_end = list->exp_start + list->exp_len;
-	STARPU_PTHREAD_MUTEX_UNLOCK(&list->mutex);
+	STARPU_COMPONENT_MUTEX_LOCK(&list->mutex);
+	_starpu_worker_task_list_finished(list, task);
+	STARPU_COMPONENT_MUTEX_UNLOCK(&list->mutex);
 }
 
 int starpu_sched_component_is_simple_worker(struct starpu_sched_component * component)
@@ -916,16 +850,26 @@ struct starpu_sched_component * starpu_sched_component_worker_get(unsigned sched
 {
 	STARPU_ASSERT(workerid >= 0 && workerid < STARPU_NMAXWORKERS);
 	/* we may need to take a mutex here */
-	if(_worker_components[sched_ctx][workerid])
-		return _worker_components[sched_ctx][workerid];
-	else
-	{
-		struct starpu_sched_component * component;
-		if(workerid < (int) starpu_worker_get_count())
-			component = starpu_sched_component_worker_create(starpu_sched_tree_get(sched_ctx), workerid);
-		else
-			component = starpu_sched_component_combined_worker_create(starpu_sched_tree_get(sched_ctx), workerid);
-		_worker_components[sched_ctx][workerid] = component;
-		return component;
-	}
+	if (!_worker_components[sched_ctx][workerid])
+		return starpu_sched_component_worker_new(sched_ctx, workerid);
+	return _worker_components[sched_ctx][workerid];
 }
+
+struct starpu_sched_component * starpu_sched_component_worker_new(unsigned sched_ctx, int workerid)
+{
+	STARPU_ASSERT(workerid >= 0 && workerid < STARPU_NMAXWORKERS);
+	/* we may need to take a mutex here */
+	if (_worker_components[sched_ctx][workerid])
+		return _worker_components[sched_ctx][workerid];
+	struct starpu_sched_component * component;
+	if(workerid < (int) starpu_worker_get_count())
+		component = starpu_sched_component_worker_create(starpu_sched_tree_get(sched_ctx), workerid);
+	else
+		component = starpu_sched_component_combined_worker_create(starpu_sched_tree_get(sched_ctx), workerid);
+	_worker_components[sched_ctx][workerid] = component;
+	return component;
+}
+
+
+
+
