@@ -1,8 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2016  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013, 2015, 2017  CNRS
- * Copyright (C) 2011  INRIA
+ * Copyright (C) 2011-2013,2015-2017                      Inria
+ * Copyright (C) 2008-2019                                Université de Bordeaux
+ * Copyright (C) 2010-2013,2015-2017                      CNRS
+ * Copyright (C) 2016                                     Uppsala University
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,32 +20,22 @@
 /*
  *	This is policy where every worker use the same JOB QUEUE, but taking
  *	task priorities into account
+ *
+ *	TODO: merge with eager, after checking the scalability
  */
 
 #include <starpu.h>
 #include <starpu_scheduler.h>
 #include <starpu_bitmap.h>
+#include "prio_deque.h"
+#include <limits.h>
 
 #include <common/fxt.h>
-
-#define DEFAULT_MIN_LEVEL	(-5)
-#define DEFAULT_MAX_LEVEL	(+5)
-
-struct _starpu_priority_taskq
-{
-	int min_prio;
-	int max_prio;
-	/* the actual lists
-	 *	taskq[p] is for priority [p - STARPU_MIN_PRIO] */
-	struct starpu_task_list *taskq;
-	unsigned *ntasks;
-
-	unsigned total_ntasks;
-};
+#include <core/workers.h>
 
 struct _starpu_eager_central_prio_data
 {
-	struct _starpu_priority_taskq *taskq;
+	struct _starpu_prio_deque taskq;
 	starpu_pthread_mutex_t policy_mutex;
 	struct starpu_bitmap *waiters;
 };
@@ -53,68 +44,27 @@ struct _starpu_eager_central_prio_data
  * Centralized queue with priorities
  */
 
-static struct _starpu_priority_taskq *_starpu_create_priority_taskq(int min_prio, int max_prio)
-{
-	struct _starpu_priority_taskq *central_queue;
-
-	_STARPU_MALLOC(central_queue, sizeof(struct _starpu_priority_taskq));
-	central_queue->min_prio = min_prio;
-	central_queue->max_prio = max_prio;
-	central_queue->total_ntasks = 0;
-	_STARPU_MALLOC(central_queue->taskq, (max_prio-min_prio+1) * sizeof(struct starpu_task_list));
-	_STARPU_MALLOC(central_queue->ntasks, (max_prio-min_prio+1) * sizeof(unsigned));
-
-	int prio;
-	for (prio = 0; prio < (max_prio-min_prio+1); prio++)
-	{
-		starpu_task_list_init(&central_queue->taskq[prio]);
-		central_queue->ntasks[prio] = 0;
-	}
-
-	return central_queue;
-}
-
-static void _starpu_destroy_priority_taskq(struct _starpu_priority_taskq *priority_queue)
-{
-	free(priority_queue->ntasks);
-	free(priority_queue->taskq);
-	free(priority_queue);
-}
-
 static void initialize_eager_center_priority_policy(unsigned sched_ctx_id)
 {
-	if (starpu_get_env_number_default("STARPU_WORKER_TREE", 0))
-	{
-#ifdef STARPU_HAVE_HWLOC
-		starpu_sched_ctx_create_worker_collection(sched_ctx_id, STARPU_WORKER_TREE);
-#else
-		_STARPU_DISP("STARPU_WORKER_TREE ignored, please rebuild StarPU with hwloc support to enable it.");
-		starpu_sched_ctx_create_worker_collection(sched_ctx_id, STARPU_WORKER_LIST);
-#endif
-	}
-	else
-		starpu_sched_ctx_create_worker_collection(sched_ctx_id, STARPU_WORKER_LIST);
-
 	struct _starpu_eager_central_prio_data *data;
 	_STARPU_MALLOC(data, sizeof(struct _starpu_eager_central_prio_data));
 
-	/* In this policy, we support more than two levels of priority. */
-
-	if (starpu_sched_ctx_min_priority_is_set(sched_ctx_id) == 0)
-		starpu_sched_ctx_set_min_priority(sched_ctx_id, DEFAULT_MIN_LEVEL);
-	if (starpu_sched_ctx_max_priority_is_set(sched_ctx_id) == 0)
-		starpu_sched_ctx_set_max_priority(sched_ctx_id, DEFAULT_MAX_LEVEL);
-
 	/* only a single queue (even though there are several internaly) */
-	data->taskq = _starpu_create_priority_taskq(starpu_sched_ctx_get_min_priority(sched_ctx_id), starpu_sched_ctx_get_max_priority(sched_ctx_id));
+	_starpu_prio_deque_init(&data->taskq);
 	data->waiters = starpu_bitmap_create();
 
 	/* Tell helgrind that it's fine to check for empty fifo in
 	 * _starpu_priority_pop_task without actual mutex (it's just an
 	 * integer) */
-	STARPU_HG_DISABLE_CHECKING(data->taskq->total_ntasks);
+	STARPU_HG_DISABLE_CHECKING(data->taskq.ntasks);
 	starpu_sched_ctx_set_policy_data(sched_ctx_id, (void*)data);
 	STARPU_PTHREAD_MUTEX_INIT(&data->policy_mutex, NULL);
+
+	/* The application may use any integer */
+	if (starpu_sched_ctx_min_priority_is_set(sched_ctx_id) == 0)
+		starpu_sched_ctx_set_min_priority(sched_ctx_id, INT_MIN);
+	if (starpu_sched_ctx_max_priority_is_set(sched_ctx_id) == 0)
+		starpu_sched_ctx_set_max_priority(sched_ctx_id, INT_MAX);
 }
 
 static void deinitialize_eager_center_priority_policy(unsigned sched_ctx_id)
@@ -123,10 +73,9 @@ static void deinitialize_eager_center_priority_policy(unsigned sched_ctx_id)
 	struct _starpu_eager_central_prio_data *data = (struct _starpu_eager_central_prio_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
 
 	/* deallocate the job queue */
-	_starpu_destroy_priority_taskq(data->taskq);
+	_starpu_prio_deque_destroy(&data->taskq);
 	starpu_bitmap_destroy(data->waiters);
 
-	starpu_sched_ctx_delete_worker_collection(sched_ctx_id);
 	STARPU_PTHREAD_MUTEX_DESTROY(&data->policy_mutex);
 	free(data);
 }
@@ -135,29 +84,34 @@ static int _starpu_priority_push_task(struct starpu_task *task)
 {
 	unsigned sched_ctx_id = task->sched_ctx;
 	struct _starpu_eager_central_prio_data *data = (struct _starpu_eager_central_prio_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
-	struct _starpu_priority_taskq *taskq = data->taskq;
+	struct _starpu_prio_deque *taskq = &data->taskq;
 
+	starpu_worker_relax_on();
 	STARPU_PTHREAD_MUTEX_LOCK(&data->policy_mutex);
-	unsigned priolevel = task->priority - starpu_sched_ctx_get_min_priority(sched_ctx_id);
-	
-	STARPU_ASSERT_MSG(task->priority >= starpu_sched_ctx_get_min_priority(sched_ctx_id) &&
-			  task->priority <= starpu_sched_ctx_get_max_priority(sched_ctx_id), "task priority %d is not between minimum %d and maximum %d\n", task->priority, starpu_sched_ctx_get_min_priority(sched_ctx_id), starpu_sched_ctx_get_max_priority(sched_ctx_id));
+	starpu_worker_relax_off();
+	_starpu_prio_deque_push_back_task(taskq, task);
+
+	if (_starpu_get_nsched_ctxs() > 1)
+	{
+		starpu_worker_relax_on();
+		_starpu_sched_ctx_lock_write(sched_ctx_id);
+		starpu_worker_relax_off();
+		starpu_sched_ctx_list_task_counters_increment_all_ctx_locked(task, sched_ctx_id);
+		_starpu_sched_ctx_unlock_write(sched_ctx_id);
+	}
 
-	starpu_task_list_push_back(&taskq->taskq[priolevel], task);
-	taskq->ntasks[priolevel]++;
-	taskq->total_ntasks++;
 	starpu_push_task_end(task);
 
 	/*if there are no tasks block */
 	/* wake people waiting for a task */
 	struct starpu_worker_collection *workers = starpu_sched_ctx_get_worker_collection(sched_ctx_id);
-	
+
 	struct starpu_sched_ctx_iterator it;
 #ifndef STARPU_NON_BLOCKING_DRIVERS
 	char dowake[STARPU_NMAXWORKERS] = { 0 };
 #endif
 
-	workers->init_iterator(workers, &it);
+	workers->init_iterator_for_parallel_tasks(workers, &it, task);
 	while(workers->has_next(workers, &it))
 	{
 		unsigned worker = workers->get_next(workers, &it);
@@ -183,7 +137,7 @@ static int _starpu_priority_push_task(struct starpu_task *task)
 	/* Let the task free */
 	STARPU_PTHREAD_MUTEX_UNLOCK(&data->policy_mutex);
 
-#ifndef STARPU_NON_BLOCKING_DRIVERS
+#if !defined(STARPU_NON_BLOCKING_DRIVERS) || defined(STARPU_SIMGRID)
 	/* Now that we have a list of potential workers, try to wake one */
 
 	workers->init_iterator(workers, &it);
@@ -191,7 +145,7 @@ static int _starpu_priority_push_task(struct starpu_task *task)
 	{
 		unsigned worker = workers->get_next(workers, &it);
 		if (dowake[worker])
-			if (starpu_wake_worker(worker))
+			if (starpu_wake_worker_relax_light(worker))
 				break; // wake up a single worker
 	}
 #endif
@@ -201,65 +155,35 @@ static int _starpu_priority_push_task(struct starpu_task *task)
 
 static struct starpu_task *_starpu_priority_pop_task(unsigned sched_ctx_id)
 {
-	struct starpu_task *chosen_task = NULL, *task, *nexttask;
+	struct starpu_task *chosen_task;
 	unsigned workerid = starpu_worker_get_id_check();
 	int skipped = 0;
 
 	struct _starpu_eager_central_prio_data *data = (struct _starpu_eager_central_prio_data*)starpu_sched_ctx_get_policy_data(sched_ctx_id);
 
-	struct _starpu_priority_taskq *taskq = data->taskq;
+	struct _starpu_prio_deque *taskq = &data->taskq;
 
-	/* block until some event happens */
 	/* Here helgrind would shout that this is unprotected, this is just an
 	 * integer access, and we hold the sched mutex, so we can not miss any
 	 * wake up. */
-	if (!STARPU_RUNNING_ON_VALGRIND && taskq->total_ntasks == 0)
+	if (!STARPU_RUNNING_ON_VALGRIND && _starpu_prio_deque_is_empty(taskq))
+	{
 		return NULL;
+	}
 
 #ifdef STARPU_NON_BLOCKING_DRIVERS
 	if (!STARPU_RUNNING_ON_VALGRIND && starpu_bitmap_get(data->waiters, workerid))
 		/* Nobody woke us, avoid bothering the mutex */
+	{
 		return NULL;
+	}
 #endif
 
-	/* release this mutex before trying to wake up other workers */
-	starpu_pthread_mutex_t *curr_sched_mutex;
-	starpu_pthread_cond_t *curr_sched_cond;
-	starpu_worker_get_sched_condition(workerid, &curr_sched_mutex, &curr_sched_cond);
-	STARPU_PTHREAD_MUTEX_UNLOCK_SCHED(curr_sched_mutex);
-	
-	/* all workers will block on this mutex anyway so 
-	   there's no need for their own mutex to be locked */
+	starpu_worker_relax_on();
 	STARPU_PTHREAD_MUTEX_LOCK(&data->policy_mutex);
+	starpu_worker_relax_off();
 
-	unsigned priolevel = taskq->max_prio - taskq->min_prio;
-	do
-	{
-		if (taskq->ntasks[priolevel] > 0)
-		{
-			for (task  = starpu_task_list_begin(&taskq->taskq[priolevel]);
-			     task != starpu_task_list_end(&taskq->taskq[priolevel]) && !chosen_task;
-			     task  = nexttask) 
-			{
-				unsigned nimpl;
-				nexttask = starpu_task_list_next(task);
-				if (starpu_worker_can_execute_task_first_impl(workerid, task, &nimpl))
-				{
-					/* there is some task that we can grab */
-					starpu_task_set_implementation(task, nimpl);
-					starpu_task_list_erase(&taskq->taskq[priolevel], task);
-					chosen_task = task;
-					taskq->ntasks[priolevel]--;
-					taskq->total_ntasks--;
-					break;
-				}
-				else
-					skipped = 1;
-			}
-		}
-	}
-	while (!chosen_task && priolevel-- > 0);
-
+	chosen_task = _starpu_prio_deque_pop_task_for_worker(taskq, workerid, &skipped);
 
 	if (!chosen_task && skipped)
 	{
@@ -267,21 +191,21 @@ static struct starpu_task *_starpu_priority_pop_task(unsigned sched_ctx_id)
 		struct starpu_worker_collection *workers = starpu_sched_ctx_get_worker_collection(sched_ctx_id);
 
 		struct starpu_sched_ctx_iterator it;
-
 		workers->init_iterator(workers, &it);
 		while(workers->has_next(workers, &it))
 		{
 			unsigned worker = workers->get_next(workers, &it);
+
 			if(worker != workerid)
 			{
 #ifdef STARPU_NON_BLOCKING_DRIVERS
 				starpu_bitmap_unset(data->waiters, worker);
 #else
-				starpu_wake_worker_locked(worker);
+				starpu_wake_worker_relax_light(worker);
 #endif
 			}
 		}
-	
+
 	}
 
 	if (!chosen_task)
@@ -289,9 +213,18 @@ static struct starpu_task *_starpu_priority_pop_task(unsigned sched_ctx_id)
 		starpu_bitmap_set(data->waiters, workerid);
 
 	STARPU_PTHREAD_MUTEX_UNLOCK(&data->policy_mutex);
+	if(chosen_task &&_starpu_get_nsched_ctxs() > 1)
+	{
+		starpu_worker_relax_on();
+		_starpu_sched_ctx_lock_write(sched_ctx_id);
+		starpu_worker_relax_off();
+		starpu_sched_ctx_list_task_counters_decrement_all_ctx_locked(chosen_task, sched_ctx_id);
 
-	/* leave the mutex how it was found before this */
-	STARPU_PTHREAD_MUTEX_LOCK_SCHED(curr_sched_mutex);
+		if (_starpu_sched_ctx_worker_is_master_for_child_ctx(sched_ctx_id, workerid, chosen_task))
+			chosen_task = NULL;
+
+		_starpu_sched_ctx_unlock_write(sched_ctx_id);
+	}
 
 	return chosen_task;
 }
@@ -302,9 +235,9 @@ static void eager_center_priority_add_workers(unsigned sched_ctx_id, int *worker
         for (i = 0; i < nworkers; i++)
         {
 		int workerid = workerids[i];
-		int curr_workerid = starpu_worker_get_id();
+		int curr_workerid = _starpu_worker_get_id();
 		if(workerid != curr_workerid)
-			starpu_wake_worker(workerid);
+			starpu_wake_worker_locked(workerid);
 
                 starpu_sched_ctx_worker_shares_tasks_lists(workerid, sched_ctx_id);
         }
@@ -322,5 +255,6 @@ struct starpu_sched_policy _starpu_sched_prio_policy =
 	.post_exec_hook = NULL,
 	.pop_every_task = NULL,
 	.policy_name = "prio",
-	.policy_description = "eager (with priorities)"
+	.policy_description = "eager (with priorities)",
+	.worker_type = STARPU_WORKER_LIST,
 };
