@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2014-2015,2017                           CNRS
- * Copyright (C) 2014-2015,2017                           Inria
- * Copyright (C) 2015                                     Université de Bordeaux
+ * Copyright (C) 2014,2015,2017,2019                      CNRS
+ * Copyright (C) 2014-2018                                Inria
+ * Copyright (C) 2015,2016                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -28,7 +28,7 @@
 
 /* ucontexts have been deprecated as of POSIX 1-2004
  * _XOPEN_SOURCE required at least on OS/X
- * 
+ *
  * TODO: add detection in configure.ac
  */
 #ifndef _XOPEN_SOURCE
@@ -36,6 +36,9 @@
 #endif
 #include <ucontext.h>
 
+extern starpu_pthread_key_t omp_thread_key;
+extern starpu_pthread_key_t omp_task_key;
+
 /*
  * Arbitrary limit on the number of nested parallel sections
  */
@@ -74,7 +77,7 @@ struct starpu_omp_place
 	int nb_numeric_places;
 };
 
-/* 
+/*
  * Internal Control Variables (ICVs) declared following
  * OpenMP 4.0.0 spec section 2.3.1
  */
@@ -96,6 +99,7 @@ struct starpu_omp_data_environment_icvs
 
 	/* program execution icvs */
 	int default_device_var;
+	int max_task_priority_var;
 };
 
 struct starpu_omp_device_icvs
@@ -143,6 +147,7 @@ struct starpu_omp_initial_icv_values
 	int place_partition_var;
 	int cancel_var;
 	int default_device_var;
+	int max_task_priority_var;
 
 	/* not a real ICV, but needed to store the contents of OMP_PLACES */
 	struct starpu_omp_place places;
@@ -193,7 +198,17 @@ enum starpu_omp_task_wait_on
 	starpu_omp_task_wait_on_barrier      = 1 << 2,
 	starpu_omp_task_wait_on_group        = 1 << 3,
 	starpu_omp_task_wait_on_critical     = 1 << 4,
-	starpu_omp_task_wait_on_condition    = 1 << 5
+	starpu_omp_task_wait_on_ordered      = 1 << 5,
+	starpu_omp_task_wait_on_lock         = 1 << 6,
+	starpu_omp_task_wait_on_nest_lock    = 1 << 7,
+};
+
+enum starpu_omp_task_flags
+{
+	STARPU_OMP_TASK_FLAGS_IMPLICIT	     = 1 << 0,
+	STARPU_OMP_TASK_FLAGS_UNDEFERRED     = 1 << 1,
+	STARPU_OMP_TASK_FLAGS_FINAL	     = 1 << 2,
+	STARPU_OMP_TASK_FLAGS_UNTIED	     = 1 << 3,
 };
 
 LIST_TYPE(starpu_omp_task,
@@ -202,10 +217,6 @@ LIST_TYPE(starpu_omp_task,
 	struct starpu_omp_thread *owner_thread;
 	struct starpu_omp_region *owner_region;
 	struct starpu_omp_region *nested_region;
-	int is_implicit;
-	int is_undeferred;
-	int is_final;
-	int is_untied;
 	int rank;
 	int child_task_count;
 	struct starpu_omp_task_group *task_group;
@@ -238,8 +249,9 @@ LIST_TYPE(starpu_omp_task,
 #endif
 
 	enum starpu_omp_task_state state;
+	enum starpu_omp_task_flags flags;
 
-	/* 
+	/*
 	 * context to store the processing state of the task
 	 * in case of blocking/recursive task operation
 	 */
@@ -257,6 +269,16 @@ LIST_TYPE(starpu_omp_task,
 	int stack_vg_id;
 
 	size_t stacksize;
+
+   /*
+    * taskloop attribute
+    * */
+   int is_loop;
+   unsigned long long nb_iterations;
+   unsigned long long grainsize;
+   unsigned long long chunk;
+   unsigned long long begin_i;
+   unsigned long long end_i;
 )
 
 LIST_TYPE(starpu_omp_thread,
@@ -369,24 +391,28 @@ struct starpu_omp_global
 	struct starpu_omp_thread *hash_workers;
 	struct _starpu_spinlock hash_workers_lock;
 	struct starpu_arbiter *default_arbiter;
-	int nb_starpu_cpu_workers;
+	unsigned nb_starpu_cpu_workers;
 	int *starpu_cpu_worker_ids;
+	int environment_valid;
 };
 
-/* 
+/*
  * internal global variables
  */
 extern struct starpu_omp_initial_icv_values *_starpu_omp_initial_icv_values;
 extern struct starpu_omp_global *_starpu_omp_global_state;
 extern double _starpu_omp_clock_ref;
 
-/* 
+/*
  * internal API
  */
 void _starpu_omp_environment_init(void);
 void _starpu_omp_environment_exit(void);
+int _starpu_omp_environment_check(void);
 struct starpu_omp_thread *_starpu_omp_get_thread(void);
+struct starpu_omp_region *_starpu_omp_get_region_at_level(int level);
 struct starpu_omp_task *_starpu_omp_get_task(void);
+int _starpu_omp_get_region_thread_num(const struct starpu_omp_region *const region);
 void _starpu_omp_dummy_init(void);
 void _starpu_omp_dummy_shutdown(void);
 #endif // STARPU_OPENMP
