@@ -2,7 +2,7 @@
  *
  * Copyright (C) 2011-2013                                Inria
  * Copyright (C) 2010-2013,2015,2017                      CNRS
- * Copyright (C) 2013-2014,2016                           Université de Bordeaux
+ * Copyright (C) 2011,2013-2016                           Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -33,14 +33,15 @@ static unsigned ntasks = 40000;
 #endif
 
 #ifdef STARPU_USE_CUDA
-extern void increment_cuda(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args);
+extern void increment_cuda(void *descr[], void *_args);
 #endif
 #ifdef STARPU_USE_OPENCL
 extern void increment_opencl(void *buffers[], void *args);
 #endif
 
-void increment_cpu(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void increment_cpu(void *descr[], void *arg)
 {
+	(void)arg;
 	unsigned *tokenptr = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
 	(*tokenptr)++;
 }
@@ -75,8 +76,9 @@ int increment_token(int synchronous)
 }
 
 static
-void callback(void *arg STARPU_ATTRIBUTE_UNUSED)
+void callback(void *arg)
 {
+	(void)arg;
         starpu_data_release(token_handle);
 }
 
@@ -115,7 +117,7 @@ int main(int argc, char **argv)
 		if (ret == -ENODEV) goto enodev;
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 
-                starpu_data_acquire_cb(token_handle, STARPU_R, callback, NULL);  // send
+                ret = starpu_data_acquire_cb(token_handle, STARPU_R, callback, NULL);  // send
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_data_acquire_cb");
 	}
 
