@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
  * Copyright (C) 2010-2013,2017                           CNRS
- * Copyright (C) 2010,2014,2016-2017                      Université de Bordeaux
+ * Copyright (C) 2010,2014,2016,2017                      Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,8 +22,10 @@
  * Try to use data_acquire_try in parallel with tasks
  */
 
-void func(STARPU_ATTRIBUTE_UNUSED void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void func(void *descr[], void *arg)
 {
+	(void)descr;
+	(void)arg;
 	starpu_sleep(0.01);
 }
 
@@ -41,8 +43,9 @@ unsigned token = 0;
 starpu_data_handle_t token_handle;
 
 static
-void callback(void *arg STARPU_ATTRIBUTE_UNUSED)
+void callback(void *arg)
 {
+	(void)arg;
         starpu_data_release(token_handle);
 }
 
