@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2011, 2016  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  CNRS
+ * Copyright (C) 2011-2013,2016,2017                      Inria
+ * Copyright (C) 2010-2013,2015,2017                      CNRS
+ * Copyright (C) 2010,2011,2013,2014,2016                 Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -28,14 +29,16 @@ static unsigned nloops = 10;
 static unsigned nloops = 1000;
 #endif
 
-void dummy_func(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg STARPU_ATTRIBUTE_UNUSED)
+void dummy_func(void *descr[], void *arg)
 {
+	(void)descr;
+	(void)arg;
 }
 
 static struct starpu_codelet cpu_codelet =
 {
         .cpu_funcs = {dummy_func},
-		.cpu_funcs_name = {"dummy_func"},
+	.cpu_funcs_name = {"dummy_func"},
         .model = NULL,
         .nbuffers = 1,
 	.modes = {STARPU_RW}
@@ -45,7 +48,6 @@ static struct starpu_codelet gpu_codelet =
 {
         .cuda_funcs = {dummy_func},
         .opencl_funcs = {dummy_func},
-		.cpu_funcs_name = {"dummy_func"},
         .model = NULL,
         .nbuffers = 1,
 	.modes = {STARPU_RW}
@@ -64,7 +66,7 @@ int main(int argc, char **argv)
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	if (starpu_worker_get_count_by_type(STARPU_CUDA_WORKER) == 0 && starpu_worker_get_count_by_type(STARPU_OPENCL_WORKER) == 0 &&
-		starpu_worker_get_count_by_type(STARPU_MIC_WORKER) == 0)
+		starpu_worker_get_count_by_type(STARPU_MIC_WORKER) == 0 && starpu_worker_get_count_by_type(STARPU_MPI_MS_WORKER) == 0)
 	{
 		FPRINTF(stderr, "This application requires a CUDA , OpenCL or MIC Worker\n");
 		starpu_shutdown();
