@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2012, 2015-2016  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  CNRS
+ * Copyright (C) 2009-2016                                Université de Bordeaux
+ * Copyright (C) 2010-2013,2015,2017                      CNRS
+ * Copyright (C) 2013                                     Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -48,6 +49,8 @@ static unsigned *v2;
 
 static void callback(void *arg)
 {
+	(void)arg;
+
 	unsigned res = STARPU_ATOMIC_ADD(&cnt, -1);
 	ANNOTATE_HAPPENS_BEFORE(&cnt);
 
@@ -61,18 +64,22 @@ static void callback(void *arg)
 	}
 }
 
-
-
-static void cuda_codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+static void cuda_codelet_null(void *descr[], void *_args)
 {
+	(void)descr;
+	(void)_args;
 }
 
-static void opencl_codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+static void opencl_codelet_null(void *descr[], void *_args)
 {
+	(void)descr;
+	(void)_args;
 }
 
-void cpu_codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void cpu_codelet_null(void *descr[], void *_args)
 {
+	(void)descr;
+	(void)_args;
 }
 
 static enum starpu_data_access_mode select_random_mode(void)
