@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012                                     Inria
+ * Copyright (C) 2012,2017                                Inria
  * Copyright (C) 2010-2016                                Université de Bordeaux
  * Copyright (C) 2010-2015,2017                           CNRS
  *
@@ -34,6 +34,8 @@ static starpu_data_handle_t handle;
 #ifdef STARPU_USE_CUDA
 static void neutral_cuda_kernel(void *descr[], void *arg)
 {
+	(void)arg;
+
 	unsigned *dst = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
 
 	/* This is a dummy technique of course */
@@ -46,6 +48,8 @@ static void neutral_cuda_kernel(void *descr[], void *arg)
 #ifdef STARPU_USE_OPENCL
 static void neutral_opencl_kernel(void *descr[], void *arg)
 {
+	(void)arg;
+
 	unsigned h_dst = 0;
 	cl_mem d_dst = (cl_mem)STARPU_VARIABLE_GET_PTR(descr[0]);
 
@@ -57,10 +61,10 @@ static void neutral_opencl_kernel(void *descr[], void *arg)
 }
 #endif
 
-
-
 void neutral_cpu_kernel(void *descr[], void *arg)
 {
+	(void)arg;
+
 	unsigned *dst = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
 	*dst = 0;
 }
@@ -85,8 +89,9 @@ static struct starpu_codelet neutral_cl =
 
 #ifdef STARPU_USE_OPENCL
 /* dummy OpenCL implementation */
-static void increment_opencl_kernel(void *descr[], void *cl_arg STARPU_ATTRIBUTE_UNUSED)
+static void increment_opencl_kernel(void *descr[], void *cl_arg)
 {
+	(void)cl_arg;
 	cl_mem d_token = (cl_mem)STARPU_VARIABLE_GET_PTR(descr[0]);
 	unsigned h_token;
 
@@ -104,6 +109,7 @@ static void increment_opencl_kernel(void *descr[], void *cl_arg STARPU_ATTRIBUTE
 #ifdef STARPU_USE_CUDA
 static void increment_cuda_kernel(void *descr[], void *arg)
 {
+	(void)arg;
 	unsigned *tokenptr = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
 	unsigned host_token;
 
@@ -120,6 +126,7 @@ static void increment_cuda_kernel(void *descr[], void *arg)
 
 void increment_cpu_kernel(void *descr[], void *arg)
 {
+	(void)arg;
 	unsigned *tokenptr = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
 	*tokenptr = *tokenptr + 1;
 }
@@ -138,7 +145,7 @@ static struct starpu_codelet increment_cl =
 	.modes = {STARPU_RW}
 };
 
-int main(int argc, char **argv)
+int main(void)
 {
 	unsigned *pvar = NULL;
 	int ret;
