@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011  Institut National de Recherche en Informatique et Automatique
+ * Copyright (C) 2011-2013                                Inria
+ * Copyright (C) 2011-2013,2015,2017,2019                 CNRS
+ * Copyright (C) 2012,2013                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -13,7 +15,7 @@
  *
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
-#include <config.h>
+
 #include <starpu.h>
 #include "../test_interfaces.h"
 #include "../../../helper.h"
@@ -24,7 +26,7 @@
 #define NNZ    (SIZE-1)
 
 #ifdef STARPU_USE_CPU
-static void test_csr_cpu_func(void *buffers[], void *args);
+void test_csr_cpu_func(void *buffers[], void *args);
 #endif /* !STARPU_USE_CPU */
 #ifdef STARPU_USE_CUDA
 extern void test_csr_cuda_func(void *buffers[], void *_args);
@@ -56,10 +58,13 @@ struct test_config csr_config =
 #endif
 #ifdef STARPU_USE_OPENCL
 	.opencl_func   = test_csr_opencl_func,
+#endif
+#ifdef STARPU_USE_MIC
+	.cpu_func_name = "test_csr_cpu_func", 
 #endif
 	.handle        = &csr_handle,
 	.dummy_handle  = &csr2_handle,
-	.copy_failed   = 0,
+	.copy_failed   = SUCCESS,
 	.name          = "csr_interface"
 };
 
@@ -87,7 +92,7 @@ register_data(void)
 	rowptr2[HEIGHT] = rowptr[HEIGHT];
 
 	starpu_csr_data_register(&csr_handle,
-				 0,
+				 STARPU_MAIN_RAM,
 				 NNZ,
 				 HEIGHT,
 				 (uintptr_t) nzval,
@@ -96,7 +101,7 @@ register_data(void)
 				 0,
 				 sizeof(nzval[0]));
 	starpu_csr_data_register(&csr2_handle,
-				 0,
+				 STARPU_MAIN_RAM,
 				 NNZ,
 				 HEIGHT,
 				 (uintptr_t) nzval2,
@@ -113,7 +118,7 @@ unregister_data(void)
 	starpu_data_unregister(csr2_handle);
 }
 
-static void
+void
 test_csr_cpu_func(void *buffers[], void *args)
 {
 	STARPU_SKIP_IF_VALGRIND;
@@ -126,44 +131,41 @@ test_csr_cpu_func(void *buffers[], void *args)
 	val = (int *) STARPU_CSR_GET_NZVAL(buffers[0]);
 	factor = *(int *) args;
 
-	for (i = 0; i < nnz; i++)
+	for (i = 0; i < (int)nnz; i++)
 	{
 		if (val[i] != (i+1) * factor)
 		{
-			csr_config.copy_failed = 1;
+			csr_config.copy_failed = FAILURE;
 			return;
 		}
 		val[i] *= -1;
 	}
 }
 
-int
-main(void)
+int main(int argc, char **argv)
 {
-	data_interface_test_summary *summary;
-	struct starpu_conf conf =
-	{
-		.ncpus   = -1,
-		.ncuda   = 2,
-		.nopencl = 1
-	};
+	struct data_interface_test_summary summary;
+	struct starpu_conf conf;
+	starpu_conf_init(&conf);
 
-	if (starpu_init(&conf) == -ENODEV || starpu_cpu_worker_get_count() == 0)
+	conf.ncuda = 2;
+	conf.nopencl = 1;
+	conf.nmic = -1;
+
+	if (starpu_initialize(&conf, &argc, &argv) == -ENODEV || starpu_cpu_worker_get_count() == 0)
 		goto enodev;
 
 	register_data();
 
-	summary = run_tests(&csr_config);
-	if (!summary)
-		exit(EXIT_FAILURE);
+	run_tests(&csr_config, &summary);
 
 	unregister_data();
 
 	starpu_shutdown();
 
-	data_interface_test_summary_print(stderr, summary);
+	data_interface_test_summary_print(stderr, &summary);
 
-	return data_interface_test_summary_success(summary);
+	return data_interface_test_summary_success(&summary);
 
 enodev:
 	return STARPU_TEST_SKIPPED;
