@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011  Institut National de Recherche en Informatique et Automatique
- * Copyright (C) 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2011,2012                                Inria
+ * Copyright (C) 2011,2012,2015-2017                      CNRS
+ * Copyright (C) 2012                                     Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,9 +16,7 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
-#include <config.h>
 #include <starpu.h>
-#include <starpu_opencl.h>
 #include "../../../helper.h"
 
 #define KERNEL_LOCATION "tests/datawizard/interfaces/multiformat/multiformat_conversion_codelets_kernel.cl"
@@ -38,7 +37,7 @@ void cpu_to_opencl_opencl_func(void *buffers[], void *args)
 	cl_mem src = (cl_mem) STARPU_MULTIFORMAT_GET_CPU_PTR(buffers[0]);
 	cl_mem dst = (cl_mem) STARPU_MULTIFORMAT_GET_OPENCL_PTR(buffers[0]);
 
-	id = starpu_worker_get_id();
+	id = starpu_worker_get_id_check();
 	devid = starpu_worker_get_devid(id);
 
 	ret = starpu_opencl_load_opencl_from_file(KERNEL_LOCATION,
