@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012  INRIA
+ * Copyright (C) 2012                                     Inria
+ * Copyright (C) 2012,2013,2017,2019                      CNRS
+ * Copyright (C) 2012,2013                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -58,10 +60,9 @@ unregister_data(void)
 	starpu_data_unregister(void2_handle);
 }
 
-int
-main(int argc, char **argv)
+int main(int argc, char **argv)
 {
-	data_interface_test_summary *summary;
+	struct data_interface_test_summary summary;
 	struct starpu_conf conf;
 	starpu_conf_init(&conf);
 	conf.ncuda = 2;
@@ -73,17 +74,15 @@ main(int argc, char **argv)
 
 	register_data();
 
-	summary = run_tests(&void_config);
-	if (!summary)
-		exit(EXIT_FAILURE);
+	run_tests(&void_config, &summary);
 
 	unregister_data();
 
 	starpu_shutdown();
 
-	data_interface_test_summary_print(stderr, summary);
+	data_interface_test_summary_print(stderr, &summary);
 
-	return data_interface_test_summary_success(summary);
+	return data_interface_test_summary_success(&summary);
 
 enodev:
 	return STARPU_TEST_SKIPPED;
