@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
- * Copyright (C) 2010-2016                                Université de Bordeaux
- * Copyright (C) 2011-2013,2015-2018                      CNRS
+ * Copyright (C) 2010-2016,2019                           Université de Bordeaux
+ * Copyright (C) 2012,2013,2016,2017                      Inria
+ * Copyright (C) 2011-2013,2015-2019                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -33,9 +33,12 @@ static starpu_data_handle_t variable_handle;
 static uintptr_t per_worker[STARPU_NMAXWORKERS];
 static starpu_data_handle_t per_worker_handle[STARPU_NMAXWORKERS];
 
+static unsigned ndone;
+
 /* Create per-worker handles */
-static void initialize_per_worker_handle(void *arg STARPU_ATTRIBUTE_UNUSED)
+static void initialize_per_worker_handle(void *arg)
 {
+	(void)arg;
 	int workerid = starpu_worker_get_id_check();
 
 	/* Allocate memory on the worker, and initialize it to 0 */
@@ -71,7 +74,11 @@ static void initialize_per_worker_handle(void *arg STARPU_ATTRIBUTE_UNUSED)
 			{
 				STARPU_CUDA_REPORT_ERROR(status);
 			}
-			cudaMemset((void *)per_worker[workerid], 0, sizeof(variable));
+			status = cudaMemsetAsync((void *)per_worker[workerid], 0, sizeof(variable), starpu_cuda_get_local_stream());
+			if (!status)
+				status = cudaStreamSynchronize(starpu_cuda_get_local_stream());
+			if (status)
+				STARPU_CUDA_REPORT_ERROR(status);
 			break;
 		}
 #endif
@@ -79,6 +86,7 @@ static void initialize_per_worker_handle(void *arg STARPU_ATTRIBUTE_UNUSED)
 			STARPU_ABORT();
 			break;
 	}
+	FPRINTF(stderr, "worker %d got data %lx\n", workerid, (unsigned long) per_worker[workerid]);
 
 	STARPU_ASSERT(per_worker[workerid]);
 }
@@ -87,8 +95,9 @@ static void initialize_per_worker_handle(void *arg STARPU_ATTRIBUTE_UNUSED)
  *	Implement reduction method
  */
 
-void cpu_redux_func(void *descr[], void *cl_arg STARPU_ATTRIBUTE_UNUSED)
+void cpu_redux_func(void *descr[], void *cl_arg)
 {
+	(void)cl_arg;
 	unsigned *a = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
 	unsigned *b = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[1]);
 
@@ -109,32 +118,56 @@ static struct starpu_codelet reduction_codelet =
  *	Use per-worker local copy
  */
 
-void cpu_func_incr(void *descr[], void *cl_arg STARPU_ATTRIBUTE_UNUSED)
+void cpu_func_incr(void *descr[], void *cl_arg)
 {
+	(void)cl_arg;
 	unsigned *val = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
 	*val = *val + 1;
+	STARPU_ATOMIC_ADD(&ndone, 1);
 }
 
 #ifdef STARPU_USE_CUDA
 /* dummy CUDA implementation */
-static void cuda_func_incr(void *descr[], void *cl_arg STARPU_ATTRIBUTE_UNUSED)
+static void cuda_func_incr(void *descr[], void *cl_arg)
 {
+	(void)cl_arg;
 	STARPU_SKIP_IF_VALGRIND;
 
 	unsigned *val = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
 
-	unsigned h_val;
-	cudaMemcpyAsync(&h_val, val, sizeof(unsigned), cudaMemcpyDeviceToHost, starpu_cuda_get_local_stream());
-	cudaStreamSynchronize(starpu_cuda_get_local_stream());
+	unsigned h_val, h_val2;
+	cudaError_t status;
+	status = cudaMemcpyAsync(&h_val, val, sizeof(unsigned), cudaMemcpyDeviceToHost, starpu_cuda_get_local_stream());
+	if (status)
+		STARPU_CUDA_REPORT_ERROR(status);
+	status = cudaStreamSynchronize(starpu_cuda_get_local_stream());
+	if (status)
+		STARPU_CUDA_REPORT_ERROR(status);
 	h_val++;
-	cudaMemcpyAsync(val, &h_val, sizeof(unsigned), cudaMemcpyHostToDevice, starpu_cuda_get_local_stream());
+	status = cudaMemcpyAsync(val, &h_val, sizeof(unsigned), cudaMemcpyHostToDevice, starpu_cuda_get_local_stream());
+	if (status)
+		STARPU_CUDA_REPORT_ERROR(status);
+	status = cudaStreamSynchronize(starpu_cuda_get_local_stream());
+	if (status)
+		STARPU_CUDA_REPORT_ERROR(status);
+
+	status = cudaMemcpyAsync(&h_val2, val, sizeof(unsigned), cudaMemcpyDeviceToHost, starpu_cuda_get_local_stream());
+	if (status)
+		STARPU_CUDA_REPORT_ERROR(status);
+	status = cudaStreamSynchronize(starpu_cuda_get_local_stream());
+	if (status)
+		STARPU_CUDA_REPORT_ERROR(status);
+	STARPU_ASSERT_MSG(h_val2 == h_val, "%lx should be %u, not %u, I have just written it ?!\n", (unsigned long)(uintptr_t) val, h_val, h_val2);
+
+	STARPU_ATOMIC_ADD(&ndone, 1);
 }
 #endif
 
 #ifdef STARPU_USE_OPENCL
 /* dummy OpenCL implementation */
-static void opencl_func_incr(void *descr[], void *cl_arg STARPU_ATTRIBUTE_UNUSED)
+static void opencl_func_incr(void *descr[], void *cl_arg)
 {
+	(void)cl_arg;
 	STARPU_SKIP_IF_VALGRIND;
 
 	cl_mem d_val = (cl_mem)STARPU_VARIABLE_GET_PTR(descr[0]);
@@ -148,6 +181,7 @@ static void opencl_func_incr(void *descr[], void *cl_arg STARPU_ATTRIBUTE_UNUSED
 	h_val++;
 	clEnqueueWriteBuffer(queue, d_val, CL_FALSE, 0, sizeof(unsigned), (void *)&h_val, 0, NULL, NULL);
 	clFinish(queue);
+	STARPU_ATOMIC_ADD(&ndone, 1);
 }
 #endif
 
@@ -175,7 +209,7 @@ int main(int argc, char **argv)
 
 	starpu_conf_init(&conf);
 	conf.nmic = 0;
-	conf.nscc = 0;
+	conf.nmpi_ms = 0;
 
 	variable = INIT_VALUE;
 
@@ -265,6 +299,7 @@ int main(int argc, char **argv)
 	else
 	{
 		FPRINTF(stderr, "%u != %d + %d\n", variable, INIT_VALUE, NTASKS);
+		FPRINTF(stderr, "ndone: %u\n", ndone);
 		ret = EXIT_FAILURE;
 	}
 	STARPU_RETURN(ret);
