@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
  * Copyright (C) 2010-2013,2015,2017                      CNRS
- * Copyright (C) 2010-2012,2014,2017                      Université de Bordeaux
+ * Copyright (C) 2010-2012,2014                           Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,17 +15,20 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
+#ifndef _STARPU_TESTS_MPI_LIKE_H
+#define _STARPU_TESTS_MPI_LIKE_H
+
 #include <starpu.h>
 
 #ifdef STARPU_USE_CUDA
 #ifdef __CUDACC__
 extern "C"
 #endif
-void cuda_codelet_unsigned_inc(void *descr[], STARPU_ATTRIBUTE_UNUSED void *cl_arg);
+void cuda_codelet_unsigned_inc(void *descr[], void *cl_arg);
 #endif
 
 #ifdef STARPU_USE_OPENCL
 void opencl_codelet_unsigned_inc(void *buffers[], void *args);
 #endif
 
-
+#endif // _STARPU_TESTS_MPI_LIKE_H
