@@ -25,7 +25,7 @@
 
 #if !defined(STARPU_HAVE_UNSETENV) || !defined(STARPU_USE_CPU)
 #warning unsetenv is not defined or no cpu are available. Skipping test
-int main(int argc, char **argv)
+int main(void)
 {
 	return STARPU_TEST_SKIPPED;
 }
@@ -33,6 +33,8 @@ int main(int argc, char **argv)
 
 void dummy_func(void *descr[], void *arg)
 {
+	(void)descr;
+	(void)arg;
 }
 
 static struct starpu_codelet gpu_only_cl =
@@ -43,7 +45,7 @@ static struct starpu_codelet gpu_only_cl =
 	.nbuffers = 0
 };
 
-int main(int argc, char **argv)
+int main(void)
 {
 	int ret;
 
