@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
- * Copyright (C) 2012-2013,2015,2017                      CNRS
+ * Copyright (C) 2012,2013,2016                           Inria
+ * Copyright (C) 2012,2013,2015,2017                      CNRS
  * Copyright (C) 2013-2015,2017                           Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
 
 #if !defined(STARPU_HAVE_UNSETENV)
 #warning unsetenv is not defined. Skipping test
-int main(int argc, char **argv)
+int main(void)
 {
 	return STARPU_TEST_SKIPPED;
 }
@@ -31,7 +31,7 @@ int main(int argc, char **argv)
 
 /*
  * Users can directly control drivers by using the starpu_driver* functions.
- * 
+ *
  * This test makes sure that the starpu_driver_run function works for CPU, CUDA
  * and OpenCL drivers, and that the starpu_drivers_request_termination function
  * correctly shuts down all drivers.
@@ -43,8 +43,7 @@ int main(int argc, char **argv)
  */
 
 #if defined(STARPU_USE_CPU) || defined(STARPU_USE_CUDA) || defined(STARPU_USE_OPENCL)
-static void
-dummy(void *buffers[], void *args)
+static void dummy(void *buffers[], void *args)
 {
 	(void) buffers;
 	(*(int *)args)++;
@@ -59,8 +58,7 @@ static struct starpu_codelet cl =
 	.nbuffers     = 0
 };
 
-static void *
-run_driver(void *arg)
+static void *run_driver(void *arg)
 {
 	struct starpu_driver *d = (struct starpu_driver *) arg;
 	int ret = starpu_driver_run(d);
@@ -70,8 +68,7 @@ run_driver(void *arg)
 #endif /* STARPU_USE_CPU || STARPU_USE_CUDA || STARPU_USE_OPENCL */
 
 #ifdef STARPU_USE_CPU
-static int
-test_cpu(void)
+static int test_cpu(void)
 {
 	int ret, var = 0;
 	static starpu_pthread_t driver_thread;
@@ -86,6 +83,8 @@ test_cpu(void)
 	conf.n_not_launched_drivers = 1;
 	conf.not_launched_drivers = &d;
 	conf.ncpus = 1;
+	conf.ncuda = 0;
+	conf.nopencl = 0;
 	ret = starpu_init(&conf);
 	if (ret == -ENODEV || starpu_cpu_worker_get_count() == 0)
 	{
@@ -130,8 +129,7 @@ out2:
 #endif /* STARPU_USE_CPU */
 
 #ifdef STARPU_USE_CUDA
-static int
-test_cuda(void)
+static int test_cuda(void)
 {
 	int ret, var = 0;
 	static starpu_pthread_t driver_thread;
@@ -145,7 +143,9 @@ test_cuda(void)
 	starpu_conf_init(&conf);
 	conf.n_not_launched_drivers = 1;
 	conf.not_launched_drivers = &d;
+	conf.ncpus = 0;
 	conf.ncuda = 1;
+	conf.nopencl = 0;
 	ret = starpu_init(&conf);
 	if (ret == -ENODEV || starpu_cuda_worker_get_count() == 0)
 	{
@@ -154,13 +154,17 @@ test_cuda(void)
 			starpu_shutdown();
 		return STARPU_TEST_SKIPPED;
 	}
+	if (starpu_cuda_worker_get_count() > 1)
+	{
+		FPRINTF(stderr, "WARNING: More than one worker, this is not supported by this test\n");
+		if (ret == 0)
+			starpu_shutdown();
+		return STARPU_TEST_SKIPPED;
+	}
 
 	ret = starpu_pthread_create(&driver_thread, NULL, run_driver, &d);
 	if (ret == -1)
-	{
-		ret = 1;
 		goto out;
-	}
 
 	struct starpu_task *task;
 	task = starpu_task_create();
@@ -173,7 +177,6 @@ test_cuda(void)
 	if (ret == -ENODEV)
 	{
 		FPRINTF(stderr, "WARNING: No worker can execute this task\n");
-		ret = STARPU_TEST_SKIPPED;
 		goto out;
 	}
 
@@ -190,8 +193,7 @@ out:
 #endif /* STARPU_USE_CUDA */
 
 #ifdef STARPU_USE_OPENCL
-static int
-test_opencl(void)
+static int test_opencl(void)
 {
 	int ret, var = 0;
 	static starpu_pthread_t driver_thread;
@@ -230,6 +232,7 @@ test_opencl(void)
 	starpu_conf_init(&conf);
 	conf.n_not_launched_drivers = 1;
 	conf.not_launched_drivers = &d;
+	conf.ncpus = 1;
 	conf.ncuda = 0;
 	conf.nopencl = 1;
 	ret = starpu_init(&conf);
@@ -243,10 +246,7 @@ test_opencl(void)
 
 	ret = starpu_pthread_create(&driver_thread, NULL, run_driver, &d);
 	if (ret == -1)
-	{
-		ret = 1;
 		goto out;
-	}
 
 	struct starpu_task *task;
 	task = starpu_task_create();
@@ -259,7 +259,6 @@ test_opencl(void)
 	if (ret == -ENODEV)
 	{
 		FPRINTF(stderr, "WARNING: No worker can execute the task\n");
-		ret = STARPU_TEST_SKIPPED;
 		goto out;
 	}
 
@@ -275,8 +274,7 @@ out:
 }
 #endif /* STARPU_USE_OPENCL */
 
-int
-main(void)
+int main(void)
 {
 	int ret = STARPU_TEST_SKIPPED;
 
