@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2014,2017                           CNRS
- * Copyright (C) 2016-2017                                Université de Bordeaux
+ * Copyright (C) 2011-2015,2017                           CNRS
+ * Copyright (C) 2015,2016                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -26,8 +26,9 @@
 
 #define NPARAMS 15
 
-void func_cpu(void *descr[], void *_args STARPU_ATTRIBUTE_UNUSED)
+void func_cpu(void *descr[], void *_args)
 {
+	(void)_args;
 	struct starpu_task *task = starpu_task_get_current();
 	int num = STARPU_TASK_GET_NBUFFERS(task);
 	int i;
@@ -42,7 +43,8 @@ void func_cpu(void *descr[], void *_args STARPU_ATTRIBUTE_UNUSED)
 		}
 }
 
-struct starpu_codelet codelet =
+/* We will fill this one with dyn_modes */
+struct starpu_codelet codelet_dyn =
 {
 	.cpu_funcs = {func_cpu},
 	/* starpu_task_get_current() doesn't work on MIC */
@@ -50,6 +52,35 @@ struct starpu_codelet codelet =
 	.nbuffers = NPARAMS,
 };
 
+/* This will warn out at compilation time when maxbuffers is less than NPARAMS.
+ * That is on purpose: we here check that we still behave correctly in that case.
+ * We are just not able to check the parameter access modes.  */
+struct starpu_codelet codelet_toomany =
+{
+	.cpu_funcs = {func_cpu},
+	/* starpu_task_get_current() doesn't work on MIC */
+	/* .cpu_funcs_name = {"func_cpu"}, */
+	.nbuffers = NPARAMS,
+	.modes =
+	{
+		STARPU_R,
+		STARPU_R,
+		STARPU_RW|STARPU_COMMUTE,
+		STARPU_RW|STARPU_COMMUTE,
+		STARPU_R,
+		STARPU_RW,
+		STARPU_R,
+		STARPU_RW|STARPU_COMMUTE,
+		STARPU_R,
+		STARPU_RW|STARPU_COMMUTE,
+		STARPU_R,
+		STARPU_R,
+		STARPU_SCRATCH,
+		STARPU_SCRATCH,
+		STARPU_SCRATCH,
+	}
+};
+
 struct starpu_codelet codelet_variable =
 {
 	.cpu_funcs = {func_cpu},
@@ -58,7 +89,7 @@ struct starpu_codelet codelet_variable =
 	.nbuffers = STARPU_VARIABLE_NBUFFERS,
 };
 
-int main(int argc, char **argv)
+int main(void)
 {
         int *x;
         int i, ret, loop;
@@ -68,6 +99,8 @@ int main(int argc, char **argv)
 #else
 	int nloops = 16;
 #endif
+	int val_int = 42;
+	double val_double = 42.;
         starpu_data_handle_t *data_handles;
 	int *expected;
 
@@ -75,22 +108,22 @@ int main(int argc, char **argv)
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
-	codelet.dyn_modes = malloc(NPARAMS * sizeof(*(codelet.modes)));
-	codelet.dyn_modes[0] = STARPU_R,
-	codelet.dyn_modes[1] = STARPU_R,
-	codelet.dyn_modes[2] = STARPU_RW|STARPU_COMMUTE,
-	codelet.dyn_modes[3] = STARPU_RW|STARPU_COMMUTE,
-	codelet.dyn_modes[4] = STARPU_R,
-	codelet.dyn_modes[5] = STARPU_RW,
-	codelet.dyn_modes[6] = STARPU_R,
-	codelet.dyn_modes[7] = STARPU_RW|STARPU_COMMUTE,
-	codelet.dyn_modes[8] = STARPU_R,
-	codelet.dyn_modes[9] = STARPU_RW|STARPU_COMMUTE,
-	codelet.dyn_modes[10] = STARPU_R,
-	codelet.dyn_modes[11] = STARPU_R,
-	codelet.dyn_modes[12] = STARPU_SCRATCH,
-	codelet.dyn_modes[13] = STARPU_SCRATCH,
-	codelet.dyn_modes[14] = STARPU_SCRATCH,
+	codelet_dyn.dyn_modes = malloc(NPARAMS * sizeof(*(codelet_dyn.modes)));
+	codelet_dyn.dyn_modes[0] = STARPU_R,
+	codelet_dyn.dyn_modes[1] = STARPU_R,
+	codelet_dyn.dyn_modes[2] = STARPU_RW|STARPU_COMMUTE,
+	codelet_dyn.dyn_modes[3] = STARPU_RW|STARPU_COMMUTE,
+	codelet_dyn.dyn_modes[4] = STARPU_R,
+	codelet_dyn.dyn_modes[5] = STARPU_RW,
+	codelet_dyn.dyn_modes[6] = STARPU_R,
+	codelet_dyn.dyn_modes[7] = STARPU_RW|STARPU_COMMUTE,
+	codelet_dyn.dyn_modes[8] = STARPU_R,
+	codelet_dyn.dyn_modes[9] = STARPU_RW|STARPU_COMMUTE,
+	codelet_dyn.dyn_modes[10] = STARPU_R,
+	codelet_dyn.dyn_modes[11] = STARPU_R,
+	codelet_dyn.dyn_modes[12] = STARPU_SCRATCH,
+	codelet_dyn.dyn_modes[13] = STARPU_SCRATCH,
+	codelet_dyn.dyn_modes[14] = STARPU_SCRATCH,
 
 	x = calloc(NPARAMS, sizeof(*x));
 	data_handles = malloc(NPARAMS * sizeof(*data_handles));
@@ -102,10 +135,36 @@ int main(int argc, char **argv)
 	{
 		for (i = 0; i < NPARAMS; i++)
 		{
-			if (codelet.dyn_modes[i] & STARPU_W)
+			if (codelet_dyn.dyn_modes[i] & STARPU_W)
 				expected[i]++;
 		}
-		ret = starpu_task_insert(&codelet,
+		ret = starpu_task_insert(&codelet_dyn,
+					 STARPU_R, data_handles[0],
+					 STARPU_R, data_handles[1],
+					 STARPU_RW|STARPU_COMMUTE, data_handles[2],
+					 STARPU_RW|STARPU_COMMUTE, data_handles[3],
+					 STARPU_R, data_handles[4],
+					 STARPU_RW, data_handles[5],
+					 STARPU_R, data_handles[6],
+					 STARPU_RW|STARPU_COMMUTE, data_handles[7],
+					 STARPU_R, data_handles[8],
+					 STARPU_RW|STARPU_COMMUTE, data_handles[9],
+					 STARPU_R, data_handles[10],
+					 STARPU_R, data_handles[11],
+					 STARPU_SCRATCH, data_handles[12],
+					 STARPU_SCRATCH, data_handles[13],
+					 STARPU_SCRATCH, data_handles[14],
+					 0);
+		if (ret == -ENODEV) goto enodev;
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
+
+		/* Same, but using the toomany codelet */
+		for (i = 0; i < NPARAMS; i++)
+		{
+			if (codelet_dyn.dyn_modes[i] & STARPU_W)
+				expected[i]++;
+		}
+		ret = starpu_task_insert(&codelet_toomany,
 					 STARPU_R, data_handles[0],
 					 STARPU_R, data_handles[1],
 					 STARPU_RW|STARPU_COMMUTE, data_handles[2],
@@ -128,7 +187,7 @@ int main(int argc, char **argv)
 		/* Same, but using the variable codelet */
 		for (i = 0; i < NPARAMS; i++)
 		{
-			if (codelet.dyn_modes[i] & STARPU_W)
+			if (codelet_dyn.dyn_modes[i] & STARPU_W)
 				expected[i]++;
 		}
 		ret = starpu_task_insert(&codelet_variable,
@@ -161,7 +220,7 @@ enodev:
 
 	starpu_shutdown();
 	free(data_handles);
-	free(codelet.dyn_modes);
+	free(codelet_dyn.dyn_modes);
 
 	if (ret == -ENODEV)
 	{
