@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2012-2016  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  CNRS
+ * Copyright (C) 2012,2013                                Inria
+ * Copyright (C) 2010-2015,2017                           CNRS
+ * Copyright (C) 2010,2012-2016,2018                      Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -54,8 +55,9 @@ static starpu_pthread_mutex_t mutex = STARPU_PTHREAD_MUTEX_INITIALIZER;
 
 extern void cuda_host_increment(void *descr[], void *_args);
 
-void cpu_increment(void *descr[], void *arg STARPU_ATTRIBUTE_UNUSED)
+void cpu_increment(void *descr[], void *arg)
 {
+	(void)arg;
 	unsigned *var = (unsigned *)STARPU_VARIABLE_GET_PTR(descr[0]);
 	(*var)++;
 }
@@ -75,8 +77,9 @@ static struct starpu_codelet dummy_codelet =
 	.nbuffers = 1
 };
 
-static void callback_task_D(void *arg STARPU_ATTRIBUTE_UNUSED)
+static void callback_task_D(void *arg)
 {
+	(void)arg;
 	STARPU_PTHREAD_MUTEX_LOCK(&mutex);
 	loop_cnt++;
 
@@ -135,12 +138,10 @@ int main(int argc, char **argv)
 	taskD.callback_func = callback_task_D;
 	taskD.handles[0] = check_data;
 
-	struct starpu_task *depsBC_array[1] = {&taskA};
-	starpu_task_declare_deps_array(&taskB, 1, depsBC_array);
-	starpu_task_declare_deps_array(&taskC, 1, depsBC_array);
+	starpu_task_declare_deps(&taskB, 1, &taskA);
+	starpu_task_declare_deps(&taskC, 1, &taskA);
 
-	struct starpu_task *depsD_array[2] = {&taskB, &taskC};
-	starpu_task_declare_deps_array(&taskD, 2, depsD_array);
+	starpu_task_declare_deps(&taskD, 2, &taskB, &taskC);
 
 	ret = starpu_task_submit(&taskA); if (ret == -ENODEV) goto enodev; STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 	ret = starpu_task_submit(&taskB); if (ret == -ENODEV) goto enodev; STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
