@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2014-2015,2017                           CNRS
- * Copyright (C) 2014                                     Inria
+ * Copyright (C) 2014,2015,2017,2019                      CNRS
+ * Copyright (C) 2014,2016                                Inria
+ * Copyright (C) 2017                                     Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,8 +21,12 @@
 #include "../helper.h"
 #include <stdio.h>
 
+/*
+ * Check the OpenMP single wait support.
+ */
+
 #if !defined(STARPU_OPENMP)
-int main(int argc, char **argv)
+int main(void)
 {
 	return STARPU_TEST_SKIPPED;
 }
@@ -30,6 +35,7 @@ __attribute__((constructor))
 static void omp_constructor(void)
 {
 	int ret = starpu_omp_init();
+	if (ret == -EINVAL) exit(STARPU_TEST_SKIPPED);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_omp_init");
 }
 
@@ -66,12 +72,16 @@ void parallel_region_f(void *buffers[], void *args)
 }
 
 int
-main (int argc, char *argv[])
+main (void)
 {
 	struct starpu_omp_parallel_region_attr attr;
 	pthread_t tid;
 	tid = pthread_self();
 	memset(&attr, 0, sizeof(attr));
+#ifdef STARPU_SIMGRID
+	attr.cl.model        = &starpu_perfmodel_nop;
+#endif
+	attr.cl.flags        = STARPU_CODELET_SIMGRID_EXECUTE;
 	attr.cl.cpu_funcs[0] = parallel_region_f;
 	attr.cl.where        = STARPU_CPU;
 	attr.if_clause       = 1;
