@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
  * Copyright (C) 2010-2016                                Université de Bordeaux
+ * Copyright (C) 2012,2013                                Inria
  * Copyright (C) 2010-2013,2015,2017                      CNRS
  * Copyright (C) 2013                                     Thibaut Lambert
  *
@@ -33,8 +33,11 @@
 #endif
 #define VECTORSIZE	1024
 
-void codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void codelet_null(void *descr[], void *_args)
 {
+	(void)descr;
+	(void)_args;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	int worker_size = starpu_combined_worker_get_size();
@@ -60,7 +63,7 @@ static struct starpu_codelet cl =
 };
 
 
-int main(int argc, char **argv)
+int main(void)
 {
 	starpu_data_handle_t v_handle;
 	unsigned *v;
