@@ -1,7 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2012  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010-2016,2019                           Université de Bordeaux
+ * Copyright (C) 2012,2013                                Inria
+ * Copyright (C) 2010-2013,2015,2017                      CNRS
+ * Copyright (C) 2013                                     Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,17 +17,27 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
-#include <config.h>
 #include <starpu.h>
 #include <limits.h>
 #include <unistd.h>
 #include "../helper.h"
 
+/*
+ * Submit dumb parallel spmd tasks
+ */
+
+#ifndef STARPU_QUICK_CHECK
 #define N	1000
+#else
+#define N	100
+#endif
 #define VECTORSIZE	1024
 
-static void codelet_null(void *descr[], __attribute__ ((unused)) void *_args)
+void codelet_null(void *descr[], void *_args)
 {
+	(void)descr;
+	(void)_args;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	int worker_size = starpu_combined_worker_get_size();
@@ -48,48 +60,44 @@ struct starpu_perfmodel model =
 
 static struct starpu_codelet cl =
 {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
 	.type = STARPU_SPMD,
 	.max_parallelism = INT_MAX,
-	.cpu_funcs = {codelet_null, NULL},
-	.cuda_funcs = {codelet_null, NULL},
-        .opencl_funcs = {codelet_null, NULL},
+	.cpu_funcs = {codelet_null},
+	.cpu_funcs_name = {"codelet_null"},
+	.cuda_funcs = {codelet_null},
+        .opencl_funcs = {codelet_null},
 	.model = &model,
 	.nbuffers = 1,
 	.modes = {STARPU_R}
 };
 
 
-int main(int argc, char **argv)
+int main(void)
 {
 	int ret;
 	starpu_data_handle_t v_handle;
 	unsigned *v;
 
-        struct starpu_conf conf =
-	{
-                .sched_policy_name = "pheft",
-                .ncpus = -1,
-                .ncuda = -1,
-                .nopencl = -1,
-                .nspus = -1,
-                .use_explicit_workers_bindid = 0,
-                .use_explicit_workers_cuda_gpuid = 0,
-                .use_explicit_workers_opencl_gpuid = 0,
-                .calibrate = 1
-        };
+        struct starpu_conf conf;
+	starpu_conf_init(&conf);
+	conf.sched_policy_name = "pheft";
+	conf.calibrate = 1;
 
 	ret = starpu_init(&conf);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	starpu_malloc((void **)&v, VECTORSIZE*sizeof(unsigned));
-	starpu_vector_data_register(&v_handle, 0, (uintptr_t)v, VECTORSIZE, sizeof(unsigned));
+	starpu_vector_data_register(&v_handle, STARPU_MAIN_RAM, (uintptr_t)v, VECTORSIZE, sizeof(unsigned));
 
 	unsigned nworker = starpu_worker_get_count() + starpu_combined_worker_get_count();
 
-	unsigned iter, worker;
-	for (iter = 0; iter < N; iter++)
+	unsigned iter, worker, n;
+	n = N;
+	if (STARPU_RUNNING_ON_VALGRIND) {
+		n /= 300;
+	}
+	for (iter = 0; iter < n; iter++)
 	{
 		for (worker = 0; worker < nworker; worker++)
 		{
@@ -99,7 +107,7 @@ int main(int argc, char **argv)
 
 			task->handles[0] = v_handle;
 
-			int ret = starpu_task_submit(task);
+			ret = starpu_task_submit(task);
 			if (ret == -ENODEV) goto enodev;
 			STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 		}
