@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2012, 2014-2016  Université de Bordeaux
- * Copyright (C) 2012, 2013  CNRS
- * Copyright (C) 2012 INRIA
+ * Copyright (C) 2012                                     Inria
+ * Copyright (C) 2011-2016                                Université de Bordeaux
+ * Copyright (C) 2011-2013,2015,2017                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,7 +16,6 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
-#include <config.h>
 #include <starpu.h>
 #include "../helper.h"
 
@@ -34,6 +33,7 @@
 #ifdef STARPU_USE_CUDA
 static void memset_cuda(void *descr[], void *arg)
 {
+	(void)arg;
 	STARPU_SKIP_IF_VALGRIND;
 
 	int *ptr = (int *)STARPU_VECTOR_GET_PTR(descr[0]);
@@ -45,6 +45,7 @@ static void memset_cuda(void *descr[], void *arg)
 
 void memset_cpu(void *descr[], void *arg)
 {
+	(void)arg;
 	STARPU_SKIP_IF_VALGRIND;
 
 	int *ptr = (int *)STARPU_VECTOR_GET_PTR(descr[0]);
