@@ -1,7 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
+ * Copyright (C) 2014,2016,2017                           Inria
  * Copyright (C) 2012-2017                                CNRS
  * Copyright (C) 2014-2016                                Université de Bordeaux
+ * Copyright (C) 2013                                     Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -25,6 +27,8 @@
 
 void func(void *descr[], void *arg)
 {
+	(void)descr;
+	(void)arg;
 }
 
 static struct starpu_perfmodel rb_model =
@@ -105,6 +109,7 @@ static int submit(struct starpu_codelet *codelet, struct starpu_perfmodel *model
 
 	// We need to call starpu_init again to initialise values used by perfmodels
 	ret = starpu_init(NULL);
+	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 	char path[256];
@@ -151,7 +156,7 @@ static int submit(struct starpu_codelet *codelet, struct starpu_perfmodel *model
 	}
 }
 
-int main(int argc, char **argv)
+int main(void)
 {
 	int ret;
 
