@@ -1,8 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012                                     Inria
+ * Copyright (C) 2012,2014,2017                           Inria
  * Copyright (C) 2012-2017                                CNRS
- * Copyright (C) 2013-2014,2016                           Université de Bordeaux
+ * Copyright (C) 2013,2014,2016,2019                      Université de Bordeaux
+ * Copyright (C) 2013                                     Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -59,7 +60,7 @@ cpu_task_gpu(struct starpu_task *task,
 	(void) arch;
 	(void) nimpl;
 
-	return 1000.0;
+	return 10000000.0;
 }
 
 static double
@@ -71,7 +72,7 @@ gpu_task_cpu(struct starpu_task *task,
 	(void) arch;
 	(void) nimpl;
 
-	return 1000.0;
+	return 10000000.0;
 }
 
 static double
@@ -252,8 +253,7 @@ static struct starpu_sched_policy *policies[] =
 	//&_starpu_sched_peager_policy
 };
 
-int
-main(void)
+int main(void)
 {
 #ifndef STARPU_HAVE_SETENV
 /* XXX: is this macro used by all the schedulers we are interested in ? */
@@ -262,9 +262,8 @@ main(void)
 #else
 	setenv("STARPU_SCHED_BETA", "0", 1);
 
-#ifdef STARPU_HAVE_UNSETENV
-	unsetenv("STARPU_SCHED");
-#endif
+	char *sched = getenv("STARPU_SCHED");
+
 	if (starpu_get_env_number_default("STARPU_NWORKER_PER_CUDA", 1) != 1)
 		return STARPU_TEST_SKIPPED;
 
@@ -273,6 +272,11 @@ main(void)
 	for (i = 0; i < n_policies; ++i)
 	{
 		struct starpu_sched_policy *policy = policies[i];
+
+		if (sched && strcmp(sched, policy->policy_name))
+			/* Testing another specific scheduler, no need to run this */
+			continue;
+
 		FPRINTF(stdout, "Running with policy %s.\n",
 			policy->policy_name);
 		int ret;
