@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2014, 2016  Universite de Bordeaux
- * Copyright (C) 2012-2015, 2017  CNRS
+ * Copyright (C) 2012-2015,2017,2019                      CNRS
+ * Copyright (C) 2011-2014,2016                           Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,12 +15,12 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
-#include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
 #include <errno.h>
 #include <string.h>
+#include <common/config.h>
 #include <common/uthash.h>
 #include <common/utils.h>
 #include <starpu.h>
@@ -47,7 +47,7 @@ int main(int argc, char *argv[])
 	uint32_t footprint = 0;
 	int already_there = 0;
 	char *model_name = NULL;
-	struct model *model, *tmp;
+	struct model *model, *tmp=NULL;
 	int ret;
 
 	if (argc >= 2)
